<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * "Affiliate Start ID" add-on class.
 * 
 */
class SliceWP_Add_On_Affiliate_Start_ID extends SliceWP_Add_On {

	/**
	 * Constructor.
	 * 
	 */
	public function __construct() {

		// Slug.
		$this->slug = 'affiliate_start_id';

		// Name.
		$this->name = 'Affiliate Start ID';

		// Description.
		$this->description = 'Enables you to set a custom starting ID for your affiliates.';

		// Icon URL.
		$this->icon_url = $this->get_dir_url() . 'assets/img/add-on-icon-affiliate-start-id.png';

		// Documentation URL.
		$this->documentation_url = 'https://slicewp.com/products/affiliate-start-id/';

		// Settings URL.
		$this->settings_url = add_query_arg( array( 'page' => 'slicewp-settings', 'tab' => 'general' ), admin_url( 'admin.php' ) ) . '#slicewp-card-settings-affiliate-start-id';

		// Author information.
		$this->author 	  = 'SliceWP';
		$this->author_url = 'https://slicewp.com/';

		parent::__construct();

		if ( $this->is_active() ) {

			// Adds a new card in the settings page.
			add_action( 'slicewp_view_settings_tab_general_bottom', array( $this, 'output_settings_card' ) );

			// Changes the auto increment value of the affiliate table on settings save.
			add_filter( 'pre_update_option_slicewp_settings', array( $this, 'maybe_alter_start_id' ), 50, 2 );
			
		}

	}


	/**
	 * Adds the Affiliate Start ID card in SliceWP's settings page, in the General tab.
	 *
	 */
	public function output_settings_card() {

		?>

			<div id="slicewp-card-settings-affiliate-start-id" class="slicewp-card">

				<div class="slicewp-card-header">
					<span class="slicewp-card-title"><?php echo __( 'Affiliate Start ID', 'slicewp' ); ?></span>
				</div>

				<div class="slicewp-card-inner">

					<div class="slicewp-field-wrapper slicewp-field-wrapper-inline slicewp-tooltip-wide">

						<div class="slicewp-field-label-wrapper">
							<label for="slicewp-affiliate-start-id">
								<?php echo __( 'Affiliate Start ID', 'slicewp' ); ?>
								<?php echo slicewp_output_tooltip( __( 'The starting ID for new affiliates. This number must be greater than the ID of the last registered affiliate. If you have no affiliates, affiliate registration will start with the ID set here.' , 'slicewp' ) ); ?>
							</label>
						</div>

						<input id="slicewp-affiliate-start-id" name="settings[affiliate_start_id]" type="number" value="<?php echo absint( ! empty( $_POST['settings']['affiliate_start_id'] ) ? $_POST['settings']['affiliate_start_id'] : ( slicewp_get_setting( 'affiliate_start_id', ( $this->get_latest_affiliate_id() + 1 ) ) ) ); ?>">

					</div>

				</div>

			</div>

		<?php

	}


	/**
	 * Changes the auto increment value of the affiliate table on settings save.
	 *
	 * @param array $new_settings
	 * @param array $old_settings
	 *
	 * @return array
	 *
	 */
	public function maybe_alter_start_id( $new_settings, $old_settings ) {

		$new_start_id = ( ! empty( $new_settings['affiliate_start_id'] ) ? absint( $new_settings['affiliate_start_id'] ) : 0 );
		$old_start_id = ( ! empty( $old_settings['affiliate_start_id'] ) ? absint( $old_settings['affiliate_start_id'] ) : 0 );

		// Bail if nothing changed.
		if ( $new_start_id == $old_start_id ) {
			return $new_settings;
		}

		// Make sure the start ID is higher than the last affiliate's ID.
		$latest_affiliate_id = $this->get_latest_affiliate_id();

		if ( $new_start_id <= $latest_affiliate_id ) {
			$new_start_id = $latest_affiliate_id + 1;
		}

		// Alter the auto increment.
		$this->alter_affiliate_table_auto_increment( $new_start_id );

		// Set the new start ID in case it changed.
		$new_settings['affiliate_start_id'] = $new_start_id;

		return $new_settings;

	}


	/**
	 * Changes the auto increment value for the affiliates table to the given value.
	 *
	 * @param int $start_id
	 *
	 * @return bool
	 *
	 */
	protected function alter_affiliate_table_auto_increment( $start_id ) {

		global $wpdb;

		$table_name = slicewp()->db['affiliates']->table_name;
		$start_id 	= absint( $start_id );

		$sql = "ALTER TABLE {$table_name}
		MODIFY `id` bigint(20) NOT NULL AUTO_INCREMENT,
		AUTO_INCREMENT = {$start_id};";

		$result = $wpdb->query( $sql );

		return false !== $result;

	}


	/**
	 * Returns the ID of the last affiliate that was inserted into the DB.
	 *
	 * @return int
	 *
	 */
	protected function get_latest_affiliate_id() {

		$affiliates = slicewp_get_affiliates( array( 'number' => 1 ) );

		return absint( ( ! empty( $affiliates ) ? $affiliates[0]->get( 'id' ) : 0 ) );

	}

}


/**
 * Registers the Affiliate Start ID add-on.
 * 
 * @param array
 * 
 * @return array
 * 
 */
function slicewp_register_add_on_affiliate_start_id( $add_ons ) {

	$add_ons['affiliate_start_id'] = 'SliceWP_Add_On_Affiliate_Start_ID';

	return $add_ons;

}
add_filter( 'slicewp_register_add_on', 'slicewp_register_add_on_affiliate_start_id', 200 );