<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Attempts to return the affiliate by the custom slug in the tracking script if no affiliate was found
 * with the given $query_arg.
 *
 * @param SliceWP_Affiliate|null $affiliate
 * @param string 				 $query_arg
 *
 * @return SliceWP_Affiliate|null
 *
 */
function slicewp_tracking_get_affiliate_by_custom_slug( $affiliate, $query_arg ) {

	// Return if we have an affiliate
	if ( ! is_null( $affiliate ) ) {
		return $affiliate;
	}

	return slicewp_get_affiliate_by_custom_slug( $query_arg );

}
add_filter( 'slicewp_tracking_get_affiliate_by_query_arg', 'slicewp_tracking_get_affiliate_by_custom_slug', 25, 2 );


/**
 * Retrieves an affiliate from the database by the given custom slug.
 *
 * @param string $custom_slug
 *
 * @return SliceWP_Affiliate|null
 *
 */
function slicewp_get_affiliate_by_custom_slug( $custom_slug ) {

	global $wpdb;

	$table_name = slicewp()->db['affiliatemeta']->table_name;

	$affiliate_id = absint( $wpdb->get_var( $wpdb->prepare( "SELECT slicewp_affiliate_id FROM {$table_name} WHERE meta_key = '%s' AND meta_value = '%s'", 'custom_slug', sanitize_text_field( $custom_slug ) ) ) );

	if ( empty( $affiliate_id ) ) {
		return null;
	}

	return slicewp_get_affiliate( $affiliate_id );

}


/**
 * Generates a unique affiliate slug.
 *
 * @return string
 *
 */
function slicewp_generate_unique_affiliate_slug() {

	// Slug type.
	$slug_type = slicewp_get_setting( 'custom_affiliate_slug_random_type', 'alphanumeric' );

	// Slug length.
	$slug_length = absint( slicewp_get_setting( 'custom_affiliate_slug_random_length', 7 ) );
	
	// Make sure the slug is between 5 and 50.
	$slug_length = ( $slug_length < 5  ? 5  : $slug_length );
	$slug_length = ( $slug_length > 50 ? 50 : $slug_length );

	// Generate a slug random string.
	$slug = _slicewp_generate_random_string( $slug_length, $slug_type );

	// If the slug is already taken, generate a new one.
	if ( ! is_null( slicewp_get_affiliate_by_custom_slug( $slug ) ) ) {

		$slug = slicewp_generate_unique_affiliate_slug();

	}

	// If the slug is numeric, generate a new one.
	// We don't want them to overal an affiliate's ID.
	if ( is_numeric( $slug ) ) {

		$slug = slicewp_generate_unique_affiliate_slug();

	}

	return $slug;

}


/**
 * Replaces the referral query argument value with the custom slug, if the affiliate has it set.
 *
 * @param mixed  $query_arg_value
 * @param int    $affiliate_id
 * @param string $format
 *
 * @return int|string
 *
 */
function slicewp_affiliate_url_referral_query_arg_value_custom_slug( $query_arg_value, $affiliate_id, $format ) {

	// Bail if the referral link format isn't set to "custom_slug".
	if ( 'custom_slug' != slicewp_get_setting( 'affiliate_referral_link_format', '' ) && $format != 'custom_slug' ) {
		return $query_arg_value;
	}

	$custom_slug = slicewp_get_affiliate_meta( $affiliate_id, 'custom_slug', true );

	// Bail if the affiliate doesn't have a custom slug.
	if ( empty( $custom_slug ) ) {
		return $query_arg_value;
	}

	return $custom_slug;

}
add_filter( 'slicewp_affiliate_url_referral_query_arg_value', 'slicewp_affiliate_url_referral_query_arg_value_custom_slug', 25, 3 );


/**
 * Generates a random slug for the affiliate right after the affiliate was inserted.
 *
 * @param int $affiliate_id
 *
 */
function slicewp_insert_affiliate_generate_random_slug( $affiliate_id ) {

	if ( slicewp_doing_admin_action( 'add_affiliate' ) ) {
		return;
	}

	if ( isset( $_POST['custom_slug'] ) ) {
		return;
	}

	if ( ! slicewp_get_setting( 'custom_affiliate_slug_auto_random', false ) ) {
		return;
	}

	if ( ! empty( slicewp_get_affiliate_meta( $affiliate_id, 'custom_slug', true ) ) ) {
		return;
	}

	slicewp_add_affiliate_meta( $affiliate_id, 'custom_slug', sanitize_text_field( slicewp_generate_unique_affiliate_slug() ) );

}
add_action( 'slicewp_insert_affiliate', 'slicewp_insert_affiliate_generate_random_slug', 20 );


/**
 * Registers the "custom_slug" column in the CSV export file.
 *
 * @param array $cols
 *
 * @return $cols
 *
 */
function slicewp_export_csv_cols_affiliates_custom_slug( $cols ) {

	$cols['custom_slug'] = __( 'Custom Slug', 'slicewp' );

	return $cols;

}
add_filter( 'slicewp_export_csv_cols_affiliates', 'slicewp_export_csv_cols_affiliates_custom_slug', 20 );


/**
 * Adds the value for the "custom_slug" column in the affiliates CSV export item data.
 *
 * @param array 			$item_data
 * @param SliceWP_Affiliate $affiliate
 *
 * @return array
 *
 */
function slicewp_export_get_data_item_affiliates_custom_slug( $item_data, $affiliate ) {

	$custom_slug = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'custom_slug', true );

	$item_data['custom_slug'] = ( ! empty( $custom_slug ) ? $custom_slug : '' );

	return $item_data;

}
add_filter( 'slicewp_export_get_data_item_affiliates', 'slicewp_export_get_data_item_affiliates_custom_slug', 20, 2 );