<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Base class for all batch exports.
 *
 */
abstract class SliceWP_Batch_Export extends SliceWP_Export {

	/**
	 * The type of the file to export.
	 *
	 * @access protected
	 * @var    string
	 *
	 */
	protected $file_type = 'csv';

	/**
	 * The file the data is stored in.
	 *
	 * @access protected
	 *
	 */
	protected $file;

	/**
	 * The current step of the batch process.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $step;

	/**
	 * The number of items to process per step.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $per_step = 50;

	/**
	 * Is the export file writable.
	 *
	 * @access public
	 * @var    bool
	 *
	 */
	public $is_file_writable = true;

	/**
	 * Is the export file empty.
	 *
	 * @access public
	 * @var    bool
	 *
	 */
	public $is_file_empty = false;

	/**
	 * Is the export file complete.
	 *
	 * @access public
	 * @var    bool
	 *
	 */
	public $complete = false;

	/**
	 * The number of processed items in this batch.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $items_processed_current_batch = 0;

	/**
	 * Total number of items process in this export.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $items_processed = 0;

	/**
	 * The total number of items of the export type.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $items_total = 0;

	/**
	 * An array of elements to filter the exported data.
	 *
	 * @access protected
	 * @var    array
	 *
	 */
	protected $filters = array();

	/**
	 * The arguments array for the export process.
	 *
	 * @access protected
	 * @var    array
	 *
	 */
	protected $args;


	/**
	 * Constructor.
	 *
	 * @param array $args
	 *
	 */
	public function __construct( $args = array() ) {

		// Set the export args
		$this->args = $args;

	}


	/**
	 * Initializez the exporter.
	 *
	 */
	public function init() {

		$upload_dir = wp_upload_dir();

		$this->file_name = ( ! empty( $this->args['file_name'] ) ? $this->args['file_name'] : $this->generate_file_name() );

		$this->file = trailingslashit( $upload_dir['basedir'] ) . $this->file_name;

		if ( ! is_writable( $upload_dir['basedir'] ) ) {
			$this->is_file_writable = false;
		}

		// Set items count
		$this->items_processed = ( ! empty( $this->args['items_processed'] ) ? absint( $this->args['items_processed'] ) : 0 );
		$this->items_total 	   = ( ! empty( $this->args['items_total'] ) ? absint( $this->args['items_total'] ) : $this->fetch_items_total() );

		// Set the current step
		$this->step = ( ! empty( $this->args['step'] ) ? absint( $this->args['step'] ) : 1 );

	}


	/**
	 * Retrieve the CSV columns.
	 *
	 * @access public
	 *
	 * @return array $cols
	 *
	 */
	public function get_csv_cols() {

		$cols = $this->csv_cols();

		return apply_filters( 'slicewp_export_csv_cols_' . $this->export_type, $cols );

	}


	/**
	 * Output the CSV columns.
	 *
	 * @access public
	 *
	 * @return string
	 *
	 */
	public function print_csv_cols() {

		$col_data = '';
		$cols     = $this->get_csv_cols();
		$i        = 1;

		foreach ( $cols as $col_id => $column ) {

			$col_data .= '"' . addslashes( $column ) . '"';
			$col_data .= ( $i == count( $cols ) ? '' : ',' );

			$i ++;

		}

		$col_data .= "\r\n";

		$this->stash_step_data( $col_data );

		return $col_data;

	}


	/**
	 * Print the CSV rows for the current step.
	 *
	 * @access public
	 *
	 * @return string|false
	 *
	 */
	public function print_csv_rows() {

		$row_data = '';
		$data     = $this->get_data();
		$cols     = $this->get_csv_cols();

		if ( empty( $data ) || ! is_array( $data ) ) {
			return false;
		}

		// Output each row.
		foreach ( $data as $row ) {

			$i = 1;

			foreach ( array_keys( $cols ) as $col_id ) {

				$column = ( isset( $row[$col_id] ) ? $row[$col_id] : '' );

				$row_data .= '"' . addslashes( preg_replace( '/"/', "'", $column ) ) . '"';
				$row_data .= ( $i == count( $cols ) ? '' : ',' );

				$i ++;

			}

			$row_data .= "\r\n";

		}

		$this->items_processed_current_batch = count( $data );
		$this->stash_step_data( $row_data );

		return $row_data;

	}


	/**
	 * Retrieve the file data is written to.
	 *
	 * @return string
	 *
	 */
	protected function fetch_file() {

		$file = '';

		if ( @file_exists( $this->file ) ) {

			if ( ! is_writable( $this->file ) ) {
				$this->is_file_writable = false;
			}

			$file = @file_get_contents( $this->file );

		} else {

			@file_put_contents( $this->file, '' );
			@chmod( $this->file, 0664 );

		}

		return $file;

	}


	/**
	 * Returns the offset for query.
	 *
	 * @return int
	 *
	 */
	protected function get_offset() {

		return ( $this->step -1 ) * $this->per_step;

	}


	/**
	 * Process a step.
	 *
	 * @return bool
	 *
	 */
	public function process_step() {

		if ( ! $this->can_export() ) {
			
			wp_die( __( 'You do not have permission to export data.', 'slicewp' ), __( 'Error', 'slicewp' ), array( 'response' => 403 ) );

		}

		if ( $this->step < 2 ) {

			// Make sure we start with a fresh file on step 1.
			@unlink( $this->file );
			$this->print_csv_cols();

		}

		$this->print_csv_rows();

		return $this->get_percentage_complete();

	}


	/**
	 * Returns the total number of items that should be processed, in batches.
	 *
	 * @return int
	 *
	 */
	protected function fetch_items_total() {

		return 0;

	}


	/**
	 * Returns a message based on the provided slug.
	 *
	 * @param string $slug
	 *
	 * @return string
	 *
	 */
	public function get_message( $slug = '' ) {

		return '';

	}


	/**
	 * Return the calculated completion percentage.
	 *
	 * @return int
	 *
	 */
	public function get_percentage_complete() {

		$percentage = absint( $this->items_processed * 100 / $this->items_total );

		return ( $percentage >= 100 ? 100 : $percentage );

	}


	/**
	 * Append data to export file.
	 *
	 * @param string $data
	 *
	 */
	protected function stash_step_data( $data = '' ) {

		$file  = $this->fetch_file();
		$file .= $data;
		@file_put_contents( $this->file, $file );

		// Set total processed items
		$this->items_processed = $this->items_processed + $this->items_processed_current_batch;

		// If we have no rows after this step, mark it as an empty export.
		$file_rows    = file( $this->file, FILE_SKIP_EMPTY_LINES );

		$default_cols = $this->get_csv_cols();
		$default_cols = empty( $default_cols ) ? 0 : 1;

		$this->is_file_empty = count( $file_rows ) == $default_cols ? true : false;

	}


	/**
	 * Perform the export.
	 *
	 * @access public
	 *
	 */
	public function export() {

		// Check if the user can perform the export.
		if ( ! $this->can_export() ) {

			wp_die( __( 'You do not have permission to export data.', 'slicewp' ), __( 'Error', 'slicewp' ), array( 'response' => 403 ) );

		}

		// Set headers.
		$this->headers();

		$file = $this->fetch_file();

		@unlink( $this->file );

		echo $file;

		die();

	}

}