<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Base class for all exports.
 *
 */
abstract class SliceWP_Export extends SliceWP_Base_Object {

	/**
	 * The export type.
	 *
	 * @access protected 
	 * @var    string
	 *
	 */
	protected $export_type = 'default';

	/**
	 * The type of the file to export.
	 *
	 * @access protected
	 * @var    string
	 *
	 */
	protected $file_type;

	/**
	 * The name of the file to export.
	 *
	 * @access protected
	 * @var    string
	 *
	 */
	protected $file_name = '';


	/**
	 * Determines whether the user can do an export or not.
	 *
	 * @access public
	 *
	 * @return bool
	 *
	 */
	public function can_export() {

		/**
		 * Filter the user's export capability.
		 *
		 * @param string
		 *
		 */
		return current_user_can( apply_filters( 'slicewp_export_capability', 'manage_options' ) );

	}


	/**
	 * Returns the file name.
	 *
	 */
	public function generate_file_name() {

		$file_name = 'slicewp-export-' . $this->export_type . '-' . date( 'Y-m-d-H_i_s' ) . '.' . $this->file_type;

		/**
		 * Filters the export file name, just before returning it.
		 *
		 * @param string $file_name
		 * @param string $export_type
		 *
		 */
		return apply_filters( 'slicewp_export_generate_file_name', $file_name, $this->export_type, $this->file_type );

	}


	/**
	 * Returns the MIME type for the given file type.
	 *
	 * @param string $file_stype
	 *
	 * @return string
	 *
	 */
	protected function get_mime_type( $file_type ) {

		$mime_types = array(
			'csv'  => 'text/csv',
			'json' => 'application/json'
		);

		/**
		 * Filter the MIME types.
		 *
		 * @param array $mime_types
		 *
		 */
		$mime_types = apply_filters( 'slicewp_export_mime_types', $mime_types );

		return ( ! empty( $mime_types[$file_type] ) ? $mime_types[$file_type] : '' );

	}


	/**
	 * Returns the data to be exported in the form of an array.
	 *
	 * Should be overwritten by extending classes.
	 *
	 * @return array
	 *
	 */
	protected function get_data() {

		return $this->filter_data( array() );

	}


	/**
	 * Applies two filters to the given export data, to modify it right before exporting it.
	 *
	 * @param array $data
	 *
	 * @return array
	 *
	 */
	protected function filter_data( $data ) {

		/**
		 * Filter the data.
		 *
		 * @param array $data
		 *
		 */
		$data = apply_filters( 'slicewp_export_get_data', $data );

		/**
		 * Filter the data by export type.
		 *
		 * @param array $data
		 *
		 */
		$data = apply_filters( 'slicewp_export_get_data_' . $this->export_type, $data );

		return $data;

	}


	/**
	 * Sets the export headers.
	 *
	 */
	protected function headers() {

		ignore_user_abort( true );

		if ( ! slicewp_is_func_disabled( 'set_time_limit' ) ) {
			set_time_limit( 0 );
		}

		nocache_headers();

		header( 'Content-Type: ' . $this->get_mime_type( $this->file_type ) . '; charset=utf-8' );
		header( 'Content-Disposition: attachment; filename=' . $this->file_name );
		header( "Expires: 0" );

	}


	/**
	 * Perform the export.
	 *
	 */
	public function export() {

		// Check if the user can perform the export.
		if ( ! $this->can_export() ) {

			wp_die( __( 'You do not have permission to export data.', 'slicewp' ), __( 'Error', 'slicewp' ), array( 'response' => 403 ) );

		}

		// Set headers
		$this->headers();

	}
	
}