<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Handles batch export for affiliates
 *
 */
class SliceWP_Batch_Export_Affiliates extends SliceWP_Batch_Export {

	/**
	 * The export type.
	 *
	 * @access protected 
	 * @var    string
	 *
	 */
	protected $export_type = 'affiliates';

	/**
	 * The number of items to process per step.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $per_step = 25;


	/**
	 * Returns the columns for the CSV export file.
	 *
	 * @return array
	 *
	 */
	protected function csv_cols() {

		$cols = array(
			'affiliate_id'    	 => __( 'Affiliate ID', 'slicewp' ),
			'user_id'			 => __( 'User ID', 'slicewp' ),
			'email'           	 => __( 'Email', 'slicewp' ),
			'first_name'      	 => __( 'First Name', 'slicewp' ),
			'last_name'       	 => __( 'Last Name', 'slicewp' ),
			'payment_email'   	 => __( 'Payment Email', 'slicewp' ),
			'commissions_paid'	 => __( 'Paid Commissions', 'slicewp' ),
			'commissions_unpaid' => __( 'Unpaid Commissions', 'slicewp' ),
			'visits'          	 => __( 'Visits', 'slicewp' ),
			'status'          	 => __( 'Status', 'slicewp' ),
			'date_registered' 	 => __( 'Date Registered', 'slicewp' )
		);

		return $cols;

	}


	/**
	 * Returns the total number of affiliates that should be processed.
	 *
	 * @return int
	 *
	 */
	protected function fetch_items_total() {

		$args = array(
			'status' => ( ! empty( $this->filters['status'] ) ? sanitize_text_field( $this->filters['status'] ) : '' ),
			'number' => -1
		);

		return absint( slicewp_get_affiliates( $args, true ) );

	}


	/**
	 * Returns an array with the affiliate data for the current batch step.
	 *
	 * @return array
	 *
	 */
	protected function get_data() {

		$data = array();

		$affiliate_statuses = slicewp_get_affiliate_available_statuses();

		$affiliates_args = array(
			'status' => ( ! empty( $this->filters['status'] ) ? sanitize_text_field( $this->filters['status'] ) : '' ),
			'number' => $this->per_step,
			'offset' => $this->get_offset()
		);

		$affiliates = slicewp_get_affiliates( $affiliates_args );

		if ( ! empty( $affiliates ) ) {

			foreach ( $affiliates as $affiliate ) {

				$user = get_user_by( 'id', $affiliate->get( 'user_id' ) );

				$data[] = apply_filters( 'slicewp_export_get_data_item_' . $this->export_type, array(
					'affiliate_id'    	 => $affiliate->get( 'id' ),
					'user_id'			 => $affiliate->get( 'user_id' ),
					'email'           	 => ( $user ? $user->user_email : '' ),
					'first_name'      	 => ( $user ? $user->first_name : '' ),
					'last_name'       	 => ( $user ? $user->last_name : '' ),
					'payment_email'   	 => $affiliate->get( 'payment_email' ),
					'commissions_paid'	 => slicewp_get_commissions( array( 'number' => -1, 'affiliate_id' => $affiliate->get( 'id' ), 'status' => 'paid' ), true ),
					'commissions_unpaid' => slicewp_get_commissions( array( 'number' => -1, 'affiliate_id' => $affiliate->get( 'id' ), 'status' => 'unpaid' ), true ),
					'visits'          	 => slicewp_get_visits( array( 'number' => -1, 'affiliate_id' => $affiliate->get( 'id' ) ), true ),
					'status'          	 => ( ! empty( $affiliate_statuses[$affiliate->get( 'status' )] ) ? $affiliate_statuses[$affiliate->get( 'status' )] : $affiliate->get( 'status' ) ),
					'date_registered' 	 => slicewp_date_i18n( esc_attr( $affiliate->get( 'date_created' ) ) )
				), $affiliate );

			}

		}

		return $this->filter_data( $data );

	}


	/**
	 * Returns a message based on the provided slug.
	 *
	 * @param string $slug
	 *
	 * @return string
	 *
	 */
	public function get_message( $slug = '' ) {

		if ( 'complete' == $slug ) {

			return sprintf( __( '%s affiliates exported.', 'slicewp' ), '<strong>' . absint( $this->items_total ) . '</strong>' );

		}

		return '';

	}

}