<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Handles batch export for leads.
 *
 */
class SliceWP_Batch_Export_Leads extends SliceWP_Batch_Export {

	/**
	 * The export type.
	 *
	 * @access protected 
	 * @var    string
	 *
	 */
	protected $export_type = 'leads';


	/**
	 * Returns the columns for the CSV export file.
	 *
	 * @return array
	 *
	 */
	protected function csv_cols() {

		$cols = array(
			'lead_id'    	  => __( 'Lead ID', 'slicewp' ),
            'lead_first_name' => __( 'Lead First Name', 'slicewp' ),
            'lead_last_name'  => __( 'Lead Last Name', 'slicewp' ),
            'lead_email'      => __( 'Lead Email', 'slicewp' ),
			'affiliate_id'	  => __( 'Affiliate ID', 'slicewp' ),
            'origin'          => __( 'Origin', 'slicewp' ),
            'entry_id'        => __( 'Entry ID', 'slicewp' ),
			'commission_id'	  => __( 'Commission ID', 'slicewp' ),
            'status'          => __( 'Status', 'slicewp' ),
			'date' 	 		  => __( 'Date', 'slicewp' )
		);

		return $cols;

	}


	/**
	 * Returns the total number of leads that should be processed.
	 *
	 * @return int
	 *
	 */
	protected function fetch_items_total() {

		// Get the start date from the filter, or set it to the default value if not present
		$date_min = ( ! empty( $this->filters['date_min'] ) ? new DateTime( $this->filters['date_min'] . ' 00:00:00' ) : '' );

		// Get the end date from the filter, or set it to the default value if not present
		$date_max = ( ! empty( $this->filters['date_max'] ) ? new DateTime( $this->filters['date_max'] . ' 23:59:59') : '' );		

		$args = array(
			'converted'    => ( ! empty( $this->filters['status'] ) ? ( $this->filters['status'] === 'converted' ? true : false ) : '' ),
			'affiliate_id' => ( ! empty( $this->filters['affiliate_id'] ) ? absint( $this->filters['affiliate_id']  ) : 0 ),
			'date_min' 	   => ( ! empty( $date_min ) ? get_gmt_from_date( $date_min->format( 'Y-m-d H:i:s' ) ) : '' ),
			'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
			'number'   	   => -1
		);

		return absint( slicewp_get_leads( $args, true ) );

	}


	/**
	 * Returns an array with the lead data for the current batch step.
	 *
	 * @return array
	 *
	 */
	protected function get_data() {

		$data = array();

		// Get the start date from the filter, or set it to the default value if not present
		$date_min = ( ! empty( $this->filters['date_min'] ) ? new DateTime( $this->filters['date_min'] . ' 00:00:00' ) : '' );

		// Get the end date from the filter, or set it to the default value if not present
		$date_max = ( ! empty( $this->filters['date_max'] ) ? new DateTime( $this->filters['date_max'] . ' 23:59:59') : '' );

		$leads_args = array(
			'converted'    => ( ! empty( $this->filters['status'] ) ? ( $this->filters['status'] === 'converted' ? true : false ) : '' ),
			'affiliate_id' => ( ! empty( $this->filters['affiliate_id'] ) ? absint( $this->filters['affiliate_id']  ) : 0 ),
			'date_min' 	   => ( ! empty( $date_min ) ? get_gmt_from_date( $date_min->format( 'Y-m-d H:i:s' ) ) : '' ),
			'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
			'number'   	   => $this->per_step,
			'offset'   	   => $this->get_offset()
		);

		$leads = slicewp_get_leads( $leads_args );

		if ( ! empty( $leads ) ) {

			foreach ( $leads as $lead ) {

				$data[] = apply_filters( 'slicewp_export_get_data_item_' . $this->export_type, array(
                    'lead_id'    	  => $lead->get( 'id' ),
                    'lead_first_name' => $lead->get( 'first_name' ),
                    'lead_last_name'  => $lead->get( 'last_name' ),
                    'lead_email'      => $lead->get( 'email' ),
                    'affiliate_id'	  => $lead->get( 'affiliate_id' ),
                    'origin'          => ( ! empty( slicewp()->integrations[$lead->get( 'origin' )] ) ? slicewp()->integrations[$lead->get( 'origin' )]->get( 'name' ) : $lead->get( 'origin' ) ),
                    'entry_id'        => $lead->get( 'entry_id' ),
                    'commission_id'	  => $lead->get( 'commission_id' ),
                    'status'          => ( ! empty( $lead->get( 'commission_id' ) ) ? __( 'Converted', 'slicewp' ) : __( 'Unconverted', 'slicewp' ) ),
                    'date' 	 		  => slicewp_date_i18n( esc_attr( $lead->get( 'date_created' ) ) )
				), $lead );

			}

		}

		return $this->filter_data( $data );

	}


	/**
	 * Returns a message based on the provided slug.
	 *
	 * @param string $slug
	 *
	 * @return string
	 *
	 */
	public function get_message( $slug = '' ) {

		if ( 'complete' == $slug ) {

			return sprintf( __( '%s leads exported.', 'slicewp' ), '<strong>' . absint( $this->items_total ) . '</strong>' );

		}

		return '';

	}

}