<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Handles batch export for visits.
 *
 */
class SliceWP_Batch_Export_Visits extends SliceWP_Batch_Export {

	/**
	 * The export type.
	 *
	 * @access protected 
	 * @var    string
	 *
	 */
	protected $export_type = 'visits';

	/**
	 * The number of items to process per step.
	 *
	 * @access protected
	 * @var    int
	 *
	 */
	protected $per_step = 100;


	/**
	 * Returns the columns for the CSV export file.
	 *
	 * @return array
	 *
	 */
	protected function csv_cols() {

		$cols = array(
			'visit_id'    	=> __( 'Visit ID', 'slicewp' ),
			'affiliate_id'	=> __( 'Affiliate ID', 'slicewp' ),
			'landing_url'	=> __( 'Landing URL', 'slicewp' ),
			'referrer_url'	=> __( 'Referrer URL', 'slicewp' ),
			'ip_address'	=> __( 'IP Address', 'slicewp' ),
			'commission_id'	=> __( 'Commission ID', 'slicewp' ),
			'status'        => __( 'Status', 'slicewp' ),
			'date' 	 		=> __( 'Date', 'slicewp' )
		);

		return $cols;

	}


	/**
	 * Returns the total number of visits that should be processed.
	 *
	 * @return int
	 *
	 */
	protected function fetch_items_total() {

		// Get the start date from the filter, or set it to the default value if not present.
		$date_min = ( ! empty( $this->filters['date_min'] ) ? new DateTime( $this->filters['date_min'] . ' 00:00:00' ) : '' );

		// Get the end date from the filter, or set it to the default value if not present.
		$date_max = ( ! empty( $this->filters['date_max'] ) ? new DateTime( $this->filters['date_max'] . ' 23:59:59') : '' );		

		$args = array(
			'converted'    => ( ! empty( $this->filters['status'] ) ? ( $this->filters['status'] === 'converted' ? true : false ) : '' ),
			'affiliate_id' => ( ! empty( $this->filters['affiliate_id'] ) ? absint( $this->filters['affiliate_id']  ) : 0 ),
			'date_min' 	   => ( ! empty( $date_min ) ? get_gmt_from_date( $date_min->format( 'Y-m-d H:i:s' ) ) : '' ),
			'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
			'number'   	   => -1
		);

		return absint( slicewp_get_visits( $args, true ) );

	}


	/**
	 * Returns an array with the visit data for the current batch step.
	 *
	 * @return array
	 *
	 */
	protected function get_data() {

		$data = array();

		// Get the start date from the filter, or set it to the default value if not present.
		$date_min = ( ! empty( $this->filters['date_min'] ) ? new DateTime( $this->filters['date_min'] . ' 00:00:00' ) : '' );

		// Get the end date from the filter, or set it to the default value if not present.
		$date_max = ( ! empty( $this->filters['date_max'] ) ? new DateTime( $this->filters['date_max'] . ' 23:59:59') : '' );

		$visits_args = array(
			'converted'    => ( ! empty( $this->filters['status'] ) ? ( $this->filters['status'] === 'converted' ? true : false ) : '' ),
			'affiliate_id' => ( ! empty( $this->filters['affiliate_id'] ) ? absint( $this->filters['affiliate_id']  ) : 0 ),
			'date_min' 	   => ( ! empty( $date_min ) ? get_gmt_from_date( $date_min->format( 'Y-m-d H:i:s' ) ) : '' ),
			'date_max' 	   => ( ! empty( $date_max ) ? get_gmt_from_date( $date_max->format( 'Y-m-d H:i:s' ) ) : '' ),
			'number'   	   => $this->per_step,
			'offset'   	   => $this->get_offset()
		);

		$visits = slicewp_get_visits( $visits_args );

		if ( ! empty( $visits ) ) {

			foreach ( $visits as $visit ) {

				$data[] = apply_filters( 'slicewp_export_get_data_item_' . $this->export_type, array(
					'visit_id'		=> $visit->get( 'id' ),
					'affiliate_id'  => $visit->get( 'affiliate_id' ),
					'landing_url'	=> rawurldecode( $visit->get( 'landing_url' ) ),
					'referrer_url'	=> rawurldecode( $visit->get( 'referrer_url' ) ),
					'ip_address'	=> $visit->get( 'ip_address' ),
					'commission_id'	=> $visit->get( 'commission_id' ),
					'status'        => ( ! empty( $visit->get( 'commission_id' ) ) ? __( 'Converted', 'slicewp' ) : __( 'Unconverted', 'slicewp' ) ),
					'date' 	 		=> slicewp_date_i18n( esc_attr( $visit->get( 'date_created' ) ) )
				), $visit );

			}

		}

		return $this->filter_data( $data );

	}


	/**
	 * Returns a message based on the provided slug.
	 *
	 * @param string $slug
	 *
	 * @return string
	 *
	 */
	public function get_message( $slug = '' ) {

		if ( 'complete' == $slug ) {

			return sprintf( __( '%s visits exported.', 'slicewp' ), '<strong>' . absint( $this->items_total ) . '</strong>' );

		}

		return '';

	}

}