<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Base class for all lifetime commissions integration objects.
 *
 */
class SliceWP_Integration_Lifetime_Commissions {

	/**
	 * The slug for the entity that generated the commission.
	 *
	 * @access protected
	 * @var    string
	 *
	 */
	protected $origin;

	/**
	 * Constructor.
	 *
	 */
	public function __construct() {

		// Updates the commission type to lifetime_sale.
		add_filter( 'slicewp_pre_insert_commission_data', array( $this, 'update_commission_type' ) );
		
		// Calculate the lifetime commission amount.
		add_filter( 'slicewp_calculate_commission_amount', array( $this, 'calculate_commission_amount' ), 100, 3 );


	}


	/**
	 * Returns the affiliate's lifetime limit.
	 *
	 * @param int $affiliate_id
	 *
	 * @return null|false|int - null if the default should be used, false for unlimited time and int for limit
	 *
	 */
	protected function get_affiliate_lifetime_limit( $affiliate_id ) {

		$lifetime_limit = null;
		
		$lifetime_duration = slicewp_get_affiliate_meta( $affiliate_id, 'commissions_lifetime_duration', true );

		if ( ! empty( $lifetime_duration ) ) {

			$lifetime_limit = ( $lifetime_duration == 'limited' ? absint( slicewp_get_affiliate_meta( $affiliate_id, 'commissions_lifetime_limit', true ) ) : false );

		}

		/**
		 * Filter the affiliate lifetime limit before returning it.
		 *
		 * @param null|int $lifetime_limit
		 * @param int 	   $affiliate_id
		 *
		 */
		return apply_filters( 'slicewp_affiliate_lifetime_commissions_lifetime_limit', $lifetime_limit, $affiliate_id );

	}
	

	/**
	 * Returns the overall lifetime duration. Priority is given to the affiliate's lifetime duration
	 * then to the general settings.
	 *
	 * @param int $affiliate_id
	 * 
	 * @return null|false|int - null if the default should be used, false for unlimited time and int for limit
	 *
	 */
	protected function get_lifetime_limit( $affiliate_id ) {

		$lifetime_limit = null;

		// If the affiliate has lifetime commissions active, see if it also has the lifetime duration set.
		// If it does, set it to the returned value.
		if ( 'custom' == slicewp_get_affiliate_meta( $affiliate_id, 'lifetime_commissions', true ) ) {

			$affiliate_lifetime_limit = $this->get_affiliate_lifetime_limit( $affiliate_id );
			$lifetime_limit 		  = ( ! is_null( $affiliate_lifetime_limit ) ? $affiliate_lifetime_limit : null );

		}

		// If there's no lifetime limit settings set on the affiliate, grab the lifetime limit settings set on the group
		// that the affiliate is part of (if they are part of a group).
		if ( is_null( $lifetime_limit ) ) {

			$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

			if ( ! empty( $affiliate_group_id ) ) {

				if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_lifetime_commissions', true ) ) {
				
					$group_lifetime_duration = slicewp_get_collection_meta( $affiliate_group_id, '_commissions_lifetime_duration', true );

					if ( ! empty( $group_lifetime_duration ) ) {

						$lifetime_limit = ( $group_lifetime_duration == 'limited' ? absint( slicewp_get_collection_meta( $affiliate_group_id, '_commissions_lifetime_limit', true ) ) : false );
			
					}

				}

			}

		}

		// If the lifetime limit isn't set for affiliate, set it to the default in settings.
		if ( is_null( $lifetime_limit ) ) {

			$default_lifetime_duration	= slicewp_get_setting( 'commissions_lifetime_duration', 'unlimited' );
			$default_lifetime_limit 	= slicewp_get_setting( 'commissions_lifetime_limit', 0 );

			$lifetime_limit = ( $default_lifetime_duration == 'limited' ? absint( $default_lifetime_limit ) : false );

		}

		
		/**
		 * Filter the affiliate lifetime limit before returning it.
		 *
		 * @param false|int $lifetime_limit
		 * @param array		$affiliate_id
		 *
		 */
		return apply_filters( 'slicewp_lifetime_commissions_lifetime_duration', $lifetime_limit, $affiliate_id );

	}

	
	/**
	 * Verifies if the lifetime duration has been reached for the given affiliate.
	 *
	 * @param int $affiliate_id
	 *
	 * @return bool
	 *
	 */
	protected function lifetime_limit_reached( $affiliate_id, $customer_id ) {

		// Get the lifetime limit.
		$lifetime_limit = $this->get_lifetime_limit( $affiliate_id );

		// Bail quickly if the lifetime duration is infinite.
		if ( false === $lifetime_limit ) {
			return false;
		}

		// Get customer registration date.
		$customer 		   = slicewp_get_customer( $customer_id );
		$registration_date = new DateTime( $customer->get( 'date_created' ) );
		$now 			   = new DateTime();

		if ( ( absint( $registration_date->diff( $now )->format('%a') ) <= $lifetime_limit ) ) {
			return false;
		}

		// Lifetime limit reached :(
		return true;

	}


	/**
	 * Checks if the affiliate already has a commission generated by a particular customer.
	 *
	 * @param int $affiliate_id
	 * @param int $customer_id
	 *
	 * @return bool
	 *
	 */
	protected function affiliate_has_customer_commission( $affiliate_id, $customer_id ) {

		$commissions = slicewp_get_commissions( array( 'number' => 1, 'affiliate_id' => $affiliate_id, 'customer_id' => $customer_id, 'status' => array( 'paid', 'unpaid', 'pending' ) ) );

		return ( ! empty( $commissions ) ? true : false );

	}


	/**
	 * Modifies the commission amount based on the lifetime commission rates.
	 *
	 * @param float $commission_amount
	 * @param float $amount
	 * @param array $args
	 *
	 * @return float
	 *
	 */
	public function calculate_commission_amount( $commission_amount, $amount, $args ) {

		// Check for customer id.
		if ( empty( $args['customer_id'] ) ) {
			return $commission_amount;
		}

		// The lifetime commission is only for sale and subscription types.
		if ( ! in_array( $args['type'], array( 'sale', 'subscription' ) ) ) {
			return $commission_amount;
		}

		// Get the affiliate linked to the customer.
		$affiliate_id = slicewp_get_customer_meta( $args['customer_id'], 'affiliate_id', true );

		// No affiliate linked to the customer.
		if ( empty( $affiliate_id ) ) {
			return $commission_amount;
		}

		// Check if the affiliate already has a commission from the customer.
		if ( ! $this->affiliate_has_customer_commission( $affiliate_id, $args['customer_id'] ) ) {
			return $commission_amount;
		}

		// Get affiliate lifetime commissions status.
		$lifetime_commissions = slicewp_get_affiliate_meta( $affiliate_id, 'lifetime_commissions', true );

		// Lifetime commissions are disabled for affiliate.
		if ( 'disabled' == $lifetime_commissions ) {
			return $commission_amount;
		}

		// Return if lifetime commissions are disabled for the group the affiliate is in.
		$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

		$group_lifetime_commissions = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_lifetime_commissions', true ) : '' );

		if ( 'disabled' == $group_lifetime_commissions ) {
			return $commission_amount;
		}

		// Affiliate and affiliate group lifetime commissions are set to site default
		// and the global lifetime commissions are disabled.
		if ( empty( $lifetime_commissions ) && empty( $group_lifetime_commissions ) && empty( slicewp_get_setting( 'lifetime_commissions' ) ) ) {
			return $commission_amount;
		}

		// Check if affiliate lifetime limit has been reached.
		if ( $this->lifetime_limit_reached( $affiliate_id, $args['customer_id'] ) ) {
			return $commission_amount;
		}

		$rate_type = '';
		$rate      = '';

		// Get the affiliate custom lifetime commission rates.
		if ( 'custom' == $lifetime_commissions ) {

			$rate      = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_lifetime_sale', true );
			$rate_type = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_type_lifetime_sale', true );

		}

		// Get the affiliate group custom lifetime commission rates.
		if ( 'custom' == $group_lifetime_commissions && ( empty( $rate_type ) || ! is_numeric( $rate ) ) ) {

			$rate      = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_lifetime_sale', true );
			$rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_lifetime_sale', true );

		}

		// Get default lifetime rate and rate type.
		if ( empty( $rate_type ) || ! is_numeric( $rate ) ) {

			$rate      = slicewp_get_setting( 'commission_rate_lifetime_sale' );
			$rate_type = slicewp_get_setting( 'commission_rate_type_lifetime_sale' );

		}

		// No lifetime rates found.
		if ( empty( $rate_type ) || ! is_numeric( $rate ) ) {
			return $commission_amount;
		}

		// Compute the lifetime commission.
		$commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

		return $commission_amount;

	}


	/**
	 * Updates the commission type to lifetime_sale.
	 *
	 * @param array $data
	 * 
	 * @return array
	 * 
	 */
	public function update_commission_type( $data ) {

		if ( empty( $data['customer_id'] ) ) {
			return $data;
		}

		// The lifetime commission is only for sale and subscription types.
		if ( ! in_array( $data['type'], array( 'sale', 'subscription' ) ) ) {
			return $data;
		}

		// Get the affiliate linked to the customer.
		$affiliate_id = slicewp_get_customer_meta( $data['customer_id'], 'affiliate_id', true );

		// No affiliate linked to the customer.
		if ( empty( $affiliate_id ) ) {
			return $data;
		}

		// Check if the affiliate already has a commission from the customer.
		if ( ! $this->affiliate_has_customer_commission( $affiliate_id, $data['customer_id'] ) ) {
			return $data;
		}

		// Check if affiliate lifetime limit has been reached.
		if ( $this->lifetime_limit_reached( $affiliate_id, $data['customer_id'] ) ) {
			return $data;
		}

		// Get affiliate lifetime commissions status.
		$lifetime_commissions = slicewp_get_affiliate_meta( $affiliate_id, 'lifetime_commissions', true );

		// Lifetime commissions are disabled for affiliate.
		if ( 'disabled' == $lifetime_commissions ) {
			return $data;
		}

		// Return if lifetime commissions are disabled for the group the affiliate is in.
		$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

		$group_lifetime_commissions = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_lifetime_commissions', true ) : '' );

		if ( 'disabled' == $group_lifetime_commissions ) {
			return $data;
		}

		// Affiliate lifetime commissions are set to site default
		// and the global lifetime commissions are disabled.
		if ( empty( $lifetime_commissions ) && empty( $group_lifetime_commissions ) && empty( slicewp_get_setting( 'lifetime_commissions' ) ) ) {
			return $data;
		}

		$rate_type = '';
		$rate      = '';

		// Get the affiliate custom lifetime commission rates.
		if ( 'custom' == $lifetime_commissions ) {

			$rate      = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_lifetime_sale', true );
			$rate_type = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_type_lifetime_sale', true );

		}

		// Get the affiliate group custom lifetime commission rates.
		if ( 'custom' == $group_lifetime_commissions && ( empty( $rate_type ) || ! is_numeric( $rate ) ) ) {

			$rate      = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_lifetime_sale', true );
			$rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_lifetime_sale', true );

		}

		// Get default lifetime rate and rate type.
		if ( empty( $rate_type ) || ! is_numeric( $rate ) ) {

			$rate      = slicewp_get_setting( 'commission_rate_lifetime_sale' );
			$rate_type = slicewp_get_setting( 'commission_rate_type_lifetime_sale' );

		}

		// No lifetime rates found.
		if ( empty( $rate_type ) || ! is_numeric( $rate ) ) {
			return $data;
		}

		// Set the commission type to lifetime_sale.
		$data['type'] = 'lifetime_sale';

		return $data;

	}

}