<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Registers the "lifetime" commission type.
 *
 * @param array $commission_types
 *
 * @return array
 *
 */
function slicewp_ltc_register_commission_type_lifetime( $commission_types ) {

	$commission_types['lifetime_sale'] = array(
		'label' 	 => __( 'Sale (lifetime)', 'slicewp' ),
		'rate_types' => array( 'percentage', 'fixed_amount' )
	);

	return $commission_types;

}
add_filter( 'slicewp_register_commission_types', 'slicewp_ltc_register_commission_type_lifetime' );


/**
 * Returns all the customers linked to an affiliate.
 * 
 * @param int $affiliate_id
 * 
 * @return array
 *
 */
function slicewp_ltc_get_affiliate_linked_customers( $affiliate_id ) {

	global $wpdb;

	// Get all the customers linked to the affiliate
	$customer_ids = $wpdb->get_col( $wpdb->prepare( "SELECT slicewp_customer_id FROM {$wpdb->slicewp_customermeta} WHERE meta_key = 'affiliate_id' AND meta_value = %d ORDER BY meta_id ASC;", absint( $affiliate_id ) ) );

	$customers = array();

	foreach ( $customer_ids as $customer_id ) {

		$customer = slicewp_get_customer( absint( $customer_id ) );

		if ( ! is_null( $customer ) ) {
			$customers[] = $customer;
		}

	}
	
	return $customers;

}


/**
 * Updates the affiliate_id for the customers that have been linked on the add new affiliate page.
 *
 * @param int $affiliate_id - the id of the affiliate
 *
 */
function slicewp_insert_affiliate_update_customers_affiliate_id( $affiliate_id ) {

	if ( ! slicewp_verify_request_action( 'add_affiliate' ) ) {
		return;
	}

	if ( empty( $_POST['customer_ids'] ) ) {
		return;
	}

	// Sanitize notes.
	$customer_ids = array_map( 'absint', explode( ',', $_POST['customer_ids'] ) );

	// Update each customer with the linked affiliate_id.
	foreach ( $customer_ids as $customer_id ) {

		if ( empty( $customer_id ) ) {
			continue;
		}

		$linked_affiliate = slicewp_get_customer_meta( $customer_id, 'affiliate_id', true );

		if ( $linked_affiliate == 0 ) {
			$customer_meta = slicewp_update_customer_meta( $customer_id, 'affiliate_id', absint( $affiliate_id ) );
		}

	}

}
add_action( 'slicewp_insert_affiliate', 'slicewp_insert_affiliate_update_customers_affiliate_id' );


/**
 * Links the customer to affiliate on purchase.
 *
 * @param int	$commission_id
 * @param array $commission_data
 * 
 */
function slicewp_ltc_link_customer_on_purchase( $commission_id, $commission_data ) {

	if ( empty( $commission_data['customer_id'] ) ) {
		return;
	}

	$lifetime_commissions = slicewp_get_affiliate_meta( $commission_data['affiliate_id'], 'lifetime_commissions', true );

	// Check if the affiliate can have lifetime commissions.
	if ( 'disabled' == $lifetime_commissions ) {
		return;
	}

	// Check if lifetime commissions are disabled sitewide.
	if ( empty( $lifetime_commissions ) && empty( slicewp_get_setting( 'lifetime_commissions' ) ) ) {
		return;
	}

	// Check if the customer is already linked to an affiliate.
	$linked_affiliate = slicewp_get_customer_meta( $commission_data['customer_id'], 'affiliate_id', true );

	// Link the customer to the affiliate.
	if ( empty( $linked_affiliate ) ) {

		$new_linked_affiliate = slicewp_add_customer_meta( $commission_data['customer_id'], 'affiliate_id', $commission_data['affiliate_id'] );

		// If affiliate was successfully added to the customer meta, the two are linked
		if ( $new_linked_affiliate ) {

			slicewp_add_log( sprintf( 'Customer #%s has been successfully linked to affiliate #%s at purchase.', $commission_data['customer_id'], $commission_data['affiliate_id'] ) );

		// If we couldn't add the customer meta in the table, something went wrong.
		} else {

			slicewp_add_log( sprintf( 'Customer #%s could not be linked at purchase to affiliate #%s due to an unexpected error.', $commission_data['customer_id'], $commission_data['affiliate_id'] ) );

		}

	} else {

		// Customer is already linked to another affiliate.
		if ( $linked_affiliate != $commission_data['affiliate_id'] ) {
			slicewp_add_log( sprintf( 'Customer #%s has not been not linked at purchase to affiliate #%s. Customer is already linked to affiliate #%s.', $commission_data['customer_id'], $commission_data['affiliate_id'], $linked_affiliate ) );
		}

	}

}
add_action( 'slicewp_insert_commission', 'slicewp_ltc_link_customer_on_purchase', 20, 2 );


/**
 * Links the customer to affiliate on user registration.
 *
 * @param int $user_id
 * 
 */
function slicewp_ltc_link_customer_on_registration( $user_id ) {

	// Check if the link customer on registration setting is enabled.
	if ( empty( slicewp_get_setting( 'link_customer_on_registration' ) ) ) {
		return;
	}

	// Check if customer is referred.
	$affiliate_id = slicewp_get_referrer_affiliate_id();

	if ( empty( $affiliate_id ) ) {
		return;
	}

	// Check if the affiliate can have lifetime commissions.
	if ( 'disabled' ==  slicewp_get_affiliate_meta( $affiliate_id, 'lifetime_commissions', true ) ) {
		return;
	}

	// Get user.
	$user = get_userdata( $user_id );

	// Prepare customer args.
	$customer_args = array(
		'email'   	 => $user->get( 'user_email' ),
		'user_id' 	 => $user_id,
		'first_name' => $user->get( 'first_name' ),
		'last_name'  => $user->get( 'last_name' )
	);

	// Process the customer.
	$customer_id = slicewp_process_customer( $customer_args );

	// Check if the customer is already linked to an affiliate.
	$linked_affiliate = slicewp_get_customer_meta( $customer_id, 'affiliate_id', true );

	// Link the customer to the affiliate.
	if ( empty( $linked_affiliate ) ) {

		$new_linked_affiliate = slicewp_add_customer_meta( $customer_id, 'affiliate_id', $affiliate_id );

		// If affiliate was successfully added to the customer meta, the two are linked.
		if ( $new_linked_affiliate ) {

			slicewp_add_log( sprintf( 'Customer #%s has been successfully linked to affiliate #%s at registration.', $customer_id, $affiliate_id ) );

		// If we couldn't add the customer meta in the table, something went wrong.
		} else {

			slicewp_add_log( sprintf( 'Customer #%s could not be linked at registration to affiliate #%s due to an unexpected error.', $customer_id, $affiliate_id ) );

		}

	} else {

		// Customer is already linked to another affiliate.
		if ( $linked_affiliate != $affiliate_id ) {
			slicewp_add_log( sprintf( 'Customer #%s has not been not linked at registration to affiliate #%s. Customer is already linked to affiliate #%s.', $customer_id, $affiliate_id, $linked_affiliate ) );
		}

	}

}
add_action( 'user_register', 'slicewp_ltc_link_customer_on_registration', 100 );


/**
 * Deletes the affiliate_id from the customer meta table.
 *
 * @param int $affiliate_id - the id of the affiliate
 *
 */
function slicewp_ltc_unlink_customers_from_affiliate( $affiliate_id ) {

	// Get the customers linked to the deleted affiliate.
	$linked_customers = slicewp_ltc_get_affiliate_linked_customers( $affiliate_id );
	
	// Remove the affiliate from the customer meta table.
	foreach ( $linked_customers as $linked_customer ) {

		slicewp_delete_customer_meta( $linked_customer->get( 'id' ), 'affiliate_id', $affiliate_id );

	}

}
add_action( 'slicewp_delete_affiliate', 'slicewp_ltc_unlink_customers_from_affiliate' );


/**
 * Adds the lifetime commission rate in the affiliate account.
 *
 * @param array $affiliate_commission_rates
 * @param int	$affiliate_id
 * 
 * @return array
 * 
 */
function slicewp_ltc_update_affiliate_account_commission_rates( $affiliate_commission_rates, $affiliate_id ) {

	// Set the affiliate lifetime rate to the values from settings.
	$affiliate_commission_rates['lifetime_sale']['rate']	  = slicewp_get_setting( 'commission_rate_lifetime_sale' );
	$affiliate_commission_rates['lifetime_sale']['rate_type'] = slicewp_get_setting( 'commission_rate_type_lifetime_sale' );

	// Get the lifetime commissions settings for affiliate.
	$affiliate_lifetime_commissions = slicewp_get_affiliate_meta( $affiliate_id, 'lifetime_commissions', true );

	// Lifetime commissions are disabled for affiliate.
	if ( 'disabled' == $affiliate_lifetime_commissions ) {

		unset( $affiliate_commission_rates['lifetime_sale'] );
		return $affiliate_commission_rates;

	}

	// If the affiliate has a custom lifetime rate, set it and return it.
	if ( 'custom' == $affiliate_lifetime_commissions ) {

		// Get the affiliate custom lifetime rate and rate type.
		$rate 	   = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_lifetime_sale', true );
		$rate_type = slicewp_get_affiliate_meta( $affiliate_id, 'commission_rate_type_lifetime_sale', true );

		if ( is_numeric( $rate ) && ! empty( $rate_type ) ) {

			$affiliate_commission_rates['lifetime_sale']['rate']	  = $rate;
			$affiliate_commission_rates['lifetime_sale']['rate_type'] = $rate_type;

			return $affiliate_commission_rates;

		}

	}

	// Check for affiliate graoup.
	$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

	if ( ! empty( $affiliate_group_id ) ) {
	
		// If there are no affiliate lifetime commissions settings, check the ones on the group the affiliate is a part of.
		$group_lifetime_commissions = slicewp_get_collection_meta( $affiliate_group_id, '_lifetime_commissions', true );

		// If lifetime commissions are disabled for the group, unset lifetime rates and return.
		if ( $group_lifetime_commissions == 'disabled' ) {

			if ( isset( $affiliate_commission_rates['lifetime_sale'] ) ) {
				unset( $affiliate_commission_rates['lifetime_sale'] );
			}
			
			return $affiliate_commission_rates;

		}

		// If there are custom rates set for the group, set them and return them.
		if ( $group_lifetime_commissions == 'custom' ) {

			$rate      = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_lifetime_sale', true );
			$rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_lifetime_sale', true );

			// Save the custom rate and rate type.
			if ( is_numeric( $rate ) && ! empty( $rate_type ) ) {

				$affiliate_commission_rates['lifetime_sale']['rate']	  = $rate;
				$affiliate_commission_rates['lifetime_sale']['rate_type'] = $rate_type;

				return $affiliate_commission_rates;

			}

		}

	}

	// If lifetime commissions are disabled sitewide and the group and affiliate ones are site default, unset the lifetime rates.
	if ( empty( slicewp_get_setting( 'lifetime_commissions' ) ) && empty( $group_lifetime_commissions ) && empty( $affiliate_lifetime_commissions ) ) {

		if ( isset( $affiliate_commission_rates['lifetime_sale'] ) ) {
			unset( $affiliate_commission_rates['lifetime_sale'] );
		}

	}

	// Return the filtered value.
	return $affiliate_commission_rates;

}
add_filter( 'slicewp_affiliate_commission_rates', 'slicewp_ltc_update_affiliate_account_commission_rates', 100, 2 );