<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Lifetime commissions MemberPress integration class.
 *
 */
class SliceWP_Integration_Lifetime_Commissions_MemberPress extends SliceWP_Integration_Lifetime_Commissions {

	/**
	 * Constructor.
	 *
	 */
	public function __construct() {

		$this->origin = 'mepr';

		// Add the "lifetime" commission type to MemberPress supports array.
		add_filter( 'slicewp_integration_supports_mepr', array( $this, 'add_integration_supports' ) );

        // Replaces the affiliate with the lifetime affiliate.
		add_filter( 'slicewp_referrer_affiliate_id_mepr', array( $this, 'get_lifetime_affiliate' ), 20, 2 );

		parent::__construct();

	}


	/**
	 * Add the "lifetime_sale" commission type to MemberPress supports array.
	 *
	 * @param array $supports
	 *
	 * @return array
	 *
	 */
	public function add_integration_supports( $supports ) {

		if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
			return $supports;
		}

		$supports['commission_types'][] = 'lifetime_sale';

		return $supports;

	}


	/**
	 * Replaces the affiliate with the lifetime affiliate
	 *
	 * @param int $affiliate_id   - the ID of the affiliate
	 * @param int $transaction_id - MemberPress transaction ID
	 * 
	 * @return int
	 *
	 */
	public function get_lifetime_affiliate( $affiliate_id, $transaction_id ) {

		// Get the transaction.
		$transaction = new MeprTransaction( $transaction_id );

        if ( empty( $transaction->id ) ) {
            return $affiliate_id;
        }

		// Get the customer.
		$customer = slicewp_get_customer_by_user_id( $transaction->user_id );

		// Verify if customer exists.
		if ( empty( $customer ) ) {
			return $affiliate_id;
		}

		// Search for customer's lifetime affiliate.
		$lifetime_affiliate_id = slicewp_get_customer_meta( $customer->get( 'id' ), 'affiliate_id', true );

		// No lifetime affiliate found.
		if ( empty( $lifetime_affiliate_id ) ) {
			return $affiliate_id;
		}

		// Get the affiliate lifetime settings.
		$lifetime_commissions = slicewp_get_affiliate_meta( $lifetime_affiliate_id, 'lifetime_commissions', true );

		// Affiliate lifetime commissions are disabled.
		if ( 'disabled' == $lifetime_commissions ) {
			return $affiliate_id;
		}

		// Return if affiliate group lifetime commissions are disabled.
		$affiliate_group_id = slicewp_get_affiliate_group_id( $lifetime_affiliate_id );

		$group_lifetime_commissions = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_lifetime_commissions', true ) : '' );

		if ( 'disabled' == $group_lifetime_commissions ) {
			return $affiliate_id;
		}

		// Affiliate and affiliate group lifetime commissions are set to site default
		// and the global lifetime commissions are disabled.
		if ( empty( $lifetime_commissions ) && empty( $group_lifetime_commissions ) && empty( slicewp_get_setting( 'lifetime_commissions' ) ) ) {
			return $affiliate_id;
		}

		// Lifetime commissions are enabled,
		// check if the lifetime limit has been reached.
		if ( $this->lifetime_limit_reached( $lifetime_affiliate_id, $customer->get( 'id' ) ) ) {
			return $affiliate_id;
		}

		// Return the lifetime affiliate
		return $lifetime_affiliate_id;

	}

}

new SliceWP_Integration_Lifetime_Commissions_MemberPress();