<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Mailchimp API Class.
 *
 */
class SliceWP_Mailchimp_API {

	/**
	 * API Key
	 *
	 */
	private $api_key;

	/**
	 * API instance
	 *
	 */
	private $api;


	/**
	 * Constructor
	 * 
	 */
	public function __construct() {

		// Include the MailChimp API lib.
		include_once 'lib/MailChimp.php';

	}


	/**
	 * Init the Mailchimp API
	 * 
	 * @param string $api_key
	 * 
	 */
	public function init( $api_key ) {

		if ( empty( $api_key ) ) {
			return false;
		}

		$this->api = new \SliceWP\MailChimp\MailChimp( $api_key );

		$this->api->verify_ssl = false;

		// Set the API key
		$this->api_key = $api_key;

		// Check if Mailchimp connection works
		return $this->ping() ? true : false;

	}


	/**
	 * Pings the Mailchimp API
	 * 
	 * @return bool
	 * 
	 */
	public function ping() {

		if ( is_null( $this->api ) ) {
			return false;
		}

		$ping = $this->api->get( 'ping' );

		if ( empty( $ping['health_status'] ) ) {
			return false;
		}
		
		return true;

	}


	/**
	 * Gets the Mailchimp audiences (lists)
	 * 
	 * @return array
	 * 
	 */
	public function get_audiences() {

		// Get the Mailchimp audiences
		$data = $this->api->get( 'lists', array( 'count' => 1000 ) );

		if ( ! empty( $data['lists'] ) ) {

			foreach ( $data['lists'] as $key => $audience ) {

				$audiences[$audience['id']] = $audience['name'];

			}

		}

		return $audiences;

	}


	/**
	 * Gets the Mailchimp tags for all the audiences
	 * 
	 * @param array $audiences
	 * 
	 * @return array
	 * 
	 */
	public function get_tags( $audiences ) {

		if ( empty( $audiences ) ) {
			return;
		}

		foreach ( $audiences as $audience_id => $audience_name ) {

			// Get the tags
			$data = $this->api->get( 'lists/' . $audience_id . '/segments', array( 'type' => 'static', 'count' => 1000 ) );
			
			if ( ! empty( $data['segments'] ) && ! empty( $data['segments'] ) ) {
				
				foreach ( $data['segments'] as $key => $tag ) {
					
					// Save the tag
					$audience_tags[$tag['id']] = $tag['name'];

				}
				
				$tags[$audience_id] = $audience_tags;

			}

		}

		return $tags;

	}


	/**
	 * Gets the Mailchimp API data and saves it in settings
	 * 
	 */
	public function save_api_data() {

		$api_data = array();

		// Get the audiences
		$api_data['audiences']	= $this->get_audiences();
		$api_data['tags']		= $this->get_tags( $api_data['audiences'] );

		// Update the API data
		slicewp_update_option( 'mailchimp_api_data', $api_data );

		// Save the API key in settings
		$settings = slicewp_get_option( 'settings', array() );
		$settings['mailchimp_api_key'] = $this->api_key;

		// Update general settings
		slicewp_update_option( 'settings', $settings );

	}


	/**
	 * Outputs the MailChimp audiences
	 * 
	 */
	public function output_audiences() {

		// Get fields data
		$api_data = slicewp_get_option( 'mailchimp_api_data' );

		// Nothing to output
		if ( empty( $api_data['audiences'] ) ) {
			return;
		}

		$audiences = array( '' => __( 'Select audience...', 'slicewp' ) );
		$audiences = $audiences + $api_data['audiences'];

		?>

		<!-- Audiences -->
		<div id="slicewp-mailchimp-audience" class="slicewp-field-wrapper slicewp-field-wrapper-inline">
			
			<div class="slicewp-field-label-wrapper">
				<label for="slicewp-mailchimp-audience">
					<?php echo __( 'Audience', 'slicewp' ); ?>
				</label>
			</div>

			<select id="slicewp-mailchimp-select-audience" name="settings[mailchimp_audience]" class="slicewp-select2">
				<?php foreach ( $audiences as $audience_id => $audience_name ): ?>
					<option value="<?php echo esc_attr( $audience_id ); ?>" <?php selected( $audience_id, slicewp_get_setting( 'mailchimp_audience' ) ); ?>><?php echo( $audience_name ); ?></option>
				<?php endforeach; ?>
			</select>
		
		</div><!-- / Audiences -->

	<?php

	}


	/**
	 * Outputs the Mailchimp tags
	 * 
	 */
	public function output_tags( $audience_id = 0 ) {
		
		if ( empty( $audience_id ) ) {
			return;
		}

		// Get fields data
		$api_data = slicewp_get_option( 'mailchimp_api_data' );

		// Nothing to output
		if ( empty( $api_data['tags'][$audience_id] ) ) {
			return;
		}

		$tags = $api_data['tags'][$audience_id];

		// Get the selected tags
		$selected_tags = slicewp_get_setting( 'mailchimp_tags', array() );

		?>

		<!-- Tags -->
		<div id="slicewp-mailchimp-tags" class="slicewp-field-wrapper slicewp-field-wrapper-inline">

			<div class="slicewp-field-label-wrapper">
				<label for="slicewp-mailchimp-tags">
					<?php echo __( 'Tags', 'slicewp' ); ?>
				</label>
			</div>

			<select id="slicewp-mailchimp-tags-select" name="settings[mailchimp_tags][]" class="slicewp-select2-mailchimp-tags" multiple placeholder="<?php echo __( 'Select tags...', 'slicewp' ); ?>">

				<?php

					if ( ! empty( $tags ) ) {

						foreach ( $tags as $key => $tag ) {
							
							echo '<option value="' . $key . '"' . selected( 1, in_array( $key, $selected_tags )  ) . '>' . $tag . '</option>';

						}

					}

				?>

			</select>

		</div><!-- / Tags -->

	<?php

	}


	/**
	 * Adds the user in Mailchimp.
	 * 
	 * @param array $data
	 * 
	 */
	public function add_subscriber( $data ) {

		// Check for email.
		if ( empty( $data['email'] ) ) {
			return;
		}

		// Get Mailchimp audience.
		$audience_id = slicewp_get_setting( 'mailchimp_audience' );

		if ( empty( $audience_id ) ) {
			return;
		}

		// Get all the tags for the selected audience.
		$api_data 	   = slicewp_get_option( 'mailchimp_api_data', array() );
		$audience_tags = ( ! empty( $api_data['tags'][$audience_id] ) ? $api_data['tags'][$audience_id] : array() );

		// Get the tags saved in settings.
		$selected_tags = slicewp_get_setting( 'mailchimp_tags', array() );

		$tag_names = array();

		// Check if we have any tags set.
		if ( ! empty( $audience_tags ) && ! empty( $selected_tags ) ) {

			// Save tag names
			foreach ( $selected_tags as $tag_id ) {

				if ( isset( $audience_tags[$tag_id] ) ) {

					$tag_names[] = $audience_tags[$tag_id];
					
				}

			}

		}

		// Prepare the subscriber data.
		$subscriber = array(
			'email_address'	=> sanitize_email( $data['email'] ),
			'status'		=> ! empty( slicewp_get_setting( 'mailchimp_double_optin' ) ) ? 'pending' : 'subscribed',
			'merge_fields'	=> array(
							'FNAME' => ( ! empty ( $data['first_name'] ) ? sanitize_text_field( $data['first_name'] ) : '' ),
							'LNAME' => ( ! empty ( $data['last_name'] ) ? sanitize_text_field( $data['last_name'] ) : '' )
			),
			'tags'			=> $tag_names
		);

		// Add the subscriber.
		$result = $this->api->post( 'lists/' . $audience_id . '/members', $subscriber );

		// Check if Mailchimp response is successful
		if ( ! $this->api->success() ) {

			slicewp_add_log( 'Mailchimp API: add_subscriber() failed. Error: ' . $this->api->getLastError() );

		}

	}

}


/**
 * Register the Mailchimp API as a service.
 *
 * @param array
 *
 * @return array
 *
 */
function slicewp_register_service_mailchimp_api( $services ) {

	$services['mailchimp'] = new SliceWP_Mailchimp_API();

	return $services;

}
add_filter( 'slicewp_register_service', 'slicewp_register_service_mailchimp_api' );