<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * MailerLite API class.
 *
 */
class SliceWP_MailerLite_API {
	
	/**
	 * API Key
	 *
	 */
	private $api_key;

	/**
	 * The arguments used for API requests
	 *
	 */
	private $request_args;

	/**
	 * Last API error
	 *
	 */
	private $last_error;


	/**
	 * Constructor
	 *
	 */
	public function __construct() {}


	/**
	 * Init the MailerLite API
	 * 
	 * @param string $api_key
	 * 
	 */
	public function init( $api_key ) {

		// Check for API key
		if ( empty( $api_key ) ) {
			return false;
		}

		// Set the API key
		$this->api_key = $api_key;

		// Set the request args
		$this->request_args = array(
			'user-agent'	=> 'SliceWP; ' . home_url(),
			'timeout'		=> 30,
			'headers'		=> array(
				'X-MailerLite-ApiKey'	=> $this->api_key,
				'Content-Type'			=> 'application/json',
			),
		);

		// Check if MailerLite connection works
		return $this->ping() ? true : false;

	}


	/**
	 * Pings the MailerLite API
	 * 
	 * @return bool
	 * 
	 */
	public function ping() {

		// Check for API request args
		if ( ! $this->request_args ) {
			return false;
		}

		// Get the groups
		$request_url = 'https://api.mailerlite.com/api/v2/groups';
		$response = wp_safe_remote_get( $request_url, $this->request_args );

		// Check for WP Error
		if ( is_wp_error( $response ) ) {

			$this->last_error = $response->get_error_message();
			return false;

		}

		$response_body = json_decode( wp_remote_retrieve_body( $response ) );

		// Check for errors
		if ( isset( $response_body->error ) ) {

			$this->last_error = $response_body->error->message;
			return false;

		}

		return true;

	}


	/**
	 * Gets the MailerLite groups
	 * 
	 * @return array|null
	 * 
	 */
	public function get_groups() {

		// Check for API request args
		if ( ! $this->request_args ) {
			return null;
		}

		// Get the groups
		$request_url = 'https://api.mailerlite.com/api/v2/groups';
		$response = wp_safe_remote_get( $request_url, $this->request_args );

		// Check for WP Error
		if ( is_wp_error( $response ) ) {

			$this->last_error = $response->get_error_message();
			return null;
		
		}

		$groups_array = json_decode( wp_remote_retrieve_body( $response ) );

		// Check for errors
		if ( isset( $groups_array->error ) ) {

			$this->last_error = $groups_array->error->message;
			return null;

		}

		$groups = array();

		// We don't have any groups
		if ( empty( $groups_array ) ) {
			return $groups;
		}

		foreach ( $groups_array as $key => $group ) {

			$groups[ $group->id ] = $group->name;

		}

		return $groups;

	}


	/**
	 * Gets the MailerLite API data and saves it in settings
	 * 
	 */
	public function save_api_data() {

		// Get the saved API data
		$api_data = slicewp_get_option( 'mailerlite_api_data', array() );

		// Get the groups from MailerLite
		$api_groups = $this->get_groups();

		// Save the groups
		if ( ! is_null( $api_groups ) ) {
			$api_data['groups'] = $api_groups;
		}

		// Update the API data
		slicewp_update_option( 'mailerlite_api_data', $api_data );

		// Save the API key in settings
		$settings = slicewp_get_option( 'settings', array() );
		$settings['mailerlite_api_key'] = $this->api_key;

		// Update general settings
		slicewp_update_option( 'settings', $settings );

	}


	/**
	 * Outputs the MailerLite groups
	 * 
	 */
	public function output_groups() {

		// Get fields data
		$api_data = slicewp_get_option( 'mailerlite_api_data', array() );

		// Nothing to output
		if ( empty( $api_data['groups'] ) ) {
			return;
		}

		$groups = array( '' => __( 'Select group...', 'slicewp' ) );
		$groups = $groups + $api_data['groups'];

	?>

		<!-- Groups -->
		<div id="slicewp-mailerlite-group" class="slicewp-field-wrapper slicewp-field-wrapper-inline">
			
			<div class="slicewp-field-label-wrapper">
				<label for="slicewp-mailerlite-group">
					<?php echo __( 'Group', 'slicewp' ); ?>
				</label>
			</div>

			<select id="slicewp-mailerlite-select-group" name="settings[mailerlite_group]" class="slicewp-select2">
				<?php foreach ( $groups as $group_id => $group_name ): ?>
					<option value="<?php echo esc_attr( $group_id ); ?>" <?php selected( $group_id, slicewp_get_setting( 'mailerlite_group' ) ); ?>><?php echo( $group_name ); ?></option>
				<?php endforeach; ?>
			</select>
		
		</div><!-- / Groups -->
	
	<?php

	}


	/**
	 * Adds the user in MailerLite
	 * 
	 * @param array $data
	 * 
	 */
	public function add_subscriber( $data ) {

		// Check for email
		if ( empty( $data['email'] ) ) {
			return;
		}

		// Check for API request args
		if ( ! $this->request_args ) {
			return;
		}

		// Prepare the subscriber data
		$subscriber = array(
			'email'		=> sanitize_email( $data['email'] ),
			'name'		=> ! empty ( $data['first_name'] ) ? sanitize_text_field( $data['first_name'] ) : '',
			'fields'	=> array(
							'last_name'	=> ! empty ( $data['last_name'] ) ? sanitize_text_field( $data['last_name'] ) : ''
			)
		);

		// Get the saved MailerLite group
		$group_id = slicewp_get_setting( 'mailerlite_group' );

		// Prepare request args
		if ( ! empty( $group_id ) ) {
			$request_url = 'https://api.mailerlite.com/api/v2/groups/' . $group_id . '/subscribers';
		} else {
			$request_url = 'https://api.mailerlite.com/api/v2/subscribers';
		}

		$request_args			= $this->request_args;
		$request_args['method']	= 'POST';
		$request_args['body']	= json_encode( $subscriber );

		// Add the subscriber
		$response = wp_safe_remote_post( $request_url, $request_args );

		// Check for WP Error
		if ( is_wp_error( $response ) ) {

			slicewp_add_log( 'MailerLite API: add_subscriber() failed. Error: ' . $response->get_error_message() );
			return;

		}

		$subscriber = json_decode( wp_remote_retrieve_body( $response ) );

		// Check for errors
		if ( isset( $subscriber->error ) ) {

			slicewp_add_log( 'MailerLite API: add_subscriber() failed. Error: ' . $subscriber->error->message );

		}

	}


	/**
	 * Returns the last error
	 * 
	 * @return array
	 * 
	 */
	public function get_last_error() {

		return $this->last_error;

	}

}


/**
 * Register the MailerLite API as a service
 *
 * @param array
 *
 * @return array
 *
 */
function slicewp_register_service_mailerlite_api( $services ) {

	$services['mailerlite'] = new SliceWP_MailerLite_API();

	return $services;

}
add_filter( 'slicewp_register_service', 'slicewp_register_service_mailerlite_api' );