<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Adds the settings card for the affiliate multi-level functionality
 * in the plugin's main settings page.
 * 
 */
function slicewp_add_multi_level_general_settings() {

    include_once 'views/view-multi-level-general-settings.php';

}
add_action( 'slicewp_view_settings_tab_general_bottom', 'slicewp_add_multi_level_general_settings' );



/**
 * Include the affiliate parent field in the admin affiliate add/edit/review forms.
 * 
 * @param string $form
 * 
 */
function slicewp_affiliate_form_field_affiliate_parent_id( $form ) {

    if ( ! in_array( $form, array( 'add_affiliate', 'edit_affiliate', 'review_affiliate' ) ) ) {
        return;
    }

    include_once 'views/form-field-affiliate-parent.php';

}
add_action( 'slicewp_admin_form_fields', 'slicewp_affiliate_form_field_affiliate_parent_id' );


/**
 * Filter the multi-level settings options before saving them in the DB.
 * 
 * @param array $settings
 * 
 * @return array
 * 
 */
function slicewp_multi_level_pre_update_option_settings( $settings ) {

    if ( empty( $settings['affiliate_levels'] ) ) {
        return $settings;
    }

    $settings['affiliate_levels'] = array_values( $settings['affiliate_levels'] );

    return $settings;

}
add_filter( 'slicewp_pre_update_option_settings', 'slicewp_multi_level_pre_update_option_settings' );


/**
 * Validates the parent affiliate field on the admin add/update affiliate actions.
 * 
 */
function slicewp_add_update_affiliate_validate_field_affiliate_parent() {

    if ( ! empty( $_POST['parent_id'] ) ) {

        // Verify if the parent affiliate exists.
        $parent_affiliate = slicewp_get_affiliate( absint( $_POST['parent_id'] ) );

        if ( is_null( $parent_affiliate ) ) {

            slicewp_form_errors()->add( 'parent_id', __( 'The selected parent affiliate does not exist.', 'slicewp' ) );
            return;

        }

        // Verify if the parent affiliate is the current affiliate.
        if ( isset( $_POST['affiliate_id'] ) && absint( $_POST['affiliate_id'] ) == absint( $_POST['parent_id'] ) ) {

            slicewp_form_errors()->add( 'parent_id', __( 'You cannot set the current affiliate as the parent affiliate.', 'slicewp' ) );
            return;

        }

        // Verify that the parent affiliate isn't a sub-child of the current affiliate.
        $parent_ancestors     = slicewp_get_affiliate_ancestors( absint( $_POST['parent_id'] ) );
        $parent_ancestors_ids = array_column( array_map( 'slicewp_object_to_array', $parent_ancestors ), 'id' );

        if ( isset( $_POST['affiliate_id'] ) && in_array( absint( $_POST['affiliate_id'] ), $parent_ancestors_ids ) ) {

            slicewp_form_errors()->add( 'parent_id', __( 'The selected parent affiliate cannot be set because it is a sub-affiliate of the current affiliate.', 'slicewp' ) );
            return;

        }

    }

}
add_action( 'slicewp_admin_action_add_affiliate', 'slicewp_add_update_affiliate_validate_field_affiliate_parent', 25 );
add_action( 'slicewp_admin_action_update_affiliate', 'slicewp_add_update_affiliate_validate_field_affiliate_parent', 25 );


/**
 * Set the "parent_id" when adding or updating an affiliate from the admin area.
 * 
 * @param array $data
 * 
 * @return array
 * 
 */
function slicewp_pre_modify_affiliate_data_parent_id( $data ) {

    // Bail if the action isn't "add_affiliate" or "update_affiliate"
    if ( ! slicewp_verify_request_action( 'add_affiliate' ) && ! slicewp_verify_request_action( 'update_affiliate' ) ) {
        return $data;
    }

    $data['parent_id'] = ( ! empty( $_POST['parent_id'] ) ? absint( $_POST['parent_id'] ) : 0 );

    return $data;

}
add_filter( 'slicewp_pre_insert_affiliate_data', 'slicewp_pre_modify_affiliate_data_parent_id' );
add_filter( 'slicewp_pre_update_affiliate_data', 'slicewp_pre_modify_affiliate_data_parent_id' );


/**
 * Adds the navigation tab and "network"  tab on the edit affiliate sub page.
 * 
 * @param string $subpage
 * 
 */
function slicewp_submenu_page_output_affiliates_affiliate_network( $subpage ) {

    if ( 'edit-affiliate' != $subpage ) {
        return;
    }

    include_once 'views/view-affiliate-network.php';

}
add_action( 'slicewp_submenu_page_output_affiliates', 'slicewp_submenu_page_output_affiliates_affiliate_network' );


/**
 * Register the "Parent" column in the "Affiliates" admin list table.
 * 
 * @param array $columns
 * 
 * @return array
 * 
 */
function slicewp_list_table_affiliates_columns_register_parent_affiliate( $columns ) {

	$columns = _slicewp_array_assoc_push_after_key( $columns, 'name', array( 'parent' => __( 'Parent', 'slicewp' ) ) );

	return $columns;

}
add_filter( 'slicewp_list_table_affiliates_columns', 'slicewp_list_table_affiliates_columns_register_parent_affiliate', 15 );


/**
 * Output for the "Parent" column in the "Affiliates" admin list table.
 * 
 * @param string $output
 * @param array  $item
 * 
 * @return string
 * 
 */
function slicewp_list_table_affiliates_column_parent_output( $output, $item ) {

    if ( empty( $item['parent_id'] ) ) {
        return $output;
    }

    $affiliate = slicewp_get_affiliate( absint( $item['parent_id'] ) );

    if ( is_null( $affiliate ) ) {
        return $output;
    }

    $output = '<a class="slicewp-affiliate-name" href="' . esc_url( add_query_arg( array( 'page' => 'slicewp-affiliates', 'subpage' => 'edit-affiliate', 'affiliate_id' => absint( $item['parent_id'] ) ) , admin_url( 'admin.php' ) ) ) . '">';
        $output .= get_avatar( absint( $affiliate->get( 'user_id' ) ), 64 );
        $output .= '<span>' . esc_html( slicewp_get_affiliate_name( $affiliate ) ) . '</span>';
    $output .= '</a>';

	return $output;

}
add_filter( 'slicewp_list_table_affiliates_column_parent', 'slicewp_list_table_affiliates_column_parent_output', 15, 2 );


/**
 * Adds the "parent" selector filter to the affiliates admin list table.
 * 
 */
function slicewp_list_table_affiliates_data_filters_output_affiliate_parent_selector() {

	echo '<div class="slicewp-list-table-data-filter slicewp-list-table-data-filter-user-search">';

		echo '<label for="slicewp-list-table-data-filter-affiliate-parent" class="screen-reader-text">' . __( 'Affiliate Parent', 'slicewp' ) . '</label>';

        slicewp_output_select2_user_search( array( 'id' => 'slicewp-list-table-data-filter-affiliate-parent', 'data_attributes' => array( 'container-class' => 'slicewp-select2-small' ), 'name' => 'affiliate_parent_id', 'placeholder' => __( 'Select affiliate parent...', 'slicewp' ), 'user_type' => 'affiliate', 'value' => ( ! empty( $_GET['affiliate_parent_id'] ) ? absint( $_GET['affiliate_parent_id'] ) : '' ) ) );

	echo '</div>';

}
add_action( 'slicewp_list_table_affiliates_data_filters_output', 'slicewp_list_table_affiliates_data_filters_output_affiliate_parent_selector', 20 );


/**
 * Appends the "parent_id" argument to the query args of the slicewp_get_affiliates queries from the "affiliates" page.
 * 
 * @param array $args
 * 
 * @return array
 * 
 */
function slicewp_get_affiliates_args_add_affiliate_parent_filter( $args ) {

	if ( empty( $_GET['page'] ) || $_GET['page'] !== 'slicewp-affiliates' ) {
		return $args;
	}

	if ( empty( $_GET['affiliate_parent_id'] ) ) {
		return $args;
	}

	if ( ! is_admin() ) {
		return $args;
	}

	global $wp_current_filter;

	if ( ! in_array( 'slicewp_page_slicewp-affiliates', $wp_current_filter ) ) {
		return $args;
	}

	$args['parent_id'] = absint( $_GET['affiliate_parent_id'] );

	return $args;

}
add_filter( 'slicewp_get_affiliates_args', 'slicewp_get_affiliates_args_add_affiliate_parent_filter' );