<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Builds and outputs the children tree diagram for the given affiliate.
 * 
 */
class SliceWP_Affiliates_Tree_Diagram {

    /**
     * The affiliate for which to build the tree diagram.
     * 
     * @var    SliceWP_Affiliate;
     * @access protected
     * 
     */
    protected $affiliate = null;

    /**
     * The affiliate tree map.
     * 
     * @var    array
     * @access protected
     * 
     */
    protected $tree_map = array();


    /**
     * Outputs the tree diagram for the given affiliate.
     * 
     * @param int|SliceWP_Affiliate $affiliate
     * 
     */
    public function output( $affiliate ) {

        $this->affiliate = slicewp_get_affiliate( $affiliate );

        if ( is_null( $this->affiliate ) ) {
            return;
        }

        $this->output_styles();

        self::build_tree_map( $this->affiliate );

        ?>

            <div class="slicewp-tree-diagram">

                <ul>

                    <li class="slicewp-tree-diagram-root">

                        <?php slicewp_get_template_part( 'affiliate-area/affiliates-tree-affiliate-card-root', null, array( 'affiliate' => $this->affiliate ) ); ?>

                        <?php self::output_children( $this->tree_map ); ?>

                    </li>

                </ul>

            </div>

        <?php

    }


    /**
     * Outputs the children of the given affiliate tree.
     * 
     * @param array $tree
     * 
     */
    private function output_children( $tree ) {

        if ( empty( $tree['children'] ) ) {
            return;
        }

        ?>

            <ul>
                
                <?php foreach ( $tree['children'] as $key => $item_data ): ?>
                    
                    <li>

                        <?php slicewp_get_template_part( 'affiliate-area/affiliates-tree-affiliate-card', null, array( 'affiliate' => $item_data['item'] ) ); ?>
                        <?php self::output_children( $tree['children'][$key] ); ?>

                    </li>

                <?php endforeach; ?>

            </ul>
        
        <?php

    }


    /**
     * Outputs the CSS needed to style the affiliate tree.
     * 
     */
    protected function output_styles() {

        ?>

            <style>

                .slicewp-tree-diagram { display: flex; overflow: scroll; padding-bottom: 25px; }

                .slicewp-tree-diagram > ul { margin: 0 auto !important; }
                .slicewp-tree-diagram ul { display: flex; position: relative; list-style-type: none; margin: 0; padding: 1.5rem 0.5rem 0rem 0.5rem; }
                .slicewp-tree-diagram ul ul::before { border-right: 1px solid #dce6ec; content: ''; position: absolute; top: 1px; right: 50%; height: 1.5rem; width: 50%; }

                .slicewp-tree-diagram li { padding: 1.5rem 0.5rem 0rem 0.5rem; position: relative; align-items: center; display: flex; flex-direction: column; }
                .slicewp-tree-diagram li::before { border-right: 1px solid #dce6ec; border-top: 1px solid #dce6ec; content: ''; position: absolute; top: 0; right: 50%; height: calc( 1.5rem - 2px ); width: 50%; }
                .slicewp-tree-diagram li::after { border-top: 1px solid #dce6ec; content: ''; position: absolute; top: 0; right: 0; width: 50%; }

                .slicewp-tree-diagram li:first-child::before,
                .slicewp-tree-diagram li:last-child::after { border-top: none; padding-top: 1px; }

                .slicewp-tree-diagram .slicewp-affiliate-card { padding: 12px; border-radius: 3px; margin: 0; box-shadow: 0 0 0 1px rgb( 200 215 225 / 50% ), 0 1px 2px #e9eff3; width: max-content; align-items: center; }

                .slicewp-tree-diagram .slicewp-affiliate-card-header { display: flex; align-items: center; }
                .slicewp-tree-diagram .slicewp-affiliate-card-header img.avatar { border-radius: 50%; vertical-align: middle; margin-right: 10px; }

                .slicewp-tree-diagram .slicewp-affiliate-card-body .slicewp-affiliate-card-stats { display: grid; grid-template-columns: repeat( 2, 1fr ); grid-gap: 4px; grid-column-gap: 12px; margin-top: 12px; }
                .slicewp-tree-diagram .slicewp-affiliate-card-body .slicewp-affiliate-card-stats :nth-child(even) { text-align: right; }

                .slicewp-tree-diagram-root > .slicewp-affiliate-card .slicewp-affiliate-card-name { font-weight: bold; }
                li.slicewp-tree-diagram-root::before { border-right: none; }

            </style>

        <?php

    }


    /**
     * Builds the relationship tree map for the given affiliate.
     * 
     * @param int|SliceWP_Affiliate $affiliate
     * 
     */
    private function build_tree_map( $affiliate ) {

        $affiliate = slicewp_get_affiliate( $affiliate );

        $this->tree_map = $this->build_item_tree_map( $affiliate, 0 );

    }


    /**
     * Builds the relationship tree map for the given item.
     * 
     * @param int|SliceWP_Affiliate $affiliate
     * @param int                   $depth
     * 
     * @return array
     * 
     */
    private function build_item_tree_map( $affiliate, $depth ) {

        $parent   = slicewp_get_affiliate( $affiliate );
        $children = slicewp_get_affiliate_children( $parent );

        $tree = array();

        $tree['item']     = $parent;
        $tree['children'] = array();

        /**
         * Filter the maximum depth the affiliate tree should have.
         * 
         * @param int
         * 
         */
        $display_depth = apply_filters( 'slicewp_affiliates_tree_display_depth', -1 );

        if ( is_numeric( $display_depth ) && $display_depth > 0 ) {

            if ( $depth >= $display_depth ) {
                return $tree;
            }

        }

        if ( ! empty( $children ) ) {

            $depth++;

            foreach ( $children as $child ) {

                $_children = $this->build_item_tree_map( $child, $depth );
    
                if ( empty( $_children ) ) {
                    continue;
                }
    
                $tree['children'][] = $_children;
    
            }
            
        }

        return $tree;

    }

}