<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * PayPal Payouts API class
 *
 * Handles API calls made to PayPal
 *
 */
class SliceWP_PayPal_Payouts_API {

	/**
	 * Stores the last error message returned by the API.
	 * 
	 * @access protected
	 * @var    string
	 * 
	 */
	protected $last_error = '';

	/**
	 * Whether the API is in test mode or not
	 *
	 * @access private
	 * @var array
	 *
	 */
	private $test_mode;

	/**
	 * Array holding the API credentials
	 *
	 * @access private
	 * @var array
	 *
	 */
	private $credentials;


	/**
	 * Construct
	 *
	 */
	public function __construct() {

		// Set test mode
		$this->test_mode = slicewp_get_setting( 'paypal_api_test_mode' );

		// Set credentials
		$this->credentials = array(
			'api_client_id_live' => slicewp_get_setting( 'paypal_api_client_id_live' ),
			'api_secret_live' 	 => slicewp_get_setting( 'paypal_api_secret_live' ),
			'api_client_id_test' => slicewp_get_setting( 'paypal_api_client_id_test' ),
			'api_secret_test' 	 => slicewp_get_setting( 'paypal_api_secret_test' )
		);

	}


	/**
	 * Returns the last API error.
	 * 
	 * @return string
	 * 
	 */
	public function get_last_error() {

		return $this->last_error;

	}


	/**
	 * Makes an API call to PayPal to generate an access token, then returns it
	 *
	 * @return null|string
	 *
	 */
	private function get_access_token() {

		$client_id = $this->get_api_client_id();
		$secret    = $this->get_api_secret();

		// If the API credentials are not set, add to the log and return null
		if ( empty( $client_id ) || empty( $secret ) ) {

			slicewp_add_log( 'PayPal Payouts API: get_access_token() failed. Credentials missing.' );
			return null;

		}

		// Set API url
		$api_url = 'https://api' . ( $this->is_test_mode() ? '.sandbox' : '' ) . '.paypal.com/v1/oauth2/token';

		// Prepare call arguments
		$args = array(
			'timeout'     => 45,
		    'redirection' => 10,
		    'httpversion' => '1.1',
		    'headers' => array(
		    	'Accept' 		  => 'application/json',
		    	'Accept-language' => 'en_US',
		    	'Authorization'   => 'Basic ' . base64_encode( $client_id . ':' . $secret ),
		    	'Content-type' 	  => 'application/x-www-form-urlencoded'
		    ),
		    'body' => array(
		    	'grant_type' => 'client_credentials'
		    )
		);

		// Make the call to the API
		$response = wp_remote_post( $api_url, $args );

		$code 	 = wp_remote_retrieve_response_code( $response );
		$message = wp_remote_retrieve_response_message( $response );

		if ( is_wp_error( $response ) ) {

			$this->last_error = $response->get_error_message();

			slicewp_add_log( 'PayPal Payouts API: get_access_token() failed. Error: ' . $response->get_error_message() );
			return null;

		}

		$body = json_decode( wp_remote_retrieve_body( $response ), true );

		// Check if response is okay
		if ( $code != 200 ) {

			$this->last_error = ( ! empty( $body['error'] ) && ! empty( $body['error_description'] ) ? $body['error'] . ' - ' . $body['error_description'] : $message );

			slicewp_add_log( 'PayPal Payouts API: get_access_token() failed. Response code: ' . $code . '. Error: ' . ( ! empty( $body['error'] ) && ! empty( $body['error_description'] ) ? $body['error'] . ' - ' . $body['error_description'] : $message ) );
			return null;

		}

		// Check if the access token is found in the response's body
		if ( ! is_array( $body ) || empty( $body['access_token'] ) ) {

			slicewp_add_log( 'PayPal Payouts API: get_access_token() failed. Access token is missing from the response body.' );
			return null;

		}

		// Return the token if all is good
		return $body['access_token'];

	}


	/**
	 * Makes an API call to PayPal to create payouts for the provided payments
	 *
	 * @param array $payments
	 *
	 * @return array|false
	 *
	 */
	public function create_payout( $payments ) {

		$client_id = $this->get_api_client_id();
		$secret    = $this->get_api_secret();

		// If the API credentials are not set, add to the log and return false
		if ( empty( $client_id ) || empty( $secret ) ) {

			slicewp_add_log( 'PayPal Payouts API: create_payout() failed. Credentials missing.' );
			return false;

		}

		// Get token
		$token = $this->get_access_token();

		if ( is_null( $token ) ) {

			slicewp_add_log( 'PayPal Payouts API: create_payout() failed. Token is invalid.' );
			return false;

		}

		// Parse payments and prepare them for payouts
		$items = array();

		foreach ( $payments as $payment ) {

			$affiliate = slicewp_get_affiliate( $payment->get( 'affiliate_id' ) );

			$items[] = array(
				'recipient_type' => 'EMAIL',
				'amount' => array(
    				'value'    => $this->sanitize_amount( $payment->get( 'amount' ) ),
    				'currency' => $payment->get( 'currency' )
    			),
    			'receiver' 		 => $affiliate->get( 'payment_email' ),
    			'sender_item_id' => 'slicewp-payment-' . absint( $payment->get( 'id' ) )
			);

		}

		// Set API url
		$api_url = 'https://api' . ( $this->is_test_mode() ? '.sandbox' : '' ) . '.paypal.com/v1/payments/payouts';

		// Prepare call arguments
		$args = array(
			'timeout'     => 45,
		    'redirection' => 10,
		    'httpversion' => '1.1',
		    'headers' => array(
		    	'Accept' 		  => 'application/json',
		    	'Authorization'   => 'Bearer ' . $token,
		    	'Content-type' 	  => 'application/json'
		    ),
		    'body' => json_encode( array(
		    	'sender_batch_header' => array(
		    		'email_subject'   => 'Affiliate Earnings Payment',
		    	),
		    	'items' => $items
		    ))
		);

		// Make the call to the API
		$response = wp_remote_post( $api_url, $args );

		$code 	 = wp_remote_retrieve_response_code( $response );
		$message = wp_remote_retrieve_response_message( $response );

		if ( is_wp_error( $response ) ) {

			$this->last_error = $response->get_error_message();

			slicewp_add_log( 'PayPal Payouts API: create_payout() failed. Error: ' . $response->get_error_message() );
			return false;

		}

		$body = json_decode( wp_remote_retrieve_body( $response ), true );

		// Check if response is okay
		if ( $code != 201 ) {

			$this->last_error = ( ! empty( $body['debug_id'] ) && ! empty( $body['name'] && ! empty( $body['message'] ) ) ? $body['name'] . ' - ' . $body['message'] : $message );

			slicewp_add_log( 'PayPal Payouts API: create_payout() failed. Response code: ' . $code . '. Error: ' . ( ! empty( $body['debug_id'] ) && ! empty( $body['name'] && ! empty( $body['message'] ) ) ? $body['name'] . ' - ' . $body['message'] . '. Details: ' . ( ! empty( $body['details'] ) ? json_encode( $body['details'] ) : 'n/a' ) : $message ) );
			return false;

		}

		// Check if the access token is found in the response's body
		if ( ! is_array( $body ) || empty( $body['batch_header']['payout_batch_id'] ) ) {

			slicewp_add_log( 'PayPal Payouts API: create_payout() failed. Payout batch ID is missing from response body.' );
			return false;

		}

		return $body;

	}


	/**
	 * Makes an API call to PayPal to ask for a payout's batch details
	 *
	 * @param string $payout_batch_id
	 *
	 * @return array
	 *
	 */
	public function get_payout_batch_details( $payout_batch_id ) {

		$client_id = $this->get_api_client_id();
		$secret    = $this->get_api_secret();

		// If the API credentials are not set, add to the log and return false
		if ( empty( $client_id ) || empty( $secret ) ) {

			slicewp_add_log( 'PayPal Payouts API: get_payout_batch_details() failed. Credentials missing.' );
			return null;

		}

		// Get token
		$token = $this->get_access_token();

		if ( is_null( $token ) ) {

			slicewp_add_log( 'PayPal Payouts API: get_payout_batch_details() failed. Token is invalid.' );
			return null;

		}

		// Set API url
		$api_url = 'https://api' . ( $this->is_test_mode() ? '.sandbox' : '' ) . '.paypal.com/v1/payments/payouts/' . $payout_batch_id;

		// Prepare call arguments
		$args = array(
			'timeout'     => 45,
		    'redirection' => 10,
		    'httpversion' => '1.1',
		    'headers' => array(
		    	'Authorization'   => 'Bearer ' . $token,
		    	'Content-type' 	  => 'application/json'
		    )
		);

		// Make the call to the API
		$response = wp_remote_get( $api_url, $args );

		$code 	 = wp_remote_retrieve_response_code( $response );
		$message = wp_remote_retrieve_response_message( $response );

		if ( is_wp_error( $response ) ) {

			slicewp_add_log( 'PayPal Payouts API: get_payout_batch_details() failed. Error: ' . $response->get_error_message() );
			return null;

		}

		$body = json_decode( wp_remote_retrieve_body( $response ), true );

		// Check if response is okay
		if ( $code != 200 ) {

			slicewp_add_log( 'PayPal Payouts API: get_payout_batch_details() failed. Response code: ' . $code . '. Error: ' . ( ! empty( $body['error'] ) && ! empty( $body['error_description'] ) ? $body['error'] . ' - ' . $body['error_description'] : $message ) );
			return null;

		}

		// Check if the batch ID is found in the response's body
		if ( ! is_array( $body ) || empty( $body['batch_header']['payout_batch_id'] ) ) {

			slicewp_add_log( 'PayPal Payouts API: get_payout_batch_details() failed. Batch ID is missing from the response body.' );
			return null;

		}

		// Return the batch details
		return $body;

	}


	/**
	 * Returns the API Client ID, either the test or live one,
	 * depending if test mode is enabled or not
	 *
	 * @return string
	 *
	 */
	private function get_api_client_id() {

		return $this->credentials['api_client_id_' . ( $this->is_test_mode() ? 'test' : 'live' )];

	}


	/**
	 * Returns the API Secret, either the test or live one,
	 * depending if test mode is enabled or not
	 *
	 * @return string
	 *
	 */
	private function get_api_secret() {

		return $this->credentials['api_secret_' . ( $this->is_test_mode() ? 'test' : 'live' )];

	}


	/**
	 * Checks if the PayPal API is in test mode
	 *
	 * @return bool
	 *
	 */
	public function is_test_mode() {

		return ( empty( $this->test_mode ) ? false : true );

	}


	/**
	 * Prepares the given amount for API calls
	 *
	 * @param string
	 *
	 * @return string
	 *
	 */
	protected function sanitize_amount( $amount ) {

		return number_format( (float)$amount, 2, '.', '' );

	}


	/**
	 * Returns the URL to the payout on PayPal's site.
	 * 
	 * @param string $paypal_payout_batch_id
	 * @param bool   $sandbox
	 * 
	 * @return string
	 * 
	 */
	public function get_paypal_masspay_site_url( $paypal_payout_batch_id, $sandbox = false ) {

		return 'https://' . ( $sandbox ? 'sandbox.' : '' ) . 'paypal.com/activity/masspay/MPA-' . $paypal_payout_batch_id;

	}


	/**
	 * Returns the URL to the transaction on PayPal's site.
	 * 
	 * @param string $transaction_id
	 * @param bool   $sandbox
	 * 
	 * @return string
	 * 
	 */
	public function get_paypal_transaction_site_url( $transaction_id, $sandbox = false ) {

		return 'https://' . ( $sandbox ? 'sandbox.' : '' ) . 'paypal.com/activity/payment/' . $transaction_id;

	}

}