<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Register "PayPal Payouts" payout method.
 *
 * @param array $payout_methods
 *
 * @return array
 *
 */
function slicewp_register_payout_method_paypal_payouts( $payout_methods ) {

	if ( ! is_array( $payout_methods ) ) {
		return array();
	}

	$payout_methods['paypal_payouts'] = array(
		'label'    => __( 'PayPal Payouts', 'slicewp' ),
		'supports' => array( 'single_payment', 'bulk_payments', 'payout_request_invoice' ),
		'messages' => array(
			'payout_action_confirmation_single_payment' => __( 'This will attempt to pay this payment through PayPal Payouts. If the payment is successful it will be marked as paid and all associated commissions will also be marked as paid. Are you sure you want to continue?', 'slicewp' ),
			'payout_action_confirmation_bulk_payments'  => __( 'This will attempt to pay unpaid payments through PayPal Payouts. If a payment is successful it will be marked as paid and all associated commissions will also be marked as paid. Are you sure you want to continue?', 'slicewp' )
		)
	);

	return $payout_methods;

}
add_filter( 'slicewp_register_payout_methods', 'slicewp_register_payout_method_paypal_payouts', 20 );


/**
 * Registers statuses needed for PayPal payouts
 *
 * @param array $statuses
 *
 * @return array
 *
 */
function slicewp_paypal_payouts_register_payment_available_status( $statuses ) {

	// Register "unclaimed" status
	$statuses['unclaimed'] = __( 'Unclaimed', 'slicewp' );

	return $statuses;

}
add_filter( 'slicewp_payment_available_statuses', 'slicewp_paypal_payouts_register_payment_available_status' );


/**
 * Checks to see whether within a payout the "paypal_payouts" payout method can performs bulk payments.
 * 
 * @param bool 	 $can_do
 * @param int 	 $payout_id
 * @param string $payout_method
 * 
 * @return bool
 * 
 */
function slicewp_can_do_bulk_payments_payout_method_paypal_payouts( $can_do, $payout_id, $payout_method ) {

	if ( 'paypal_payouts' != $payout_method ) {
		return $can_do;
	}

	$payments = slicewp_get_payments( array( 'number' => -1, 'payout_id' => $payout_id, 'payout_method' => $payout_method, 'status' => 'unpaid' ) );

	// Filter out certain payments.
	foreach ( $payments as $key => $payment ) {

		// Take into account only payments that haven't been processed through PayPal already.
		$paypal_payout_batch_id = slicewp_get_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', true );

		if ( ! empty( $paypal_payout_batch_id ) ) {

			unset( $payments[$key] );
			continue;

		}

		// Filter out payments for affiliates that don't have the payment email set.
		$affiliate 	   = slicewp_get_affiliate( $payment->get( 'affiliate_id' ) );
		$payment_email = $affiliate->get( 'payment_email' );

		if ( empty( $payment_email ) ) {

			unset( $payments[$key] );
			continue;

		}
		
	}

	if ( ! empty( $payments ) ) {
		$can_do = true;
	}

	return $can_do;

}
add_filter( 'slicewp_can_do_bulk_payments', 'slicewp_can_do_bulk_payments_payout_method_paypal_payouts', 10, 3 );


/**
 * Handles "paypal_payouts" single payment.
 *
 * @param int $payment_id
 *
 */
function slicewp_do_single_payment_paypal_payouts( $payment_id ) {

	$payment = slicewp_get_payment( $payment_id );

	if ( ! in_array( $payment->get( 'status' ), array( 'unpaid', 'requested' ) ) ) {
		return;
	}

	// Bail if the payment has a PayPal payout batch ID saved.
	$paypal_payout_batch_id = slicewp_get_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', true );

	if ( ! empty( $paypal_payout_batch_id ) ) {
		return;
	}

	// Return with failed message if the affiliate doesn't have a payment email set.
	$affiliate = slicewp_get_affiliate( $payment->get( 'affiliate_id' ) );

	if ( empty( $affiliate->get( 'payment_email' ) ) ) {

		slicewp_admin_notices()->register_notice( 'do_single_payment_paypal_payouts_fail', '<p>' . __( "The affiliate cannot be paid through PayPal Payouts because they don't have a payment email set for their account.", 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'do_single_payment_paypal_payouts_fail' );

		return;

	}

	// Init the PayPal API class.
	$paypal_api = new SliceWP_PayPal_Payouts_API();

	// Create the PayPal payout.
	$paypal_payout = $paypal_api->create_payout( array( $payment ) );

	// Return with failed message if the payout could not be processed.
	if ( ! $paypal_payout ) {

		slicewp_admin_notices()->register_notice( 'do_single_payment_paypal_payouts_fail', '<p>' . __( 'PayPal payout could not be generated. The payment was not processed.', 'slicewp' ) . '</p>' . ( ! empty( $paypal_api->get_last_error() ) ? '<p>' . sprintf( __( 'The following error was returned by PayPal: %s', 'slicewp' ), $paypal_api->get_last_error() ) . '</p>' : '' ), 'error' );
		slicewp_admin_notices()->display_notice( 'do_single_payment_paypal_payouts_fail' );

		return;

	}

	// Update payment.
	$payment_data = array(
		'date_modified' => slicewp_mysql_gmdate(),
		'status' 		=> 'processing'
	);

	$updated = slicewp_update_payment( $payment->get( 'id' ), $payment_data );

	if ( $updated ) {

		// Add the PayPal payout batch ID to the payment meta.
		slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', sanitize_text_field( $paypal_payout['batch_header']['payout_batch_id'] ) );

		// Mark the payment as test if we're in test mode.
		if ( $paypal_api->is_test_mode() ) {

			slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_is_test', 1 );

		}

	}

	// Redirect to the current page.
	wp_redirect( remove_query_arg( array( 'slicewp_action', 'slicewp_token', 'payout_method' ), add_query_arg( array( 'slicewp_message' => 'payout_single_payment_paypal_payouts_success' ) ) ) );
	exit;

}
add_action( 'slicewp_do_single_payment_paypal_payouts', 'slicewp_do_single_payment_paypal_payouts', 50 );


/**
 * Handles "paypal_payouts" bulk payments.
 *
 * @param int $payout_id
 *
 */
function slicewp_do_bulk_payments_paypal_payouts( $payout_id ) {

	$payments = slicewp_get_payments( array( 'number' => -1, 'payout_id' => $payout_id, 'payout_method' => 'paypal_payouts', 'status' => array( 'unpaid', 'requested' ) ) );

	// Filter out certain payments.
	foreach ( $payments as $key => $payment ) {

		// Take into account only payments that haven't been processed through PayPal already.
		$paypal_payout_batch_id = slicewp_get_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', true );

		if ( ! empty( $paypal_payout_batch_id ) ) {

			unset( $payments[$key] );
			continue;

		}

		// Filter out payments for affiliates that don't have the payment email set.
		$affiliate 	   = slicewp_get_affiliate( $payment->get( 'affiliate_id' ) );
		$payment_email = $affiliate->get( 'payment_email' );

		if ( empty( $payment_email ) ) {

			unset( $payments[$key] );
			continue;

		}

	}

	$payments = array_values( $payments );

	if ( empty( $payments ) ) {

		slicewp_admin_notices()->register_notice( 'do_bulk_payments_paypal_payouts_fail', '<p>' . __( 'PayPal payout could not be generated. There are no eligible payments.', 'slicewp' ) . '</p>', 'error' );
		slicewp_admin_notices()->display_notice( 'do_bulk_payments_paypal_payouts_fail' );

		return;

	}

	// Init the PayPal API class.
	$paypal_api = new SliceWP_PayPal_Payouts_API();

	// Create the PayPal payout.
	$paypal_payout = $paypal_api->create_payout( $payments );

	// Return with failed message if the payout could not be processed.
	if ( ! $paypal_payout ) {

		slicewp_admin_notices()->register_notice( 'do_bulk_payments_paypal_payouts_fail', '<p>' . __( 'PayPal payout could not be generated. No payments were processed.', 'slicewp' ) . '</p>' . ( ! empty( $paypal_api->get_last_error() ) ? '<p>' . sprintf( __( 'The following error was returned by PayPal: %s', 'slicewp' ), $paypal_api->get_last_error() ) . '</p>' : '' ), 'error' );
		slicewp_admin_notices()->display_notice( 'do_bulk_payments_paypal_payouts_fail' );

		return;

	}

	// Add the PayPal payout batch ID to the payout meta.
	slicewp_add_payout_meta( $payout_id, 'paypal_payout_batch_id', sanitize_text_field( $paypal_payout['batch_header']['payout_batch_id'] ) );

	// Add the timestamp when the PayPal batch was updated.
	slicewp_update_payout_meta( $payout_id, 'paypal_payout_batch_last_updated', time() );

	// Go through each payment and mark it as processing
	foreach ( $payments as $payment ) {

		// Update payment
		$payment_data = array(
			'date_modified' => slicewp_mysql_gmdate(),
			'status' 		=> 'processing'
		);

		$updated = slicewp_update_payment( $payment->get( 'id' ), $payment_data );

		if ( $updated ) {

			// Add the PayPal payout batch ID to the payment meta.
			slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', sanitize_text_field( $paypal_payout['batch_header']['payout_batch_id'] ) );

			// Mark the payment as test if we're in test mode.
			if ( $paypal_api->is_test_mode() ) {

				slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_is_test', 1 );

			}

		}

	}

	// Redirect to the current page.
	wp_redirect( add_query_arg( array( 'page' => 'slicewp-payouts', 'subpage' => 'view-payout', 'payout_id' => absint( $payout_id ), 'slicewp_message' => 'payout_bulk_payments_paypal_payouts_success' ), admin_url( 'admin.php' ) ) );
	exit;

}
add_action( 'slicewp_do_bulk_payments_paypal_payouts', 'slicewp_do_bulk_payments_paypal_payouts', 10, 2 );


/**
 * Connects to PayPal to see if the payments from the payout have been processed
 *
 */
function slicewp_paypal_payouts_handler_bulk_payments_status() {

	// Return if not on the view payout page
	if ( empty( $_GET['subpage'] ) || $_GET['subpage'] != 'view-payout' ) {
		return;
	}

	// Return if the payout id isn't provided
	if ( empty( $_GET['payout_id'] ) ) {
		return;
	}

	if ( ! empty( $_GET['updated'] ) ) {
		return;
	}

	// Return if the admin is currently doing an action
	if ( ! empty( $_POST ) ) {
		return;
	}

	$payout_id = absint( $_GET['payout_id'] );

	// Check if batch was checked in the past two minutes
	$paypal_payout_batch_last_updated = absint( slicewp_get_payout_meta( $payout_id, 'paypal_payout_batch_last_updated', true ) );

	if ( $paypal_payout_batch_last_updated + MINUTE_IN_SECONDS > time() ) {
		return;
	}

	// Check to see if all payments are paid.
	$all_payments_count  = slicewp_get_payments( array( 'payout_id' => $payout_id, 'payout_method' => 'paypal_payouts' ), true );
	$paid_payments_count = slicewp_get_payments( array( 'payout_id' => $payout_id, 'payout_method' => 'paypal_payouts', 'status' => 'paid' ), true );

	if ( $all_payments_count == $paid_payments_count ) {
		return;
	}

	// Go through each payment and get the batch ID it has saved.
	$payments = slicewp_get_payments( array( 'number' => -1, 'payout_id' => $payout_id, 'payout_method' => 'paypal_payouts' ) );

	// Check for PayPal payout batch IDs
	$paypal_payout_batch_ids = slicewp_get_payout_meta( $payout_id, 'paypal_payout_batch_id' );

	// Make sure the paypal payout batch ids variable is an array
	if ( ! is_array( $paypal_payout_batch_ids ) ) {
		$paypal_payout_batch_ids = array();
	}

	foreach ( $payments as $payment ) {

		if ( ! in_array( $payment->get( 'status' ), array( 'processing', 'unclaimed' ) ) ) {
			continue;
		}

		$paypal_payout_batch_id = slicewp_get_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', true );

		if ( empty( $paypal_payout_batch_id ) ) {
			continue;
		}

		$paypal_payout_batch_ids[] = $paypal_payout_batch_id;

	}

	// Remove any duplicates and empty values
	$paypal_payout_batch_ids = array_filter( $paypal_payout_batch_ids );
	$paypal_payout_batch_ids = array_unique( $paypal_payout_batch_ids );

	if ( empty( $paypal_payout_batch_ids ) ) {
		return;
	}

	// Init PayPal Payouts API class
	$paypal_api = new SliceWP_PayPal_Payouts_API();

	// Save all payout items grabbed from PayPal in this variable
	$paypal_payout_items = array();

	// Go through each payout batch and grab the payout items returned by PayPal
	foreach ( $paypal_payout_batch_ids as $paypal_payout_batch_id ) {

		// Get batch details
		$payout_batch_details = $paypal_api->get_payout_batch_details( $paypal_payout_batch_id );

		if ( is_null( $payout_batch_details ) ) {
			continue;
		}

		// Go through each payout item and update the corresponding payment
		foreach ( $payout_batch_details['items'] as $payout_item ) {

			$payment_id = ( ! empty( $payout_item['payout_item']['sender_item_id'] ) ? absint( str_replace( 'slicewp-payment-', '', $payout_item['payout_item']['sender_item_id'] ) ) : 0 );

			if ( empty( $payment_id ) ) {
				continue;
			}

			$paypal_payout_items[$payment_id] = $payout_item;

		}

	}

	// Go through each payout payment and update it if necessary
	foreach ( $payments as $payment ) {

		if ( empty( $paypal_payout_items[$payment->get( 'id' )] ) ) {
			continue;
		}

		if ( ! in_array( $payment->get( 'status' ), array( 'processing', 'unclaimed' ) ) ) {
			continue;
		}

		// For more readable code get the payout item from the array
		$payout_item = $paypal_payout_items[$payment->get( 'id' )];

		// Get SliceWP payment status from PayPal payment status
		$payment_status = slicewp_paypal_payouts_sanitize_transaction_status( $payout_item['transaction_status'] );

		// If the status hasn't changed, continue to next payment
		if ( $payment->get( 'status' ) == $payment_status ) {
			continue;
		}

		$payment_args = array(
			'date_modified' => slicewp_mysql_gmdate(),
			'status'		=> $payment_status
		);

		$updated = slicewp_update_payment( $payment->get( 'id' ), $payment_args );

		if ( $updated ) {

			// If the payment was paid, update each of the generated commissions.
			if ( $payment_status == 'paid' ) {

				$commission_ids = slicewp_get_commissions( array( 'number' => -1, 'payment_id' => $payment->get( 'id' ), 'fields' => 'id' ) );

				$commission_data = array(
					'date_modified' => slicewp_mysql_gmdate(),
					'status'		=> 'paid'
				);

				foreach ( $commission_ids as $commission_id ) {

					slicewp_update_commission( $commission_id, $commission_data );
				
				}

			}

			// Save the PayPal payout metadata.
			if ( ! empty( $payout_item['payout_item_id'] ) ) {
				slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_payout_item_id', sanitize_text_field( $payout_item['payout_item_id'] ) );
			}

			if ( ! empty( $payout_item['transaction_id'] ) ) {
				slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_transaction_id', sanitize_text_field( $payout_item['transaction_id'] ) );
			}

			if ( ! empty( $payout_item['payout_batch_id'] ) ) {
				slicewp_update_payment_meta( $payment->get( 'id' ), 'paypal_payout_batch_id', sanitize_text_field( $payout_item['payout_batch_id'] ) );
			}

		}

	}

	// Update the timestamp when the PayPal batch was updated
	slicewp_update_payout_meta( $payout_id, 'paypal_payout_batch_last_updated', time() );

}
add_action( 'admin_init', 'slicewp_paypal_payouts_handler_bulk_payments_status', 15 );


/**
 * Connects to PayPal to see if the current payment has been processed.
 *
 */
function slicewp_paypal_payouts_handler_single_payment_status() {

	// Return if not on the review payment page.
	if ( empty( $_GET['subpage'] ) || $_GET['subpage'] != 'review-payment' ) {
		return;
	}

	// Return if the payment id isn't provided.
	if ( empty( $_GET['payment_id'] ) ) {
		return;
	}

	if ( ! empty( $_GET['updated'] ) ) {
		return;
	}

	// Return if the admin is currently doing an action.
	if ( ! empty( $_POST ) ) {
		return;
	}

	$payment_id = absint( $_GET['payment_id'] );
	$payment    = slicewp_get_payment( $payment_id );

	if ( is_null( $payment ) ) {
		return;
	}

	// Exit quickly if the payment isn't processing or unclaimed.
	if ( ! in_array( $payment->get( 'status' ), array( 'processing', 'unclaimed' ) ) ) {
		return;
	}

	// Check for PayPal payout batch ID.
	$paypal_payout_batch_id = slicewp_get_payment_meta( $payment_id, 'paypal_payout_batch_id', true );

	if ( empty( $paypal_payout_batch_id ) ) {
		return;
	}

	// Check if batch was checked in the past two minutes.
	$paypal_payout_batch_last_updated = absint( slicewp_get_payment_meta( $payment_id, 'paypal_payout_batch_last_updated', true ) );

	if ( $paypal_payout_batch_last_updated + 2 * MINUTE_IN_SECONDS > time() ) {
		return;
	}

	// Init PayPal Payouts API class.
	$paypal_api = new SliceWP_PayPal_Payouts_API();

	// Get batch details.
	$payout_batch_details = $paypal_api->get_payout_batch_details( $paypal_payout_batch_id );

	if ( is_null( $payout_batch_details ) ) {
		return;
	}

	// Go through each payout item and update the payment if found.
	foreach ( $payout_batch_details['items'] as $payout_item ) {

		$_payment_id = ( ! empty( $payout_item['payout_item']['sender_item_id'] ) ? absint( str_replace( 'slicewp-payment-', '', $payout_item['payout_item']['sender_item_id'] ) ) : 0 );

		if ( empty( $_payment_id ) || $_payment_id != $payment_id ) {
			continue;
		}

		// Get the status.
		$payment_status = slicewp_paypal_payouts_sanitize_transaction_status( $payout_item['transaction_status'] );

		// Prepare the update args.
		$payment_args = array(
			'date_modified' => slicewp_mysql_gmdate(),
			'status'		=> $payment_status
		);

		// Update the payment.
		$updated = slicewp_update_payment( $payment_id, $payment_args );

		if ( $updated ) {

			// If the payment was paid, update each of the generated commissions.
			if ( $payment_status == 'paid' ) {

				$commission_ids = slicewp_get_commissions( array( 'number' => -1, 'payment_id' => $payment->get( 'id' ), 'fields' => 'id' ) );

				$commission_data = array(
					'date_modified' => slicewp_mysql_gmdate(),
					'status'		=> 'paid'
				);

				foreach ( $commission_ids as $commission_id ) {

					slicewp_update_commission( $commission_id, $commission_data );
				
				}

			}

			// Save the PayPal payout metadata.
			if ( ! empty( $payout_item['payout_item_id'] ) ) {
				slicewp_update_payment_meta( $payment_id, 'paypal_payout_item_id', sanitize_text_field( $payout_item['payout_item_id'] ) );
			}

			if ( ! empty( $payout_item['transaction_id'] ) ) {
				slicewp_update_payment_meta( $payment_id, 'paypal_transaction_id', sanitize_text_field( $payout_item['transaction_id'] ) );
			}

			if ( ! empty( $payout_item['payout_batch_id'] ) ) {
				slicewp_update_payment_meta( $payment_id, 'paypal_payout_batch_id', sanitize_text_field( $payout_item['payout_batch_id'] ) );
			}

		}

	}

	// Update the timestamp when the PayPal batch was updated
	slicewp_update_payment_meta( $payment_id, 'paypal_payout_batch_last_updated', time() );

}
add_action( 'admin_init', 'slicewp_paypal_payouts_handler_single_payment_status', 50 );


/**
 * Returns the status used by SliceWP based on the PayPal Payouts API given status.
 *
 * @param string $status
 *
 * @return string
 *
 */
function slicewp_paypal_payouts_sanitize_transaction_status( $status ) {

	if ( in_array( $status, array( 'SUCCESS' ) ) ) {
		return 'paid';
	}

	if ( in_array( $status, array( 'RETURNED' ) ) ) {
		return 'unpaid';
	}

	if ( in_array( $status, array( 'PENDING', 'ONHOLD' ) ) ) {
		return 'processing';
	}

	if ( in_array( $status, array( 'FAILED', 'BLOCKED', 'REFUNDED', 'REVERSED' ) ) ) {
		return 'failed';
	}

	if ( in_array( $status, array( 'UNCLAIMED' ) ) ) {
		return 'unclaimed';
	}

}


/**
 * Checks whether the API credentials have been set in Settings.
 * 
 * @return bool
 * 
 */
function slicewp_has_paypal_api_credentials_set() {

	// Get test mode
	$test_mode = slicewp_get_setting( 'paypal_api_test_mode' );

	// Get credentials
	$api_client_id_live = slicewp_get_setting( 'paypal_api_client_id_live' );
	$api_secret_live 	= slicewp_get_setting( 'paypal_api_secret_live' );
	$api_client_id_test = slicewp_get_setting( 'paypal_api_client_id_test' );
	$api_secret_test 	= slicewp_get_setting( 'paypal_api_secret_test' );

	if ( empty( $test_mode ) && ! empty( $api_client_id_live ) && ! empty( $api_secret_live ) ) {
		return true;
	}

	if ( ! empty( $test_mode ) && ! empty( $api_client_id_test ) && ! empty( $api_secret_test ) ) {
		return true;
	}

	return false;

}