<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Register the class that handles database queries for the bonus rewards.
 *
 * @param array $classes
 *
 * @return array
 *
 */
function slicewp_register_database_classes_bonus_rewards( $classes ) {

	$classes['bonus_rewards'] 	 = 'SliceWP_Object_DB_Bonus_Rewards';
	$classes['bonus_rewardmeta'] = 'SliceWP_Object_Meta_DB_Bonus_Rewards';

	return $classes;

}
add_filter( 'slicewp_register_database_classes', 'slicewp_register_database_classes_bonus_rewards', 100 );


/**
 * Returns an array with SliceWP_Bonus_Rewards objects from the database.
 *
 * @param array $args
 * @param bool  $count
 *
 * @return array
 *
 */
function slicewp_get_bonus_rewards( $args = array(), $count = false ) {

	$bonus_rewards = slicewp()->db['bonus_rewards']->get_bonus_rewards( $args, $count );

	/**
	 * Add a filter hook just before returning.
	 *
	 * @param array $bonus_rewards
	 * @param array $args
	 * @param bool  $count
	 *
	 */
	return apply_filters( 'slicewp_get_bonus_rewards', $bonus_rewards, $args, $count );

}


/**
 * Gets a bonus reward from the database.
 *
 * @param mixed int|object      - bonus reward id or object representing the bonus reward.
 *
 * @return SliceWP_Bonus_Reward|null
 *
 */
function slicewp_get_bonus_reward( $bonus_reward ) {

	return slicewp()->db['bonus_rewards']->get_object( $bonus_reward );

}


/**
 * Inserts a new bonus reward into the database.
 *
 * @param array $data
 *
 * @return mixed int|false
 *
 */
function slicewp_insert_bonus_reward( $data ) {

	return slicewp()->db['bonus_rewards']->insert( $data );

}


/**
 * Updates a bonus reward from the database.
 *
 * @param int 	$bonus_reward_id
 * @param array $data
 *
 * @return bool
 *
 */
function slicewp_update_bonus_reward( $bonus_reward_id, $data ) {

	return slicewp()->db['bonus_rewards']->update( $bonus_reward_id, $data );

}


/**
 * Deletes a bonus reward from the database.
 * Deletes all bonus reward metadata.
 *
 * @param int $bonus_reward_id
 *
 * @return bool
 *
 */
function slicewp_delete_bonus_reward( $bonus_reward_id ) {

	$deleted = slicewp()->db['bonus_rewards']->delete( $bonus_reward_id );

	if ( ! $deleted ) {
		return false;
	}

	$bonus_reward_meta = slicewp_get_bonus_reward_meta( $bonus_reward_id );

	// Delete the bonus reward's metadata.
	if ( ! empty( $bonus_reward_meta ) ) {

		foreach ( $bonus_reward_meta as $key => $value ) {

			slicewp_delete_bonus_reward_meta( $bonus_reward_id, $key );

		}

	}

	return true;

}


/**
 * Inserts a new meta entry for the bonus reward.
 *
 * @param int    $bonus_reward_id
 * @param string $meta_key
 * @param string $meta_value
 * @param bool   $unique
 *
 * @return mixed int|false
 *
 */
function slicewp_add_bonus_reward_meta( $bonus_reward_id, $meta_key, $meta_value, $unique = false ) {

	return slicewp()->db['bonus_rewardmeta']->add( $bonus_reward_id, $meta_key, $meta_value, $unique );

}


/**
 * Updates a meta entry for the bonus reward.
 *
 * @param int    $bonus_reward_id
 * @param string $meta_key
 * @param string $meta_value
 * @param bool   $prev_value
 *
 * @return bool
 *
 */
function slicewp_update_bonus_reward_meta( $bonus_reward_id, $meta_key, $meta_value, $prev_value = '' ) {

	return slicewp()->db['bonus_rewardmeta']->update( $bonus_reward_id, $meta_key, $meta_value, $prev_value );

}


/**
 * Returns a meta entry for the bonus reward.
 *
 * @param int    $bonus_reward_id
 * @param string $meta_key
 * @param bool   $single
 *
 * @return mixed
 *
 */
function slicewp_get_bonus_reward_meta( $bonus_reward_id, $meta_key = '', $single = false ) {

	return slicewp()->db['bonus_rewardmeta']->get( $bonus_reward_id, $meta_key, $single );

}


/**
 * Removes a meta entry for the bonus reward.
 *
 * @param int    $bonus_reward_id
 * @param string $meta_key
 * @param string $meta_value
 * @param bool   $delete_all
 *
 * @return bool
 *
 */
function slicewp_delete_bonus_reward_meta( $bonus_reward_id, $meta_key, $meta_value = '', $delete_all = '' ) {

	return slicewp()->db['bonus_rewardmeta']->delete( $bonus_reward_id, $meta_key, $meta_value, $delete_all );

}