<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Registers the "bonus_reward" commission type.
 *
 * @param array $commission_types
 *
 * @return array
 *
 */
function slicewp_register_commission_type_bonus_reward( $commission_types ) {

	$commission_types['bonus_reward'] = array(
		'label' 	 => __( 'Bonus Reward', 'slicewp' ),
		'rate_types' => array( 'fixed_amount' )
	);

	return $commission_types;

}
add_filter( 'slicewp_register_commission_types', 'slicewp_register_commission_type_bonus_reward', 25 );


/**
 * Returns the progress the given affiliate has made for the given bonus.
 * 
 * @param int $affiliate_id
 * @param int $bonus_id
 * 
 * @return null|array
 * 
 */
function slicewp_get_affiliate_bonus_progress( $affiliate_id, $bonus_id ) {

    $bonuses = array_filter( (array)get_option( 'slicewp_performance_bonuses', array() ) );

    if ( empty( $bonuses[$bonus_id] ) ) {
        return null;
    }

    $bonus      = $bonuses[$bonus_id];
    $date_range = _slicewp_get_bonus_date_range( $bonus_id );
    $progress   = array();

    // Build the progress.
    if ( 'referred_orders_count' == $bonus['target_type'] ) {

        $commissions_count = slicewp_get_commissions( array( 'affiliate_id' => $affiliate_id, 'status' => array( 'paid', 'unpaid' ), 'type' => array( 'sale', 'subscription' ), 'date_min' => $date_range['date_start'], 'date_max' => $date_range['date_end'] ), true );

        $progress['referred_orders_count'] = array(
            'target'     => $bonus['target'],
            'current'    => $commissions_count,
            'percentage' => ( $commissions_count >= $bonus['target'] ? 100 : round( $commissions_count / $bonus['target'] * 100, 2 ) ),
            'date_end'   => $date_range['date_end']
        );

    }

    if ( 'referred_orders_totals' == $bonus['target_type'] ) {

        $commissions        = array_filter( slicewp_get_commissions( array( 'number' => -1, 'affiliate_id' => $affiliate_id, 'status' => array( 'paid', 'unpaid' ), 'type' => array( 'sale', 'subscription' ), 'fields' => 'reference_amount', 'date_min' => $date_range['date_start'], 'date_max' => $date_range['date_end'] ) ) );
        $commissions_totals = array_sum( $commissions );
        
        $progress['referred_orders_totals'] = array(
            'target'     => slicewp_format_amount( $bonus['target'], slicewp_get_setting( 'active_currency', 'USD' ) ),
            'current'    => slicewp_format_amount( $commissions_totals, slicewp_get_setting( 'active_currency', 'USD' ) ),
            'percentage' => ( $commissions_totals >= $bonus['target'] ? 100 : round( $commissions_totals / $bonus['target'] * 100, 2 ) ),
            'date_end'   => $date_range['date_end']
        );

    }

    if ( 'earned_commissions_totals' == $bonus['target_type'] ) {

        $commissions        = array_filter( slicewp_get_commissions( array( 'number' => -1, 'affiliate_id' => $affiliate_id, 'status' => array( 'paid', 'unpaid' ), 'type' => array( 'sale', 'subscription' ), 'fields' => 'amount', 'date_min' => $date_range['date_start'], 'date_max' => $date_range['date_end'] ) ) );
        $commissions_totals = array_sum( $commissions );

        $progress['earned_commissions_totals'] = array(
            'target'     => slicewp_format_amount( $bonus['target'], slicewp_get_setting( 'active_currency', 'USD' ) ),
            'current'    => slicewp_format_amount( $commissions_totals, slicewp_get_setting( 'active_currency', 'USD' ) ),
            'percentage' => ( $commissions_totals >= $bonus['target'] ? 100 : round( $commissions_totals / $bonus['target'] * 100, 2 ) ),
            'date_end'   => $date_range['date_end']
        );

    }

    return $progress;

}


/**
 * Parses the bonuses available to the given affiliate and, if the thresholds for the bonuses
 * have been reached, generate the rewards associated to the bonuses.
 * 
 * @param int $affiliate_id
 * 
 */
function slicewp_maybe_generate_affiliate_bonuses_rewards( $affiliate_id ) {

    $affiliate = slicewp_get_affiliate( $affiliate_id );

    if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {
        return;
    }

    $bonuses = array_filter( (array)get_option( 'slicewp_performance_bonuses', array() ) );

    if ( empty( $bonuses ) ) {
        return;
    }

    // Filter out inactive bonuses.
    foreach ( $bonuses as $bonus_id => $bonus ) {

        if ( empty( $bonus['status'] ) || $bonus['status'] != 'active' ) {
            unset( $bonuses[$bonus_id] );
        }

    }

    // Filter out bonuses that are not meant for this affiliate.
    $affiliate_id       = $affiliate_id;
    $affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

    foreach ( $bonuses as $bonus_id => $bonus ) {

        if ( empty( $bonus['availability_type'] ) ) {
            continue;
        }

        if ( $bonus['availability_type'] == 'exclude' && ( in_array( $affiliate_id, $bonus['availability_affiliate_id'] ) || in_array( $affiliate_group_id, $bonus['availability_affiliate_group_id'] ) ) ) {
            unset( $bonuses[$bonus_id] );
        }

        if ( $bonus['availability_type'] == 'include' && ( ! in_array( $affiliate_id, $bonus['availability_affiliate_id'] ) && ! in_array( $affiliate_group_id, $bonus['availability_affiliate_group_id'] ) ) ) {
            unset( $bonuses[$bonus_id] );
        }

    }

    // Go through each bonus and check if we need to reward the affiliate.
    foreach ( $bonuses as $bonus_id => $bonus ) {

        // Continue, if the reward type isn't set.
        if ( empty( $bonus['reward_type'] ) ) {
            continue;
        }

        $date_range = _slicewp_get_bonus_date_range( $bonus['id'] );

        // Check if the reward has already been given out.
        $rewards = slicewp_get_bonus_rewards( array( 'affiliate_id' => $affiliate_id, 'bonus_id' => $bonus['id'], 'date_min' => $date_range['date_start'], 'date_max' => $date_range['date_end'] ), true );

        if ( ! empty( $rewards ) ) {
            continue;
        }

        // Check the progress of the affiliate for this bonus.
        // If the target isn't reached, continue to the next bonus.
        $progress = slicewp_get_affiliate_bonus_progress( $affiliate_id, $bonus['id'] );

        if ( $progress[$bonus['target_type']]['percentage'] != 100 ) {
            continue;
        }


        // Prepare reward data.
        $reward_data = array(
            'affiliate_id'  => absint( $affiliate_id ),
            'bonus_id'      => absint( $bonus_id ),
            'type'          => sanitize_text_field( $bonus['reward_type'] ),
            'date_created'  => slicewp_mysql_gmdate(),
            'date_modified' => slicewp_mysql_gmdate()
        );

        // Insert the bonus reward.
        $bonus_reward_id = slicewp_insert_bonus_reward( $reward_data );

        if ( $bonus_reward_id ) {

            // Generate the reward based on the type.
            if ( 'commission' == $bonus['reward_type'] ) {

                $commission_data = array(
                    'affiliate_id'  => absint( $affiliate_id ),
                    'amount'        => slicewp_sanitize_amount( $bonus['reward_commission_amount'] ),
                    'currency'      => slicewp_get_setting( 'active_currency', 'USD' ),
                    'origin'        => 'slicewp',
                    'reference'     => absint( $bonus_reward_id ),
                    'type'			=> 'bonus_reward',
                    'status'		=> 'unpaid',
                    'date_created'	=> slicewp_mysql_gmdate(),
                    'date_modified'	=> slicewp_mysql_gmdate(),
                );

                $commission_id = slicewp_insert_commission( $commission_data );

                if ( $commission_id ) {
                    slicewp_add_bonus_reward_meta( $bonus_reward_id, '_commission_id', $commission_id );
                    slicewp_add_bonus_reward_meta( $bonus_reward_id, '_commission_data', $commission_data );
                }

            }

            if ( 'store_credit' == $bonus['reward_type'] ) {

                if ( slicewp_is_add_on_active( 'store_credit' ) ) {

                    $store_credit_data = array(
                        'amount'   => slicewp_sanitize_amount( $bonus['reward_store_credit_amount'] ),
                        'currency' => slicewp_get_setting( 'active_currency', 'USD' ),
                    );

                    slicewp_increase_user_store_credit_balance( $affiliate->get( 'user_id' ), slicewp_sanitize_amount( $bonus['reward_store_credit_amount'] ) );

                    slicewp_add_bonus_reward_meta( $bonus_reward_id, '_store_credit_data', $store_credit_data );

                }

            }

            // @todo - Generate reward for discount code.

            // Add the target data to the bonus.
            $target_data = array(
                'target_type' => sanitize_text_field( $bonus['target_type'] ),
                'target'      => sanitize_text_field( $bonus['target'] )
            );
            
            slicewp_add_bonus_reward_meta( $bonus_reward_id, '_target_data', $target_data );

            // Add the bonus data.
            slicewp_add_bonus_reward_meta( $bonus_reward_id, '_bonus_data', $bonus );

        }

        // Prepare email subject and content.
        switch ( $bonus['reward_type'] ) {

            case 'commission':
                $reward_string = sprintf( __( 'a %s bonus commission', 'slicewp' ), slicewp_format_amount( $bonus['reward_commission_amount'], slicewp_get_setting( 'active_currency', 'USD' ) ) );
                break;

            case 'store_credit':
                $reward_string = sprintf( __( '%s in store credit', 'slicewp' ), slicewp_format_amount( $bonus['reward_store_credit_amount'], slicewp_get_setting( 'active_currency', 'USD' ) ) );
                break;

            default:
                $reward_string = '';
                break;

        }

        switch ( $bonus['target_type'] ) {

            case 'referred_orders_count':
                $target_string = sprintf( __( 'referring %s orders', 'slicewp' ), $bonus['target'] );
                break;

            case 'referred_orders_totals':
                $target_string = sprintf( __( 'referring %s in order totals', 'slicewp' ), slicewp_format_amount( $bonus['target'], slicewp_get_setting( 'active_currency', 'USD' ) ) );
                break;

            case 'earned_commissions_totals':
                $target_string = sprintf( __( 'reaching %s in earned commission totals', 'slicewp' ), slicewp_format_amount( $bonus['target'], slicewp_get_setting( 'active_currency', 'USD' ) ) );
                break;

            default:
                $target_string = '';
                break;

        }

        $email_subject_admin     = __( "One of your affiliates has received a bonus reward.", 'slicewp' );
        $email_subject_affiliate = __( "Congratulations! You just got a bonus reward.", 'slicewp' );

        $email_content_admin     = sprintf( "
            <h2>Affiliate rewarded!</h2>
            <p>{{affiliate_first_name}} {{affiliate_last_name}} has been rewarded %s for %s.</p>
        ", $reward_string, $target_string );

        $email_content_affiliate = sprintf( "
            <h2>Congratulations!</h2>
            <p>You've been rewarded %s for %s.</p>
        ", $reward_string, $target_string );

        // Replace the tags with data.
        $merge_tags = new SliceWP_Merge_Tags();
        $merge_tags->set_data( 'affiliate', $affiliate );

        $email_subject_admin     = $merge_tags->replace_tags( $email_subject_admin );
        $email_subject_affiliate = $merge_tags->replace_tags( $email_subject_affiliate );

        $email_content_admin     = $merge_tags->replace_tags( $email_content_admin );
        $email_content_affiliate = $merge_tags->replace_tags( $email_content_affiliate );

        // Send admin emails.
        $admin_emails = array_filter( array_map( 'trim', explode( ',', slicewp_get_setting( 'admin_emails', '' ) ) ) );

        // Remove items that are not email addresses
        if ( ! empty( $admin_emails ) ) {

            foreach ( $admin_emails as $key => $value ) {

                if ( ! is_email( $value ) ) {
                    unset( $admin_emails[$key] );
                }

            }

            $admin_emails = array_values( $admin_emails );

        }

        if ( ! empty( $admin_emails ) ) {

            slicewp_wp_email( $admin_emails, $email_subject_admin, $email_content_admin );

        }

        // Send affiliate email.
        $user = get_user_by( 'id', $affiliate->get('user_id') );

	    if ( ! empty( $user->user_email ) ) {

            slicewp_wp_email( $user->user_email, $email_subject_affiliate, $email_content_affiliate );

        }

    }

}


/**
 * When a new commission is inserted or updated, trigger the generate bonus rewards function on the affiliate
 * linked to the commission.
 * 
 * @param int   $commission_id
 * @param array $commission_data
 * 
 * @return void
 * 
 */
function slicewp_insert_update_commission_listener_maybe_generate_reward( $commission_id, $commission_data ) {

    // Try to get the affiliate ID from the commission data.
    $affiliate_id = absint( ! empty( $commission_data['affiliate_id'] ) ? $commission_data['affiliate_id'] : 0 );

    // If we don't have the affiliate ID in the commission data,
    // pull the commission from the DB and grab the affiliate ID.
    if ( empty( $affiliate_id ) ) {

        $commission   = slicewp_get_commission( $commission_id );
        $affiliate_id = absint( ! is_null( $commission ) ? $commission->get( 'affiliate_id' ) : 0 );

    }

    // Bail if we don't have an affiliate ID by now.
    if ( empty( $affiliate_id ) ) {
        return;
    }

    slicewp_maybe_generate_affiliate_bonuses_rewards( absint( $affiliate_id ) );

}
add_action( 'slicewp_insert_commission', 'slicewp_insert_update_commission_listener_maybe_generate_reward', 50, 2 );
add_action( 'slicewp_update_commission', 'slicewp_insert_update_commission_listener_maybe_generate_reward', 50, 2 );


/**
 * Returns the start and end date based on the bonus' recurrence.
 * 
 * This function is marked as private. Do not use it to extend SliceWP.
 * 
 * @access private
 * 
 * @param int $bonus_id
 * 
 */
function _slicewp_get_bonus_date_range( $bonus_id ) {

    $bonuses = array_filter( (array)get_option( 'slicewp_performance_bonuses', array() ) );

    if ( empty( $bonuses[$bonus_id] ) ) {
        return null;
    }

    $bonus    = $bonuses[$bonus_id];
    $progress = array();

    // Determine start and end date based on the bonus' recurrence.
    $date_start = '';
    $date_end   = '';

    if ( 'one_time' == $bonus['recurrence'] ) {
        $date_start = $bonus['date_created'];
    }

    if ( 'monthly' == $bonus['recurrence'] ) {
        $date_start = gmdate( 'Y-m-01 00:00:00' );
        $date_end   = gmdate( 'Y-m-t 23:59:59' );
    }

    if ( 'quarterly' == $bonus['recurrence'] ) {

        $current_month = absint( gmdate( 'n' ) );
        $current_year  = absint( gmdate( 'Y' ) );

        $quarter_start_month = ceil( $current_month / 3 ) * 3 - 2;
        $quarter_end_month   = $quarter_start_month + 2;

        $quarter_start_month = str_pad( $quarter_start_month, 2, '0', STR_PAD_LEFT );
        $quarter_end_month   = str_pad( $quarter_end_month, 2, '0', STR_PAD_LEFT );

        $date_start = gmdate( 'Y-' . $quarter_start_month . '-01 00:00:00' );
        $date_end   = gmdate( 'Y-' . $quarter_end_month . '-' . ( gmdate( 't', strtotime( 'Y-' . $quarter_end_month . '-15' ) ) ) . ' 23:59:59' );

    }

    if ( 'yearly' == $bonus['recurrence'] ) {
        $date_start = gmdate( 'Y-01-01 00:00:00' );
        $date_end   = gmdate( 'Y-12-31 23:59:59' );
    }

    // Return.
    return array( 'date_start' => $date_start, 'date_end' => $date_end );

}