<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Base class for all recurring commissions integration objects.
 *
 */
class SliceWP_Integration_Recurring_Commissions {

	/**
	 * The slug for the entity that generated the commission.
	 *
	 * @access protected
	 * @var    string
	 *
	 */
	protected $origin;

	/**
	 * Constructor.
	 *
	 */
	public function __construct() {}


	/**
	 * Verifies if the recurrence limit has been reached for the given product, affiliate and commission parent.
	 *
	 * @param array $args
	 *
	 * @return bool
	 *
	 */
	protected function recurrence_limit_reached( $args ) {

		// Get the commission recurrence limit.
		$recurrence_limit = $this->get_recurrence_limit( $args );

		// Bail quickly if the recurrence limit is infinite.
		if ( false === $recurrence_limit ) {
			return false;
		}

		// Get the count for commissions under the same parent.
		$recurring_commissions_count = slicewp_get_commissions( array( 'parent_id' => $args['parent_id'], 'origin' => $this->origin, 'status' => array( 'paid', 'unpaid' ) ), true );

		return ( $recurring_commissions_count >= $recurrence_limit ? true : false );

	}


	/**
	 * Returns the affiliate's recurrence limit.
	 *
	 * @param int $affiliate_id
	 *
	 * @return null|false|int - Null if the default should be used, false for unlimited usages and int for limit.
	 *
	 */
	protected function get_affiliate_recurrence_limit( $affiliate_id ) {

		$recurrence_limit = null;
		
		$recurrence = slicewp_get_affiliate_meta( $affiliate_id, 'commissions_recurrence', true );

		if ( ! empty( $recurrence ) ) {

			$recurrence_limit = ( $recurrence == 'limited' ? absint( slicewp_get_affiliate_meta( $affiliate_id, 'commissions_recurrence_limit', true ) ) : false );

		}

		/**
		 * Filter the affiliate recurrence limit before returning it.
		 *
		 * @param null|int $recurrence_limit
		 * @param string   $origin
		 * @param int 	   $affiliate_id
		 *
		 */
		return apply_filters( 'slicewp_recurring_commissions_affiliate_recurrence_limit', $recurrence_limit, $this->origin, $affiliate_id );

	}


	/**
	 * Returns the product's/membership plan's recurrence limit.
	 *
	 * Subclasses should implement their own getter to overwrite this one.
	 *
	 * @param int $product_id
	 *
	 * @return null|false|int - Null if the default should be used, false for unlimited usages and int for limit.
	 *
	 */
	protected function get_product_recurrence_limit( $product_id ) { return null; }


	/**
	 * Returns the overall recurrence limit. Priority is given to the affiliate's limit, then to the product's
	 * and then to the general settings.
	 *
	 * @return null|false|int - Null if the default should be used, false for unlimited usages and int for limit.
	 *
	 */
	protected function get_recurrence_limit( $args ) {

		// Firstly use the recurrence limit of the product.
		$recurrence_limit = $this->get_product_recurrence_limit( $args['product_id'] );

		// If the affiliate has recurring commissions active, see if it also has the recurrence set.
		// If it does, set it to the returned value.
		if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {

			$affiliate_recurrence_limit = $this->get_affiliate_recurrence_limit( $args['affiliate_id'] );
			$recurrence_limit 		    = ( ! is_null( $affiliate_recurrence_limit ) ? $affiliate_recurrence_limit : $recurrence_limit );

		}

		// If there's no recurrence limit settings set on the product or on the affiliate, grab the recurrence limit settings set on the group
		// that the affiliate is part of (if they are part of a group).
		if ( is_null( $recurrence_limit ) ) {

			$affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

			if ( ! empty( $affiliate_group_id ) ) {

				if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
				
					$group_recurrence = slicewp_get_collection_meta( $affiliate_group_id, '_commissions_recurrence', true );

					if ( ! empty( $group_recurrence ) ) {

						$recurrence_limit = ( $group_recurrence == 'limited' ? absint( slicewp_get_collection_meta( $affiliate_group_id, '_commissions_recurrence_limit', true ) ) : false );
			
					}

				}

			}

		}

		// If the recurrence limit isn't explicitly set, set it to the default in settings.
		if ( is_null( $recurrence_limit ) ) {

			$default_recurrence 	  = slicewp_get_setting( 'commissions_recurrence', 'unlimited' );
			$default_recurrence_limit = slicewp_get_setting( 'commissions_recurrence_limit', 0 );

			$recurrence_limit = ( $default_recurrence == 'limited' ? absint( $default_recurrence_limit ) : false );

		}

		/**
		 * Filter the affiliate recurrence limit before returning it.
		 *
		 * @param null|int $recurrence_limit
		 * @param string   $origin
		 * @param array    $args
		 *
		 */
		return apply_filters( 'slicewp_recurring_commissions_recurrence_limit', $recurrence_limit, $this->origin, $args );

	}

}