<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions Easy Digital Downloads integration class
 *
 */
class SliceWP_Integration_Recurring_Commissions_Easy_Digital_Downloads extends SliceWP_Integration_Recurring_Commissions {

	/**
     * Constructor
     *
     */
    public function __construct() {

        $this->origin = 'edd';

        // Add the "recurring" commission type to Easy Digital Downloads supports array
        add_filter( 'slicewp_integration_supports_edd', array( $this, 'add_integration_supports' ) );

        // Adds a new recurring commission to the database
        add_action( 'edd_recurring_record_payment', array( $this, 'insert_recurring_commission' ), 10, 3 );

        // Add recurring commission rate settings in product/download page
        add_action( 'slicewp_edd_metabox_commission_settings_bottom', array( $this, 'add_product_recurring_commissions_settings' ) );

        // Save the product commission rate settings
        add_action( 'save_post_download', array( $this, 'save_product_recurring_commissions_settings' ), 15, 2 );

        // Calculate the recurring commission amount
        add_filter( 'slicewp_calculate_commission_amount', array( $this, 'calculate_commission_amount' ), 50, 3 );

    }


    /**
     * Add the "recurring" commission type to Easy Digital Downloads supports array
     *
     * @param array $supports
     *
     * @return array
     *
     */
    public function add_integration_supports( $supports ) {

        if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
            return $supports;
        }

        $supports['commission_types'][] = 'recurring';

        return $supports;

    }


    /**
     * Adds a new recurring commission to the database
     *
     * @param int    $payment_id
     * @param int    $parent_payment_id
     * @param string $amount
     *
     */
    public function insert_recurring_commission( $payment_id, $parent_payment_id, $amount ) {

        // Get the first commission, generated for the first subscription payment
        $parent_commission = slicewp_get_commissions( array( 'origin' => 'edd', 'reference' => $parent_payment_id, 'order' => 'ASC', 'parent_id' => 0 ) );
        $parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );

        // Bail if the commission is missing or is rejected
        if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {
            return;
        }

        // Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "EDD: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

        $recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

        // Bail if the affiliate has recurring commissions disabled
        if ( 'disabled' == $recurring_commissions_affiliate ) {

            slicewp_add_log( 'EDD: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
            return;

        }

        // Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

        // Get all cart items
        $cart_items = edd_get_payment_meta_cart_details( $payment_id );

        if ( ! is_array( $cart_items ) ) {

            slicewp_add_log( 'EDD: Pending commission was not created because the cart details were not valid.' );
            return;

        }

        $order = ( function_exists( 'edd_get_order' ) ? edd_get_order( $payment_id ) : edd_get_payment( $payment_id ) );

        $commission_amount = 0;

        // Go through each cart item and calculate the commission amount
        foreach ( $cart_items as $cart_item ) {

            /**
             * @todo - If all items from cart are disabled or for all items the recurrence limit has been reached,
             *         a zero amount commission will be inserted. Ideally, in this case, no commission should be generated.
             */

            // Get the product categories
            $categories = get_the_terms( $cart_item['id'], 'download_category' );

            // Verify if commissions are disabled for this product category
            if ( ! empty( $categories[0]->term_id ) && get_term_meta( $categories[0]->term_id, 'slicewp_disable_commissions', true ) ) {
                continue;
            }

            // Verify if commissions are disabled for this product
            if ( get_post_meta( $cart_item['id'], 'slicewp_disable_commissions', true ) ) {
                continue;
            }

            $recurring_commissions_product = get_post_meta( $cart_item['id'], 'slicewp_recurring_commissions', true );
            
            // Verify if product has recurring commissions disabled.
            if ( 'disabled' == $recurring_commissions_product ) {
                
                slicewp_add_log( 'EDD: Recurring commission was not calculated for this product because product has recurring commissions disabled. Product: ' . $cart_item['id'] );
                continue;

            }

            if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' && $recurring_commissions_product != 'custom' ) {
                continue;
            }

            // Verify if there are custom settings for the product/affiliate
            // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions
            if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $cart_item['id'] ) ) ) {
                continue;
            }

            // Verify if the recurrence limit has been reached
            if ( $this->recurrence_limit_reached( array( 'parent_id' => $parent_commission->get( 'id' ), 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $cart_item['id'] ) ) ) {

                slicewp_add_log( 'EDD: Recurring commission was not calculated for this product because the recurrence limit has been reached. Product: ' . $cart_item['id'] );
                continue;

            }

            $commission_basis = $cart_item['price'];

            // Exclude tax
            if ( slicewp_get_setting( 'exclude_tax', false ) ) {
                $commission_basis = $commission_basis - $cart_item['tax'];
            }

            // Add shipping fees if they exist
            if ( ! slicewp_get_setting( 'exclude_shipping', false ) ) {

                if ( ! empty( $cart_item['fees'] ) ) {

                    foreach ( $cart_item['fees'] as $key => $fee ) {

                        if ( empty( $fee['amount'] ) ) {
                            continue;
                        }

                        if ( false === strpos( $key, 'shipping' ) ) {
                            continue;
                        }
                            
                        $commission_basis = $commission_basis + $fee['amount'];

                    }

                }

            }

            // Calculate commission amount
            $args = array(
                'origin'       => 'edd',
                'type'         => 'recurring',
                'affiliate_id' => $parent_commission->get( 'affiliate_id' ),
                'product_id'   => $cart_item['id']
            );

            if ( function_exists( 'slicewp_maybe_convert_amount' ) ) {
                $commission_basis = slicewp_maybe_convert_amount( $commission_basis, $order->currency, slicewp_get_setting( 'active_currency', 'USD' ) );
            }

            $commission_amount += slicewp_calculate_commission_amount( $commission_basis, $args );

        }

        // Verify if there are custom settings for the product/affiliate
        // If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions
        if ( $commission_amount == 0 && empty( slicewp_get_setting( 'recurring_commissions' ) ) ) {

            slicewp_add_log( 'EDD: Recurring commission was not inserted because sitewide recurring commissions are disabled.' );
            return;

        }

        // Check that the commission amount is not zero
        if ( $commission_amount == 0 && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

            slicewp_add_log( 'EDD: Recurring commission was not inserted because the commission amount is zero.' );
            return;

        }

        // Prepare commission data
        $commission_data = array(
            'affiliate_id'		=> $parent_commission->get( 'affiliate_id' ),
            'parent_id'			=> $parent_commission->get( 'id' ),
            'visit_id'			=> 0,
            'date_created'		=> slicewp_mysql_gmdate(),
            'date_modified'		=> slicewp_mysql_gmdate(),
            'type'				=> 'recurring',
            'status'			=> 'unpaid',
            'reference'			=> $payment_id,
            'reference_amount'	=> slicewp_sanitize_amount( $amount ),
            'origin'			=> 'edd',
            'amount'			=> slicewp_sanitize_amount( $commission_amount ),
            'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
            'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
        );

        // Insert the commission
        $commission_id = slicewp_insert_commission( $commission_data );

        if ( ! empty( $commission_id ) ) {

            slicewp_add_log( sprintf( 'EDD: Recurring commission #%s has been successfully inserted.', $commission_id ) );
            
        } else {

            slicewp_add_log( 'EDD: Recurring commission could not be inserted due to an unexpected error.' );
            
        }

    }


    /**
     * Adds recurring commission rate settings in product/download page
     *
     */
    public function add_product_recurring_commissions_settings() {

        global $post;

        // Check if the recurring fields are hidden by a parent category or product
        $is_hidden_by_parent = false;

        // Get the product categories
        $categories = get_the_terms( $post->ID, 'download_category' );

        // Verify if commissions are disabled for category
        if ( ! empty( $categories[0]->term_id ) && get_term_meta( $categories[0]->term_id, 'slicewp_disable_commissions', true ) ) {
            $is_hidden_by_parent = true;
        }

        // Get recurring commissions status
        $recurring_commissions = get_post_meta( $post->ID, 'slicewp_recurring_commissions', true );

        // Get the current product settings
        $commission_rate_type = get_post_meta( $post->ID, 'slicewp_commission_rate_type_recurring', true );
        $commission_rate      = get_post_meta( $post->ID, 'slicewp_commission_rate_recurring', true );

        // Get the recurrence
        $recurrence       = get_post_meta( $post->ID, 'slicewp_commissions_recurrence', true );
        $recurrence_limit = get_post_meta( $post->ID, 'slicewp_commissions_recurrence_limit', true );

        ?>

        <div class="slicewp-options-group slicewp-show-if-subscription slicewp-hide-if-disabled-commissions <?php echo ( $is_hidden_by_parent ? 'slicewp-hidden-by-parent' : '' ); ?>">

            <p class="slicewp-option-field-wrapper form-row form-row-full">
                <label for="slicewp_recurring_commissions"><strong><?php echo __( 'Recurring Commissions', 'slicewp' ); ?></strong></label>
                <select name="slicewp_recurring_commissions" id="slicewp_recurring_commissions" class="slicewp-option-field-recurring-commissions">
                    <option value=""><?php echo __( 'Site Default', 'slicewp' ); ?></option>
                    <option value="custom" <?php selected( $recurring_commissions, 'custom' ); ?>><?php echo __( 'Custom', 'slicewp' ); ?></option>
                    <option value="disabled" <?php selected( $recurring_commissions, 'disabled' ); ?>><?php echo __( 'Disabled', 'slicewp' ); ?></option>
                </select>
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate-type slicewp-commission-type-recurring">
                <label for="slicewp_commission_rate_type_recurring"><strong><?php echo __( 'Recurring Rate Type', 'slicewp' ); ?></strong></label>
                <select name="slicewp_commission_rate_type_recurring" id="slicewp_commission_rate_type_recurring">
                    <option value=""><?php echo __( 'Site default', 'slicewp' ); ?></option>
                    <?php foreach( slicewp_get_commission_type_rate_types( 'recurring' ) as $rate_type => $rate_type_label ) : ?>
                        <option value="<?php echo esc_attr( $rate_type ); ?>"<?php selected( $commission_rate_type, $rate_type ); ?>><?php echo esc_html( $rate_type_label ); ?></option>
                    <?php endforeach; ?>
                </select>
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate slicewp-commission-type-recurring">
                <label for="slicewp_commission_rate_recurring"><strong><?php echo __( 'Recurring Rate', 'slicewp' ); ?></strong></label>
                <input type="text" size="5" name="slicewp_commission_rate_recurring" value="<?php echo esc_attr( $commission_rate ); ?>" id="slicewp_commission_rate_recurring" />
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence">
                <label for="slicewp_commissions_recurrence"><strong><?php echo __( 'Commissions Recurrence', 'slicewp' ); ?></strong></label>
                <select name="slicewp_commissions_recurrence" id="slicewp_commissions_recurrence">
                    <option value=""><?php echo __( 'Site default', 'slicewp' ); ?></option>
                    <option value="unlimited" <?php selected( $recurrence, 'unlimited' ); ?>><?php echo __( 'Unlimited', 'slicewp' ); ?></option>
                    <option value="limited" <?php selected( $recurrence, 'limited' ); ?>><?php echo __( 'Limited', 'slicewp' ); ?></option>
                </select>
            </p>

            <p class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence-limit">
                <label for="slicewp_commissions_recurrence_limit"><strong><?php echo __( 'Recurrence Limit', 'slicewp' ); ?></strong></label>
                <input type="number" name="slicewp_commissions_recurrence_limit" value="<?php echo esc_attr( $recurrence_limit ); ?>" id="slicewp_commissions_recurrence_limit" />
            </p>

        </div>

        <?php

    }


    /**
     * Saves the product recurring commissions data into the product meta
     * 
     * @param int     $post_id
     * @param WP_Post $post
     * 
     */
    public function save_product_recurring_commissions_settings( $post_id, $post ) {

        // Verify for nonce
        if ( empty( $_POST['slicewp_token'] ) || ! wp_verify_nonce( $_POST['slicewp_token'], 'slicewp_save_meta' ) ) {
            return $post_id;
        }

        // Verify for autosave
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
            return $post_id;
        }

        if ( wp_is_post_revision( $post_id ) || wp_is_post_autosave( $post_id ) ) {
            return $post_id;
        }

        // Update recurring commissions
        if ( ! empty( $_POST['slicewp_recurring_commissions'] ) ) {
            update_post_meta( $post_id, 'slicewp_recurring_commissions', sanitize_text_field( $_POST['slicewp_recurring_commissions'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_recurring_commissions' );
        }

        // Update recurring commission rate type
        if ( ! empty( $_POST['slicewp_commission_rate_type_recurring'] ) ) {
            update_post_meta( $post_id, 'slicewp_commission_rate_type_recurring', sanitize_text_field( $_POST['slicewp_commission_rate_type_recurring'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commission_rate_type_recurring' );
        }

        // Update recurring commission rate
        if ( ! empty( $_POST['slicewp_commission_rate_recurring'] ) ) {
            update_post_meta( $post_id, 'slicewp_commission_rate_recurring', sanitize_text_field( $_POST['slicewp_commission_rate_recurring'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commission_rate_recurring' );
        }

        // Update recurring commissions recurrence
        if( ! empty( $_POST['slicewp_commissions_recurrence'] ) ) {
            update_post_meta( $post_id, 'slicewp_commissions_recurrence', sanitize_text_field( $_POST['slicewp_commissions_recurrence'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commissions_recurrence' );
        }

        // Update recurring commissions recurrence limit
        if( isset( $_POST['slicewp_commissions_recurrence_limit'] ) ) {
            update_post_meta( $post_id, 'slicewp_commissions_recurrence_limit', absint( $_POST['slicewp_commissions_recurrence_limit'] ) );
        } else {
            delete_post_meta( $post_id, 'slicewp_commissions_recurrence_limit' );
        }

    }


    /**
     * Returns the product's recurrence limit
     *
     * @param int $product_id
     *
     * @return null|false|int - null if the default should be used, false for unlimited usages and int for limit
     *
     */
    protected function get_product_recurrence_limit( $product_id ) {

        $recurrence_limit = null;

        // Bail if recurring commissions settings aren't set for the product
        if ( 'custom' != get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {
            return $recurrence_limit;
        }

        // Get recurrence limit for the product/variation
        $recurrence = get_post_meta( $product_id, 'slicewp_commissions_recurrence', true );

        if ( ! empty( $recurrence ) ) {

            $recurrence_limit = ( $recurrence == 'limited' ? absint( get_post_meta( $product_id, 'slicewp_commissions_recurrence_limit', true ) ) : false );

        }

        return $recurrence_limit;

    }


    /**
     * Verifies if the given product and affiliate have custom recurring rate settings enabled
     *
     * @param array
     *
     * @return bool
     *
     */
    protected function has_custom_settings( $args ) {

        // Check for affiliate.
        if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
            return true;
        }

        // Check for product.
        if ( 'custom' == get_post_meta( $args['product_id'], 'slicewp_recurring_commissions', true ) ) {
            return true;
        }

        // Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

        return false;

    }


    /**
     * Calculates the recurring commission amount for products and variations
     * 
     * @param float $commission_amount
     * @param float $amount
     * @param array $args
     * 
     * @return float $commission_amount
     * 
     */
    public function calculate_commission_amount( $commission_amount, $amount, $args ) {

        // Check that the commission is from Easy Digital Downloads
        if ( $args['origin'] != $this->origin ) {
            return $commission_amount;
        }

        // Check for the recurring type
        if ( $args['type'] != 'recurring' ) {
            return $commission_amount;
        }

        // Get the product
        $product_id = absint( $args['product_id'] );

        $rate_type = '';
        $rate      = '';

        // Get the rate type and rate for the product
        if( 'custom' == get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {

            $rate_type = get_post_meta( $product_id, 'slicewp_commission_rate_type_recurring', true );
            $rate      = get_post_meta( $product_id, 'slicewp_commission_rate_recurring', true );

        }

        // Calculate the recurring commission amount
        if( ! empty( $rate_type ) && is_numeric( $rate ) ) {

            $commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

        }

        // Return the commission amount
        return $commission_amount;

    }

}

new SliceWP_Integration_Recurring_Commissions_Easy_Digital_Downloads();