<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions Restrict Content Pro integration class
 *
 */
class SliceWP_Integration_Recurring_Commissions_Restrict_Content_Pro extends SliceWP_Integration_Recurring_Commissions {

	/**
	 * Constructor
	 *
	 */
	public function __construct() {

		$this->origin = 'rcp';

		// Add the "recurring" commission type to Restrict Content Pro supports array
		add_filter( 'slicewp_integration_supports_rcp', array( $this, 'add_integration_supports' ) );

		// Adds a new recurring commission to the database
		add_action( 'rcp_create_payment', array( $this, 'insert_recurring_commission' ), -1, 2 );

		// Add recurring commission rate settings in the membership level page
		add_action( 'slicewp_rcp_commission_settings_bottom', array( $this, 'add_membership_level_recurring_commissions_settings' ) );

		// Save the membership level recurring commissions settings
		add_action( 'rcp_add_subscription', array( $this, 'save_membership_level_recurring_commissions_settings' ) );
		add_action( 'rcp_edit_subscription_level', array( $this, 'save_membership_level_recurring_commissions_settings' ) );

		// Calculate the recurring commission amount
		add_filter( 'slicewp_calculate_commission_amount', array( $this, 'calculate_commission_amount' ), 50, 3 );

	}


	/**
	 * Add the "recurring" commission type to Restrict Content Pro supports array
	 *
	 * @param array $supports
	 *
	 * @return array
	 *
	 */
	public function add_integration_supports( $supports ) {

		if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
			return $supports;
		}

		$supports['commission_types'][] = 'recurring';

		return $supports;

	}


	/**
	 * Adds a new recurring commission to the database
	 *
	 * @param int   $payment_id
	 * @param array $args
	 *
	 */
	public function insert_recurring_commission( $payment_id, $args ) {

		// Check if the transaction is a renewal
		if ( $args['transaction_type'] != 'renewal' ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because the payment is the first for the subscription. Payment: ' . var_export( $payment_id, true ) );
			return;

		}
		
		// Get the parent order
		$parent_order = $this->get_parent_order( $args['subscription_key'] );

		if ( is_null( $parent_order ) ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because the payment is the first for the subscription. Payment: ' . var_export( $payment_id, true ) );
			return;

		}

		// Get the first commission, generated for the first subscription payment
		$parent_commission = slicewp_get_commissions( array( 'origin' => 'rcp', 'reference' => $parent_order->id, 'order' => 'ASC', 'parent_id' => 0 ) );
		$parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );


		// Bail if the commission is missing or is rejected
		if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because parent commission is either missing or rejected. Payment: ' . var_export( $payment_id, true ) );
			return;

		}

		// Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "RCP: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

		$recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

		// Bail if the affiliate has recurring commissions disabled
		if ( 'disabled' == $recurring_commissions_affiliate ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
			return;

		}

		// Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

		$level_id = $parent_order->object_id;

		// Verify if commissions are disabled for this product
		if ( rcp_get_membership_level_meta( $level_id, 'slicewp_disable_commissions', true ) ) {
			return;
		}

		$recurring_commissions_product = rcp_get_membership_level_meta( $level_id, 'slicewp_recurring_commissions', true );
		
		// Verify if product has recurring commissions disabled.
		if ( 'disabled' == $recurring_commissions_product ) {
			
			slicewp_add_log( 'RCP: Recurring commission was not inserted for this membership level because it has recurring commissions disabled. Membership level: ' . $level_id );
			return;

		}

		if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' && $recurring_commissions_product != 'custom' ) {
            return;
        }

		// Verify if there are custom settings for the product/affiliate
		// If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions
		if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $level_id ) ) ) {
			return;
		}

		// Verify if the recurrence limit has been reached
		if ( $this->recurrence_limit_reached( array( 'parent_id' => $parent_commission->get( 'id' ), 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $level_id ) ) ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because the recurrence limit has been reached. Membership level: ' . $level_id );
			return;

		}

		// Get the order amount
		$amount = $args['amount'];
		
		// Calculate the commission amount for the entire order
		$args = array(
			'origin'       => 'rcp',
			'type'         => 'recurring',
			'affiliate_id' => $parent_commission->get( 'affiliate_id' ),
			'product_id'   => $level_id
		);

		if ( function_exists( 'slicewp_maybe_convert_amount' ) ) {
            $amount = slicewp_maybe_convert_amount( $amount, rcp_get_currency(), slicewp_get_setting( 'active_currency', 'USD' ) );
        }

		$commission_amount = slicewp_calculate_commission_amount( $amount, $args );

		// Verify if the commission amount is zero
		// If zero and recurring commissions aren't enabled sitewide, we don't generate commissions
		if ( $commission_amount == 0 && empty( slicewp_get_setting( 'recurring_commissions' ) ) ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because sitewide recurring commissions are disabled.' );
			return;

		}

		// Verify if the commission amount is zero
		// If zero and zero amount commissions registration is disabled, we don't generate commissions
		if ( $commission_amount == 0 && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

			slicewp_add_log( 'RCP: Recurring commission was not inserted because the commission amount is zero.' );
			return;

		}

		// Prepare commission data
		$commission_data = array(
			'affiliate_id'		=> $parent_commission->get( 'affiliate_id' ),
			'parent_id'			=> $parent_commission->get( 'id' ),
			'visit_id'			=> 0,
			'date_created'		=> slicewp_mysql_gmdate(),
			'date_modified'		=> slicewp_mysql_gmdate(),
			'type'				=> 'recurring',
			'status'			=> 'unpaid',
			'reference'			=> $payment_id,
			'reference_amount'	=> slicewp_sanitize_amount( $amount ),
			'origin'			=> 'rcp',
			'amount'			=> slicewp_sanitize_amount( $commission_amount ),
			'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
			'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
		);

		// Insert the commission
		$commission_id = slicewp_insert_commission( $commission_data );

		if ( ! empty( $commission_id ) ) {

			slicewp_add_log( sprintf( 'RCP: Recurring commission #%s has been successfully inserted.', $commission_id ) );
			
		} else {

			slicewp_add_log( 'RCP: Recurring commission could not be inserted due to an unexpected error.' );
			
		}

	}


	/**
	 * Adds recurring commission rate settings in the membership level page
	 *
	 */
	public function add_membership_level_recurring_commissions_settings() {

		// Get current membership level ID
		$level_id = ( ! empty( $_GET['edit_subscription'] ) ? absint( $_GET['edit_subscription'] ) : 0 );

		// Get the recurring settings for this membership level
		$settings = ( ! empty( $level_id ) ? get_option( 'slicewp_rcp_recurring_settings_' . $level_id, array() ) : array() );

		// Get recurring commissions status
		$recurring_commissions = rcp_get_membership_level_meta( $level_id, 'slicewp_recurring_commissions', true );

		// Get the current product settings
		$commission_rate_type = rcp_get_membership_level_meta( $level_id, 'slicewp_commission_rate_type_recurring', true );
		$commission_rate      = rcp_get_membership_level_meta( $level_id, 'slicewp_commission_rate_recurring', true );

		// Get the recurrence
		$recurrence       = rcp_get_membership_level_meta( $level_id, 'slicewp_commissions_recurrence', true );
		$recurrence_limit = rcp_get_membership_level_meta( $level_id, 'slicewp_commissions_recurrence_limit', true );

		?>

		<table class="slicewp-options-group slicewp-hide-if-disabled-commissions form-table">
			<tbody>

				<tr class="slicewp-option-field-wrapper form-field">
					<th scope="row" valign="top">
						<label for="slicewp_recurring_commissions"><?php echo __( 'Recurring Commissions', 'slicewp' ); ?></label>
					</th>
					<td>
						<select name="slicewp_recurring_commissions" id="slicewp_recurring_commissions" class="slicewp-option-field-recurring-commissions regular-text">
							<option value=""><?php echo __( 'Site Default', 'slicewp' ); ?></option>
							<option value="custom" <?php selected( $recurring_commissions, 'custom' ); ?>><?php echo __( 'Custom', 'slicewp' ); ?></option>
							<option value="disabled" <?php selected( $recurring_commissions, 'disabled' ); ?>><?php echo __( 'Disabled', 'slicewp' ); ?></option>
						</select>
					</td>
				</tr>

				<tr class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate-type slicewp-commission-type-recurring form-field">
					<th scope="row" valign="top">
						<label for="slicewp_commission_rate_type_recurring"><?php echo __( 'Recurring Rate Type', 'slicewp' ); ?></label>
					</th>
					<td>
						<select name="slicewp_commission_rate_type_recurring" id="slicewp_commission_rate_type_recurring" class="regular-text">
							<option value=""><?php echo __( 'Site default', 'slicewp' ); ?></option>
							<?php foreach( slicewp_get_commission_type_rate_types( 'recurring' ) as $rate_type => $rate_type_label ) : ?>
								<option value="<?php echo esc_attr( $rate_type ); ?>"<?php selected( $commission_rate_type, $rate_type ); ?>><?php echo esc_html( $rate_type_label ); ?></option>
							<?php endforeach; ?>
						</select>
					</td>
				</tr>

				<tr class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commission-rate slicewp-commission-type-recurring form-field">
					<th scope="row" valign="top">
						<label for="slicewp_commission_rate_recurring"><?php echo __( 'Recurring Rate', 'slicewp' ); ?></label>
					</th>
					<td>
						<input type="text" name="slicewp_commission_rate_recurring" value="<?php echo esc_attr( $commission_rate ); ?>" id="slicewp_commission_rate_recurring" class="regular-text" />
					</td>
				</tr>

				<tr class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence form-field">
					<th scope="row" valign="top">
						<label for="slicewp_commissions_recurrence"><?php echo __( 'Commissions Recurrence', 'slicewp' ); ?></label>
					</th>
					<td>
						<select name="slicewp_commissions_recurrence" id="slicewp_commissions_recurrence" class="regular-text">
							<option value=""><?php echo __( 'Site default', 'slicewp' ); ?></option>
							<option value="unlimited" <?php selected( $recurrence, 'unlimited' ); ?>><?php echo __( 'Unlimited', 'slicewp' ); ?></option>
							<option value="limited" <?php selected( $recurrence, 'limited' ); ?>><?php echo __( 'Limited', 'slicewp' ); ?></option>
						</select>
					</td>
				</tr>

				<tr class="slicewp-option-field-wrapper slicewp-option-field-wrapper-commissions-recurrence-limit form-field">
					<th scope="row" valign="top">
						<label for="slicewp_commissions_recurrence_limit"><?php echo __( 'Recurrence Limit', 'slicewp' ); ?></label>
					</th>
					<td>
						<input type="number" name="slicewp_commissions_recurrence_limit" value="<?php echo esc_attr( $recurrence_limit ); ?>" id="slicewp_commissions_recurrence_limit" class="regular-text" />
					</td>
				</tr>

			</tbody>
		</table>

		<?php

	}


	/**
	 * Saves the membership level recurring commissions settings
	 *
	 * @param int $level_id - the ID of the subscription plan
	 *
	 */
	function save_membership_level_recurring_commissions_settings( $level_id ) {

		// Update recurring commissions
		if ( ! empty( $_POST['slicewp_recurring_commissions'] ) ) {
			rcp_update_membership_level_meta( $level_id, 'slicewp_recurring_commissions', sanitize_text_field( $_POST['slicewp_recurring_commissions'] ) );
		} else {
			rcp_delete_membership_level_meta( $level_id, 'slicewp_recurring_commissions' );
		}

		// Update recurring commission rate type
		if ( ! empty( $_POST['slicewp_commission_rate_type_recurring'] ) ) {
			rcp_update_membership_level_meta( $level_id, 'slicewp_commission_rate_type_recurring', sanitize_text_field( $_POST['slicewp_commission_rate_type_recurring'] ) );
		} else {
			rcp_delete_membership_level_meta( $level_id, 'slicewp_commission_rate_type_recurring' );
		}

		// Update recurring commission rate
		if ( ! empty( $_POST['slicewp_commission_rate_recurring'] ) ) {
			rcp_update_membership_level_meta( $level_id, 'slicewp_commission_rate_recurring', sanitize_text_field( $_POST['slicewp_commission_rate_recurring'] ) );
		} else {
			rcp_delete_membership_level_meta( $level_id, 'slicewp_commission_rate_recurring' );
		}

		// Update recurring commissions recurrence
		if ( ! empty( $_POST['slicewp_commissions_recurrence'] ) ) {
			rcp_update_membership_level_meta( $level_id, 'slicewp_commissions_recurrence', sanitize_text_field( $_POST['slicewp_commissions_recurrence'] ) );
		} else {
			rcp_delete_membership_level_meta( $level_id, 'slicewp_commissions_recurrence' );
		}

		// Update recurring commissions recurrence limit
		if ( isset( $_POST['slicewp_commissions_recurrence_limit'] ) ) {
			rcp_update_membership_level_meta( $level_id, 'slicewp_commissions_recurrence_limit', absint( $_POST['slicewp_commissions_recurrence_limit'] ) );
		} else {
			rcp_delete_membership_level_meta( $level_id, 'slicewp_commissions_recurrence_limit' );
		}

	}


	/**
	 * Returns the initiating order for the subscription
	 *
	 * @param string $subscription_key
	 * 
	 * @return object
	 *
	 */
	public function get_parent_order( $subscription_key ) {

		if ( empty( $subscription_key ) ) {
			return null;
		}

		// Get the payments associated to the subscription key
		$payments = new RCP_Payments();

		$payments_args = array(
			's'			=> $subscription_key,
			'orderby'	=> 'id',
			'order'		=> 'ASC'
		);

		$payments = $payments->get_payments( $payments_args );

		if ( empty( $payments ) ) {
			return null;
		}

		return $payments[0];

	}


    /**
	 * Returns the membership level's recurrence limit
	 *
	 * @param int $product_id - the membership level id
	 *
	 * @return null|false|int - null if the default should be used, false for unlimited usages and int for limit
	 *
	 */
	protected function get_product_recurrence_limit( $product_id ) {

		$recurrence_limit = null;

		// Bail if recurring commissions settings aren't set for the product
		if ( 'custom' != rcp_get_membership_level_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {
			return $recurrence_limit;
		}

		// Get recurrence limit for the product/variation
		$recurrence = rcp_get_membership_level_meta( $product_id, 'slicewp_commissions_recurrence', true );

		if ( ! empty( $recurrence ) ) {

			$recurrence_limit = ( $recurrence == 'limited' ? absint( rcp_get_membership_level_meta( $product_id, 'slicewp_commissions_recurrence_limit', true ) ) : false );

		}

		return $recurrence_limit;

	}


	/**
	 * Verifies if the given product and affiliate have custom recurring rate settings enabled.
	 *
	 * @param array
	 *
	 * @return bool
	 *
	 */
	protected function has_custom_settings( $args ) {

		// Check for affiliate.
		if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
			return true;
		}

		// Check for product.
		if ( 'custom' == rcp_get_membership_level_meta( $args['product_id'], 'slicewp_recurring_commissions', true ) ) {
			return true;
		}

		// Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

		return false;

	}


	/**
	 * Calculates the recurring commission amount for products and variations.
	 * 
	 * @param float $commission_amount
	 * @param float $amount
	 * @param array $args
	 * 
	 * @return float $commission_amount
	 * 
	 */
	public function calculate_commission_amount( $commission_amount, $amount, $args ) {

		// Check that the commission is from Restrict Content Pro
		if ( $args['origin'] != $this->origin ) {
			return $commission_amount;
		}

		// Check for the recurring type
		if ( $args['type'] != 'recurring' ) {
			return $commission_amount;
		}

		// Get the product
		$product_id = absint( $args['product_id'] );

		$rate_type = '';
		$rate      = '';

		// Get the rate type and rate for the product
		if ( 'custom' == rcp_get_membership_level_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {

			$rate_type = rcp_get_membership_level_meta( $product_id, 'slicewp_commission_rate_type_recurring', true );
			$rate      = rcp_get_membership_level_meta( $product_id, 'slicewp_commission_rate_recurring', true );

		}

		// Calculate the recurring commission amount
		if ( ! empty( $rate_type ) && is_numeric( $rate ) ) {

			$commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

		}

		// Return the commission amount
		return $commission_amount;

	}

}

new SliceWP_Integration_Recurring_Commissions_Restrict_Content_Pro();