<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Recurring commissions Upsell integration class
 *
 */
class SliceWP_Integration_Recurring_Commissions_Upsell extends SliceWP_Integration_Recurring_Commissions {

	/**
	 * Constructor
	 *
	 */
	public function __construct() {

		$this->origin = 'upsell';

		// Add the "recurring" commission type to Upsell supports array
		add_filter( 'slicewp_integration_supports_upsell', array( $this, 'add_integration_supports' ) );

		// Adds a new recurring commission to the database
		add_action( 'upsell_subscription_order_renewed', array( $this, 'insert_recurring_commission' ), -1 );

		// Add recurring commission rate settings in the product page
		add_filter( 'slicewp_upsell_product_field_group_commission_settings_bottom', array( $this, 'add_product_recurring_commissions_settings' ), 20, 2 );

		// Calculate the recurring commission amount
		add_filter( 'slicewp_calculate_commission_amount', array( $this, 'calculate_commission_amount' ), 50, 3 );

	}


	/**
	 * Add the "recurring" commission type to Upsell supports array
	 *
	 * @param array $supports
	 *
	 * @return array
	 *
	 */
	public function add_integration_supports( $supports ) {

		if ( empty( $supports['commission_types'] ) || ! is_array( $supports['commission_types'] ) ) {
			return $supports;
		}

		$supports['commission_types'][] = 'recurring';

		return $supports;

	}


	/**
	 * Adds a new recurring commission to the database.
	 *
	 * @param Upsell\Entities\Order $order
	*
	*/
	public function insert_recurring_commission( $order ) {

		// Get the parent order id		
		$parent_order_id = get_post_meta( $order->getId(), '_parent_order', true );

		if ( empty( $parent_order_id ) ) {
			return;
		}

		// Get the parent commission
		$parent_commission = slicewp_get_commissions( array( 'origin' => 'upsell', 'reference' => $parent_order_id, 'order' => 'ASC', 'parent_id' => 0 ) );
		$parent_commission = ( ! empty( $parent_commission ) ? $parent_commission[0] : null );

		// Bail if the commission is missing or is rejected
		if ( is_null( $parent_commission ) || $parent_commission->get( 'status' ) == 'rejected' ) {
			return;
		}

		// Bail if the affiliate doesn't exist or is inactive.
        $affiliate = slicewp_get_affiliate( absint( $parent_commission->get( 'affiliate_id' ) ) );

        if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {

            slicewp_add_log( "UPSELL: Recurring commission was not inserted because the affiliate either doesn't exist or is not active." );
            return;

        }

		$recurring_commissions_affiliate = slicewp_get_affiliate_meta( $affiliate->get( 'id' ), 'recurring_commissions', true );

		// Bail if the affiliate has recurring commissions disabled.
		if ( 'disabled' == $recurring_commissions_affiliate ) {

			slicewp_add_log( 'UPSELL: Recurring commission was not inserted because the affiliate has recurring commissions disabled.' );
			return;

		}

		// Get affiliate group settings.
        $affiliate_group_id          = slicewp_get_affiliate_group_id( $affiliate->get( 'id' ) );
        $recurring_commissions_group = ( ! empty( $affiliate_group_id ) ? slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) : '' );

		$commission_amount = 0;

		// Calculate the commission amount for each item in the cart.
		foreach ( $order->getItems() as $cart_item ) {

			/**
             * @todo - If all items from cart are disabled or for all items the recurrence limit has been reached,
             *         a zero amount commission will be inserted. Ideally, in this case, no commission should be generated.
             */

			// Verify if commissions are disabled for this product.
			if ( get_post_meta( $cart_item['id'], 'slicewp_disable_commissions', true ) ) {
				continue;
			}

			$recurring_commissions_product = get_post_meta( $cart_item['id'], 'slicewp_recurring_commissions', true );
			
			// Verify if product has recurring commissions disabled.
			if ( 'disabled' == $recurring_commissions_product ) {
				
				slicewp_add_log( 'UPSELL: Recurring commission was not inserted for this product because it has recurring commissions disabled. Product: ' . $cart_item['id'] );
				continue;

			}

			if ( 'disabled' == $recurring_commissions_group && $recurring_commissions_affiliate != 'custom' && $recurring_commissions_product != 'custom' ) {
                continue;
            }

			// Verify if there are custom settings for the product/affiliate.
			// If there aren't and recurring commissions aren't enabled sitewide, we don't generate commissions.
			if ( empty( slicewp_get_setting( 'recurring_commissions' ) ) && ! $this->has_custom_settings( array( 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $cart_item['id'] ) ) ) {
				continue;
			}

			// Verify if the recurrence limit has been reached.
			if ( $this->recurrence_limit_reached( array( 'parent_id' => $parent_commission->get( 'id' ), 'affiliate_id' => $parent_commission->get( 'affiliate_id' ), 'product_id' => $cart_item['id'] ) ) ) {

				slicewp_add_log( 'UPSELL: Recurring commission was not inserted because the recurrence limit has been reached. Product: ' . $cart_item['id'] );
				continue;

			}

			// Get the order amount
			$amount = $cart_item['total'];

			// Exclude tax
			if ( slicewp_get_setting( 'exclude_tax', false ) ) {
				$amount -= $cart_item['tax_total'];
			}
			
			// Calculate the commission amount for the entire order
			$args = array(
				'origin'       => 'upsell',
				'type'         => 'recurring',
				'affiliate_id' => $parent_commission->get( 'affiliate_id' ),
				'product_id'   => $cart_item['id']
			);

			$commission_amount += slicewp_calculate_commission_amount( $amount, $args );

		}

		// Verify if the commission amount is zero
		// If zero and recurring commissions aren't enabled sitewide, we don't generate commissions
		if ( $commission_amount == 0 && empty( slicewp_get_setting( 'recurring_commissions' ) ) ) {

			slicewp_add_log( 'UPSELL: Recurring commission was not inserted because sitewide recurring commissions are disabled.' );
			return;

		}

		// Verify if the commission amount is zero
		// If zero and zero amount commissions registration is disabled, we don't generate commissions
		if ( $commission_amount == 0 && empty( slicewp_get_setting( 'zero_amount_commissions' ) ) ) {

			slicewp_add_log( 'UPSELL: Recurring commission was not inserted because the commission amount is zero.' );
			return;

		}

		// Prepare commission data
		$commission_data = array(
			'affiliate_id'		=> $parent_commission->get( 'affiliate_id' ),
			'parent_id'			=> $parent_commission->get( 'id' ),
			'visit_id'			=> 0,
			'date_created'		=> slicewp_mysql_gmdate(),
			'date_modified'		=> slicewp_mysql_gmdate(),
			'type'				=> 'recurring',
			'status'			=> 'unpaid',
			'reference'			=> $order->getId(),
			'reference_amount'	=> slicewp_sanitize_amount( $order->getTotal() ),
			'origin'			=> 'upsell',
			'amount'			=> slicewp_sanitize_amount( $commission_amount ),
			'currency'			=> slicewp_get_setting( 'active_currency', 'USD' ),
			'customer_id'       => ( ! empty( $parent_commission->get( 'customer_id' ) ) ? absint( $parent_commission->get( 'customer_id' ) ) : 0 )
		);

		// Insert the commission
		$commission_id = slicewp_insert_commission( $commission_data );

		if ( ! empty( $commission_id ) ) {

			slicewp_add_log( sprintf( 'UPSELL: Recurring commission #%s has been successfully inserted.', $commission_id ) );
			
		} else {

			slicewp_add_log( 'UPSELL: Recurring commission could not be inserted due to an unexpected error.' );
			
		}

	}


	/**
	 * Adds recurring commission rate settings in the product page
	 *
	 * @param array $fieldGroup
	 * @param int $id
	 * 
	 * @return array
	 * 
	 */
	public function add_product_recurring_commissions_settings( $fieldGroup, $id ) {

		// Add the recurring commission field
		$fields['slicewp_recurring_commissions'] =	array(
			'key' => 'slicewp_recurring_commissions',
			'label' => __( 'Recurring Commissions', 'slicewp' ),
			'name' => 'slicewp_recurring_commissions',
			'type' => 'select',
			'choices' => array(
							''			=> __( 'Site Default', 'slicewp' ),
							'custom'	=> __( 'Custom', 'slicewp' ),
							'disabled'	=> __( 'Disabled', 'slicewp' )
						),
			'instructions' => __( 'Customize or disable the recurring commissions for this subscription plan.' ,'slicewp' ),
			'required' => 0,
			'conditional_logic' => array(
					array(
						'field' => 'slicewp_disable_commissions',
						'operator' => '==',
						'value' => '0',
					),
					array(
						'field' => 'field_5cb6c00780eff',
						'operator' => '==',
						'value' => 'subscription',
					),
			),
			'wrapper' => array(
				'width' => '',
				'class' => '',
				'id' => '',
			),
			'message' => '',
			'default_value' => '',
			'ui' => 0,
			'ui_on_text' => '',
			'ui_off_text' => '',
		);

		// Prepare the commission types for select field
		$commissions_select[''] = __( 'Site Default', 'slicewp' );
		
		foreach ( slicewp_get_commission_type_rate_types( 'recurring' ) as $rate_type => $rate_type_label ) {
			$commissions_select[$rate_type] = $rate_type_label;
		}

		// Add the recurring commission rate type field
		$fields['slicewp_commission_rate_type_recurring'] =	array(
			'key' => 'slicewp_commission_rate_type_recurring',
			'label' => __( 'Recurring Rate Type', 'slicewp' ),
			'name' => 'slicewp_commission_rate_type_recurring',
			'type' => 'select',
			'choices' => $commissions_select,
			'instructions' => __( 'The selected commission rate type will be used to calculate the commission for this product.', 'slicewp' ),
			'required' => 0,
			'conditional_logic' => array(
					array(
						'field' => 'slicewp_recurring_commissions',
						'operator' => '==',
						'value' => 'custom',
					)
			),
			'wrapper' => array(
				'width' => '',
				'class' => '',
				'id' => '',
			),
			'message' => '',
			'default_value' => '',
			'ui' => 0,
			'ui_on_text' => '',
			'ui_off_text' => '',
		);

		// Add the recurring commission rate field
		$fields['slicewp_commission_rate_recurring'] = array(
			'key' => 'slicewp_commission_rate_recurring',
			'label' => __( 'Recurring Rate', 'slicewp' ),
			'name' => 'slicewp_commission_rate_recurring',
			'type' => 'number',
			'instructions' => __( 'This rate will be used to calculate the commission for this product.', 'slicewp' ),
			'required' => 0,
			'conditional_logic' => array(
					array(
						'field' => 'slicewp_commission_rate_type_recurring',
						'operator' => '!=',
						'value' => '',
					)
				),
			'wrapper' => array(
				'width' => '',
				'class' => '',
				'id' => '',
			),
			'message' => '',
			'ui' => 0,
			'ui_on_text' => '',
			'ui_off_text' => '',
		);

		// Add the commission recurrence
		$fields['slicewp_commissions_recurrence'] =	array(
			'key' => 'slicewp_commissions_recurrence',
			'label' => __( 'Commissions Recurrence', 'slicewp' ),
			'name' => 'slicewp_commissions_recurrence',
			'type' => 'select',
			'choices' => array(
				''			=> __( 'Site Default', 'slicewp' ),
				'unlimited'	=> __( 'Unlimited', 'slicewp' ),
				'limited'	=> __( 'Limited', 'slicewp' )
			),
			'instructions' => __( 'Select whether the affiliate should be rewarded for all recurring payments for this subscription product or for a limited number of recurring payments.', 'slicewp' ),
			'required' => 0,
			'conditional_logic' => array(
					array(
						'field' => 'slicewp_recurring_commissions',
						'operator' => '==',
						'value' => 'custom',
					)
			),
			'wrapper' => array(
				'width' => '',
				'class' => '',
				'id' => '',
			),
			'message' => '',
			'default_value' => __( 'Site Default', 'slicewp' ),
			'ui' => 0,
			'ui_on_text' => '',
			'ui_off_text' => '',
		);

		// Add the commission recurrence limit
		$fields['slicewp_commissions_recurrence_limit'] = array(
			'key' => 'slicewp_commissions_recurrence_limit',
			'label' => __( 'Recurring Limit', 'slicewp' ),
			'name' => 'slicewp_commissions_recurrence_limit',
			'type' => 'number',
			'instructions' => __( 'Set the maximum number of commissions an affiliate is rewarded for recurring payments made by this subscription product.', 'slicewp' ),
			'required' => 0,
			'conditional_logic' => array(
					array(
						'field' => 'slicewp_commissions_recurrence',
						'operator' => '==',
						'value' => 'limited',
					)
				),
			'wrapper' => array(
				'width' => '',
				'class' => '',
				'id' => '',
			),
			'message' => '',
			'ui' => 0,
			'ui_on_text' => '',
			'ui_off_text' => '',
		);

		$fieldGroup['fields'] = array_merge( $fieldGroup['fields'], $fields );
		
		return $fieldGroup;

	}


	/**
	 * Returns the product's recurrence limit
	 *
	 * @param int $product_id - the product id
	 *
	 * @return null|false|int - null if the default should be used, false for unlimited usages and int for limit
	 *
	 */
	protected function get_product_recurrence_limit( $product_id ) {

		$recurrence_limit = null;

		// Bail if recurring commissions settings aren't set for the product
		if ( 'custom' != get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {
			return $recurrence_limit;
		}

		// Get recurrence limit for the product
		$recurrence = get_post_meta( $product_id, 'slicewp_commissions_recurrence', true );

		if ( ! empty( $recurrence ) ) {

			$recurrence_limit = ( $recurrence == 'limited' ? absint( get_post_meta( $product_id, 'slicewp_commissions_recurrence_limit', true ) ) : false );

		}

		return $recurrence_limit;

	}


	/**
	 * Verifies if the given product and affiliate have custom recurring rate settings enabled.
	 *
	 * @param array
	 *
	 * @return bool
	 *
	 */
	protected function has_custom_settings( $args ) {

		// Check for affiliate.
		if ( 'custom' == slicewp_get_affiliate_meta( $args['affiliate_id'], 'recurring_commissions', true ) ) {
			return true;
		}

		// Check for product.
		if ( 'custom' == get_post_meta( $args['product_id'], 'slicewp_recurring_commissions', true ) ) {
			return true;
		}

		// Check for affiliate group.
        $affiliate_group_id = slicewp_get_affiliate_group_id( absint( $args['affiliate_id'] ) );

        if ( ! empty( $affiliate_group_id ) ) {

            if ( 'custom' == slicewp_get_collection_meta( $affiliate_group_id, '_recurring_commissions', true ) ) {
                return true;
            }

        }

		return false;

	}


	/**
	 * Calculates the recurring commission amount for products and variations
	 * 
	 * @param float $commission_amount
	 * @param float $amount
	 * @param array $args
	 * 
	 * @return float $commission_amount
	 * 
	 */
	public function calculate_commission_amount( $commission_amount, $amount, $args ) {

		// Check that the commission is from Upsell
		if ( $args['origin'] != $this->origin ) {
			return $commission_amount;
		}

		// Check for the recurring type
		if ( $args['type'] != 'recurring' ) {
			return $commission_amount;
		}

		// Get the product
		$product_id = absint( $args['product_id'] );

		$rate_type = '';
		$rate      = '';

		// Get the rate type and rate for the product
		if ( 'custom' == get_post_meta( $product_id, 'slicewp_recurring_commissions', true ) ) {

			$rate_type = get_post_meta( $product_id, 'slicewp_commission_rate_type_recurring', true );
			$rate      = get_post_meta( $product_id, 'slicewp_commission_rate_recurring', true );

		}

		// Calculate the recurring commission amount
		if ( ! empty( $rate_type ) && is_numeric( $rate ) ) {

			$commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

		}

		// Return the commission amount
		return $commission_amount;

	}

}

new SliceWP_Integration_Recurring_Commissions_Upsell();