<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * REST API controller for "visits".
 * 
 */
class SliceWP_REST_API_Controller_Visits extends SliceWP_REST_API_Controller {

    /**
	 * Route base.
	 *
	 * @var    string
     * @access protected
     * 
	 */
    protected $rest_base = 'visits';


    /**
     * Registers the routes.
     * 
     */
    public function register_routes() {

        // Get all visits.
        register_rest_route( $this->namespace, '/' . $this->rest_base, array(
			'methods' 			  => WP_REST_Server::READABLE,
			'callback'			  => array( $this, 'get_items' ),
			'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => $this->get_collection_params()
		));

        // Get single visits.
		register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
			'methods'			  => WP_REST_Server::READABLE,
			'callback'			  => array( $this, 'get_item' ),
			'permission_callback' => array( $this, 'permissions_check' )
		));

        // Create single visit.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/', array(
            'methods'             => WP_REST_Server::CREATABLE,
            'callback'			  => array( $this, 'create_item' ),
            'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => array(
                'affiliate_id' => array(
                    'description'       => __( 'The ID of the affiliate that referred the visit.', 'slicewp' ),
                    'type'              => 'integer',
                    'required'          => true,
                    'validate_callback' => 'rest_validate_request_arg'
                )
            )
        ));

        // Update single visit.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
            'methods'             => WP_REST_Server::EDITABLE,
            'callback'			  => array( $this, 'update_item' ),
            'permission_callback' => array( $this, 'permissions_check' ),
            'args'                => array(
                'affiliate_id' => array(
                    'description'       => __( 'The ID of the affiliate that referred the visit.', 'slicewp' ),
                    'type'              => 'integer',
                    'validate_callback' => 'rest_validate_request_arg'
                )
            )
        ));

        // Delete single visit.
        register_rest_route( $this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
            'methods'             => WP_REST_Server::DELETABLE,
            'callback'			  => array( $this, 'delete_item' ),
            'permission_callback' => array( $this, 'permissions_check' )
        ));

    }


    /**
     * Returns an array of visits data as a rest response.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function get_items( $request ) {

        // Prepare query args.
        $args = array();

        foreach ( $request->get_params() as $key => $value ) {

            if ( in_array( $key, array_keys( $this->get_collection_params() ) ) ) {
                $args[$key] = $value;
            }

        }

        // Get visits.
        $visits = slicewp_get_visits( $args );

        foreach ( $visits as $key => $visit ) {
            $visits[$key] = (object)$visit->to_array();
        }

        return rest_ensure_response( $visits );

    }


    /**
     * Returns the visit with the given ID as a rest response.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function get_item( $request ) {

        $visit = slicewp_get_visit( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $visit ) ) {
            return new WP_Error( 'invalid_visit_id', 'Visit does not exist.', array( 'status' => 404 ) );
        }

        return rest_ensure_response( (object)$visit->to_array() );

    }


    /**
     * Creates a new visit.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function create_item( $request ) {

        // Prepare item data.
        $visit_data = $this->prepare_item_data_for_database( $request );

        // If there's no data to create object, return.
        if ( empty( $visit_data ) ) {
            return new WP_Error( 'visit_create_false', 'Did not create visit. No writable data provided.', array( 'status' => 400 ) );
        }

        // Add dates.
        $visit_data['date_created']  = slicewp_mysql_gmdate();
        $visit_data['date_modified'] = slicewp_mysql_gmdate();

        // Insert the item.
        $inserted = slicewp_insert_visit( $visit_data );

        if ( $inserted ) {

            $visit = slicewp_get_visit( $inserted );

            return rest_ensure_response( (object)$visit->to_array() );

        }

        return new WP_Error( 'visit_create_false', 'Something went wrong. Could not create visit.', array( 'status' => 500 ) );

    }


    /**
     * Updates the visit with the given ID.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function update_item( $request ) {
        
        $visit = slicewp_get_visit( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $visit ) ) {
            return new WP_Error( 'invalid_visit_id', 'Visit does not exist.', array( 'status' => 404 ) );
        }

        // Prepare item data.
        $visit_data = $this->prepare_item_data_for_database( $request );

        // If there's no data to update, return.
        if ( empty( $visit_data ) ) {
            return new WP_Error( 'visit_update_false', 'Did not update visit. No writable data provided.', array( 'status' => 400 ) );
        }

        // Add date modified before updating.
        $visit_data['date_modified'] = slicewp_mysql_gmdate();

        // Update the item.
        $updated = slicewp_update_visit( $visit->get( 'id' ), $visit_data );

        if ( $updated ) {

            $visit = slicewp_get_visit( absint( $request->get_param( 'id' ) ) );

            return rest_ensure_response( (object)$visit->to_array() );

        }

        return new WP_Error( 'visit_update_false', 'Something went wrong. Could not update visit.', array( 'status' => 500 ) );

    }


    /**
     * Delete the visit with the given ID.
     * 
     * @param WP_REST_Request $request
     * 
     */
    public function delete_item( $request ) {

        $visit = slicewp_get_visit( absint( $request->get_param( 'id' ) ) );

        if ( is_null( $visit ) ) {
            return new WP_Error( 'invalid_visit_id', 'Visit does not exist.', array( 'status' => 404 ) );
        }

        $deleted = slicewp_delete_visit( $visit->get( 'id' ) );

        if ( $deleted ) {
            return rest_ensure_response( (object)$visit->to_array() );
        }

        return new WP_Error( 'visit_delete_false', 'Something went wrong. Could not delete visit.', array( 'status' => 500 ) );

    }


    /**
     * Filters and sanitizes the request data to prepare it for the database.
     * 
     * @param WP_REST_Request $request
     * 
     * @return array
     * 
     */
    protected function prepare_item_data_for_database( $request ) {

        $schema    = $this->get_item_schema();
        $data_keys = array_keys( array_filter( $schema['properties'], array( $this, 'filter_writable_properties' ) ) );

        $data = array();

        foreach ( $data_keys as $key ) {

            $value = $request->get_param( $key );

            if ( is_null( $value ) ) {
                continue;
            }

            if ( $key == 'affiliate_id' ) {
                $data[$key] = absint( $value );
            }

            if ( $key == 'landing_url' ) {
                $data[$key] = slicewp_sanitize_visit_landing_url( esc_url_raw( $value ) );
            }

            if ( $key == 'referrer_url' ) {
                $data[$key] = esc_url_raw( $value );
            }

            if ( $key == 'commission_id' ) {
                $data[$key] = absint( $value );
            }

        }

        return $data;

    }


    /**
     * Get the visit's schema, conforming to JSON Schema.
     * 
     * @return array
     * 
     */
    public function get_item_schema() {

        $schema = array(
            '$schema'    => 'http://json-schema.org/draft-04/schema#',
            'title'      => 'visit',
            'type'       => 'object',
            'properties' => array(
                'id' => array(
                    'description' => esc_html( __( 'Unique identifier for the object.', 'slicewp' ) ),
                    'type'        => 'integer',
                    'readonly'    => true
                ),
                'affiliate_id' => array(
                    'description' => esc_html( __( 'The unique identifier of the affiliate that referred the visit.', 'slicewp' ) ),
                    'type'        => 'integer'
                ),
                'date_created' => array(
                    'description' => esc_html( __( 'The date at which the item was created, in the GMT timezone.', 'slicewp' ) ),
                    'type'        => 'date-time',
                    'readonly'    => true
                ),
                'date_modified' => array(
                    'description' => esc_html( __( 'The date at which the item was last modified, in the GMT timezone.', 'slicewp' ) ),
                    'type'        => 'date-time',
                    'readonly'    => true
                ),
                'ip_address' => array(
                    'description' => esc_html( __( 'The IP of the referred visitor.', 'slicewp' ) ),
                    'type'        => 'string'
                ),
                'landing_url' => array(
                    'description' => esc_html( __( 'The URL the visitor landed on.', 'slicewp' ) ),
                    'type'        => 'string'
                ),
                'referrer_url' => array(
                    'description' => esc_html( __( 'The URL the referred visitor came from.', 'slicewp' ) ),
                    'type'        => 'string'
                ),
                'commission_id' => array(
                    'description' => esc_html( __( 'The unique ID of the commission that resulted from this visit.', 'slicewp' ) ),
                    'type'        => 'integer'
                )
            )
        );

        return $schema;

    }


    /**
     * Get the query params for collections.
     *
     * @return array
     * 
     */
    public function get_collection_params() {

        return array(
            'number' => array(
                'description'       => __( 'The number of items to query for. Use -1 to query for all.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => function( $param ) {
                    return (int)$param;
                },
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'offset' => array(
                'description'       => __( 'The number of items to offset the query by.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => 'absint',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'affiliate_id' => array(
                'description'       => __( 'The affiliate ID to query by.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => 'absint',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'commission_id' => array(
                'description'       => __( 'The commission ID to query by.', 'slicewp' ),
                'type'              => 'integer',
                'sanitize_callback' => 'absint',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'converted' => array(
                'description'       => __( 'Whether the visit was converted or not.', 'slicewp' ),
                'type'              => 'boolean',
                'sanitize_callback' => function( $param ) {
                    return filter_var( $param, FILTER_VALIDATE_BOOLEAN );
                },
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'date_min' => array(
                'description'       => __( 'The minimum creation date of the visit to query by.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'date_max' => array(
                'description'       => __( 'The maximum creation date of the visit to query by.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            ),
            'order' => array(
                'description'       => __( 'How to order the results. Use ASC for ascending order or DESC for descending order.', 'slicewp' ),
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field',
                'validate_callback' => 'rest_validate_request_arg'
            )
        );

    }

}

new SliceWP_REST_API_Controller_Visits();