<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Register "Store Credit" payout method
 *
 * @param array $payout_methods
 *
 * @return array
 *
 */
function slicewp_register_payout_method_store_credit( $payout_methods ) {

	if ( ! is_array( $payout_methods ) ) {
        return array();
    }

	$payout_methods['store_credit'] = array(
		'label'    => __( 'Store Credit', 'slicewp' ),
		'supports' => array( 'single_payment', 'bulk_payments' ),
		'messages' => array(
			'warning_woocommerce_only'				    => sprintf( __( '%sImportant: %s Please note that rewarding your affiliates with store credit works only with WooCommerce.', 'slicewp' ), '<strong>', '</strong>' ),
            'payout_action_confirmation_single_payment' => __( "This will mark this payment as paid, add the payment amount to the affiliate's store credit balance and mark all associated commissions as paid. Are you sure you want to continue?", 'slicewp' ),
			'payout_action_confirmation_bulk_payments'  => __( 'This will mark all unpaid payments for this payout as paid. All commissions associated with these payments will also be marked as paid. Are you sure you want to continue?', 'slicewp' )
		)
	);

	return $payout_methods;

}
add_filter( 'slicewp_register_payout_methods', 'slicewp_register_payout_method_store_credit', 50 );


/**
 * Returns the store credit balance of a user.
 * 
 * @param int $user_id
 * 
 * @return float
 * 
 */
function slicewp_get_user_store_credit_balance( $user_id ) {

    return (float)get_user_meta( $user_id, 'slicewp_store_credit_balance', true );

}


/**
 * Updates the store credit balance for the user.
 * 
 * @param int   $user_id
 * @param float $amount
 * 
 * @return bool
 * 
 */
function slicewp_update_user_store_credit_balance( $user_id, $amount ) {

    return (bool)update_user_meta( $user_id, 'slicewp_store_credit_balance', slicewp_sanitize_amount( $amount ) );

}


/**
 * Increases the store credit balance of a user with the given amount.
 * 
 * @param int   $user_id
 * @param float $amount
 * 
 * @return bool
 * 
 */
function slicewp_increase_user_store_credit_balance( $user_id, $amount ) {

    $balance = slicewp_get_user_store_credit_balance( $user_id ) + $amount;

    return slicewp_update_user_store_credit_balance( $user_id, $balance );

}


/**
 * Decreases the store credit balance of a user with the given amount.
 * 
 * @param int   $user_id
 * @param float $amount
 * 
 * @return bool
 * 
 */
function slicewp_decrease_user_store_credit_balance( $user_id, $amount ) {

    $balance = slicewp_get_user_store_credit_balance( $user_id ) - $amount;

    if ( $balance < 0 ) {
        $balance = 0;
    }

    return slicewp_update_user_store_credit_balance( $user_id, $balance );

}


/**
 * Returns the store credit balance of an affiliate.
 * 
 * @param int $affiliate_id
 * 
 * @return float
 * 
 */
function slicewp_get_affiliate_store_credit_balance( $affiliate_id ) {

    $affiliate = slicewp_get_affiliate( $affiliate_id );

    if ( is_null( $affiliate ) ) {
        return 0;
    }

    return slicewp_get_user_store_credit_balance( $affiliate->get( 'user_id' ) );

}


/**
 * Updates the store credit balance for the affiliate.
 * 
 * @param int   $affiliate_id
 * @param float $amount
 * 
 * @return bool
 * 
 */
function slicewp_update_affiliate_store_credit_balance( $affiliate_id, $amount ) {

    $affiliate = slicewp_get_affiliate( $affiliate_id );

    if ( is_null( $affiliate ) ) {
        return false;
    }

    return slicewp_update_user_store_credit_balance( $affiliate->get( 'user_id' ), $amount );

}


/**
 * Increases the store credit balance of an affiliate with the given amount.
 * 
 * @param int   $affiliate_id
 * @param float $amount
 * 
 * @return bool
 * 
 */
function slicewp_increase_affiliate_store_credit_balance( $affiliate_id, $amount ) {

    $balance = slicewp_get_affiliate_store_credit_balance( $affiliate_id ) + $amount;

    return slicewp_update_affiliate_store_credit_balance( $affiliate_id, $balance );

}


/**
 * Decreases the store credit balance of an affiliate with the given amount.
 * 
 * @param int   $affiliate_id
 * @param float $amount
 * 
 * @return bool
 * 
 */
function slicewp_decrease_affiliate_store_credit_balance( $affiliate_id, $amount ) {

    $balance = slicewp_get_affiliate_store_credit_balance( $affiliate_id ) - $amount;

    if ( $balance < 0 ) {
        $balance = 0;
    }

    return slicewp_update_affiliate_store_credit_balance( $affiliate_id, $balance );

}


/**
 * Checks to see whether within a payout the "store_credit" payout method can performs bulk payments.
 * 
 * @param bool 	 $can_do
 * @param int 	 $payout_id
 * @param string $payout_method
 * 
 * @return bool
 * 
 */
function slicewp_can_do_bulk_payments_payout_method_store_credit( $can_do, $payout_id, $payout_method ) {

	if ( 'store_credit' != $payout_method ) {
        return $can_do;
    }

	$payments = slicewp_get_payments( array( 'payout_id' => $payout_id, 'payout_method' => $payout_method, 'status' => 'unpaid' ), true );

	if ( ! empty( $payments ) ) {
        $can_do = true;
    }

	return $can_do;

}
add_filter( 'slicewp_can_do_bulk_payments', 'slicewp_can_do_bulk_payments_payout_method_store_credit', 10, 3 );


/**
 * Registers the store credit balance field on the affiliate account form
 * for affiliates that have the "store_credit" payout method selected.
 * 
 * @param array $fields
 * 
 */
function slicewp_register_affiliate_form_field_store_credit_balance( $fields ) {

    if ( is_admin() ) {
        return $fields;
    }

    $affiliate_id = slicewp_get_current_affiliate_id();
    $balance      = slicewp_get_affiliate_store_credit_balance( $affiliate_id );

    if ( empty( $balance ) && 'store_credit' != slicewp_get_affiliate_payout_method( $affiliate_id ) ) {
        return $fields;
    }

    $fields[] = array(
		'type'  	  		  => 'text',
		'id'				  => 'slicewp-affiliate-store-credit-balance',
		'label'				  => __( 'Store Credit Balance', 'slicewp' ),
		'is_disabled'		  => true,
        'value'               => slicewp_format_amount( $balance, slicewp_get_setting( 'active_currency', 'USD' ) ),
        'description'         => __( 'This is your store credit balance, that you can use towards purchases.', 'slicewp' ),
		'output_conditionals' => array( 'form' => array( 'affiliate_account' ) )
	);

    return $fields;

}
add_filter( 'slicewp_register_affiliate_fields', 'slicewp_register_affiliate_form_field_store_credit_balance', 9 );

/**
 * @todo check to see if we can create automatic store credit payments
 * 
 */