<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Class for the WooCommerce integration for affiliate store credits.
 *
 */
class SliceWP_Store_Credit_Integration_WooCommerce {

	/**
	 * The code of the store credit coupon.
	 * 
	 * @access protected
	 * @var    string
	 * 
	 */
	protected $coupon_code = 'slicewp-store-credit';


	/**
	 * Constructor.
	 *
	 */
	public function __construct() {

		/**
		 * Filter to customize the store credit coupon.
		 * 
		 * @param string $coupon_code
		 * 
		 */
		$this->coupon_code = apply_filters( 'slicewp_store_credit_coupon_code', $this->coupon_code );

		add_filter( 'the_content', array( $this, 'the_content_add_store_credit_balance_notice' ) );

		add_action( 'woocommerce_before_cart', array( $this, 'add_store_credit_balance_notice' ) );
		add_action( 'woocommerce_before_checkout_form', array( $this, 'add_store_credit_balance_notice' ) );

		add_filter( 'woocommerce_cart_totals_coupon_label', array( $this, 'coupon_label' ), 10, 2 );
		add_filter( 'woocommerce_get_shop_coupon_data', array( $this, 'get_coupon_data' ), 10, 2 );

		add_action( 'woocommerce_cart_loaded_from_session', array( $this, 'apply_store_credit_as_discount' ) );

		add_action( 'woocommerce_checkout_order_processed', array( $this, 'process_store_credit_coupon_code' ) );
		add_action( 'woocommerce_store_api_checkout_order_processed', array( $this, 'process_store_credit_coupon_code' ) );

	}


	/**
	 * Returns the currently logged-in affiliate.
	 * 
	 * @return null|SliceWP_Affiliate
	 * 
	 */
	protected function get_current_affiliate() {

		$affiliate = slicewp_get_affiliate( slicewp_get_current_affiliate_id() );

		if ( is_null( $affiliate ) || $affiliate->get( 'status' ) != 'active' ) {
			return null;
		}

		return $affiliate;

	}


	/**
	 * Changes the label for the coupon in the cart into something human friendly.
	 * 
	 * @param string 	$label
	 * @param WC_Coupon $coupon
	 * 
	 * @return string
	 * 
	 */
	public function coupon_label( $label, $coupon ) {

		if ( $this->coupon_code != $coupon->get_code() ) {
			return $label;
		}

		return __( 'Store Credit', 'slicewp' );

	}


	/**
	 * Returns the coupon data for the dynamic coupon.
	 * 
	 * @param bool   $false
	 * @param string $data
	 * 
	 * @return bool|array
	 * 
	 */
	public function get_coupon_data( $false, $data ) {

		if ( ! is_string( $data ) ) {
			return $false;
		}

		if ( $data != $this->coupon_code ) {
			return $false;
		}

		$affiliate = $this->get_current_affiliate();

		// Bail quickly if we don't have a valid affiliate.
		if ( is_null( $affiliate ) ) {
			return $false;
		}

		$store_credit_balance = slicewp_get_affiliate_store_credit_balance( $affiliate->get( 'id' ) );

		// Bail if the affiliate doesn't have any credit.
		if ( empty( $store_credit_balance ) ) {
			return $false;
		}

		// Make sure the cart is set.
		if ( empty( WC()->cart ) ) {
			return $false;
		}

		// Set cart subtotal.
		$cart_subtotal = (float)( 'excl' == WC()->cart->get_tax_price_display_mode() ) ? WC()->cart->subtotal_ex_tax : WC()->cart->subtotal;

		return array(
			'id' 	 => 999999,
			'code' 	 => $this->coupon_code,
			'amount' => ( $cart_subtotal > $store_credit_balance ? $store_credit_balance : $cart_subtotal )
		);

	}


	/**
	 * Returns the store credit coupon item from the given order.
	 * 
	 * @param WC_Order $order
	 * 
	 * @return null|WC_Order_Item_Coupon
	 * 
	 */
	protected function get_order_coupon_item( $order ) {

		foreach ( $order->get_items( 'coupon' ) as $key => $coupon ) {

			if ( $this->coupon_code == $coupon->get_code() ) {

				return $coupon;

			}

		}

		return null;

	}


	/**
	 * Prepends the store credit balance notice before the cart and checkout page's content when blocks are
	 * used instead of shortcodes.
	 * 
	 * This approach should be temporary. Ideally, we would hook into the cart and checkout blocks
	 * to have functionality natively integrated into these blocks' JS.
	 * 
	 */
	function the_content_add_store_credit_balance_notice( $content ) {

		if ( is_admin() ) {
			return $content;
		}
	
		if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			return $content;
		}
		
		if ( ! is_singular() ) {
			return $content;
		}

		if ( ! function_exists( 'is_cart' ) || ! function_exists( 'is_checkout' ) ) {
			return $content;
		}

		if ( ! is_cart() && ! is_checkout() ) {
			return $content;
		}

		if ( ! has_block( 'woocommerce/cart' ) && ! has_block( 'woocommerce/checkout' ) ) {
			return $content;
		}

		ob_start();

		$this->add_store_credit_balance_notice();
		$contents = ob_get_contents();

		ob_end_clean();

		if ( ! empty( slicewp()->globals() ) ) {

			slicewp()->globals()->set( 'store_credit_balance_notice_in_post_content', true );

		}

		return $contents . $content;

	}


	/**
	 * Print the store credit balance one the shop and checkout pages,
	 * if the affiliate is valid and has store credit.
	 *
	 */
	public function add_store_credit_balance_notice() {

		// Bail if the notice has already been displayed in the post's content.
		if ( ! empty( slicewp()->globals()->get( 'store_credit_balance_notice_in_post_content' ) ) ) {
			return;
		}

		// Bail if cart is empty.
		if ( WC()->cart->get_cart_contents_count() == 0 ) {
			return;
		}

		$affiliate = $this->get_current_affiliate();

		// Bail quickly if we don't have a valid affiliate.
		if ( is_null( $affiliate ) ) {
			return;
		}

		$store_credit_balance = slicewp_get_affiliate_store_credit_balance( $affiliate->get( 'id' ) );

		// Bail if the affiliate doesn't have any credit.
		if ( empty( $store_credit_balance ) ) {
			return;
		}

		// Add styling for the notice.
		echo '<style>body .woocommerce .button.slicewp-store-credit-apply-balance { float: none; margin-top: 10px; margin-left: 0; } </style>';

		// Bail if coupon code is already applied.
		if ( WC()->cart->has_discount( $this->coupon_code ) ) {
			return;
		}

		// Print the account balance notice
		wc_print_notice( sprintf( '%1$s <strong>%2$s</strong>. %3$s <br /><a href="%4$s" class="button slicewp-store-credit-apply-balance">%5$s</a>', 
			__( 'Your account balance is', 'slicewp' ),
			wc_price( $store_credit_balance ),
			__( 'Would you like to use it as a discount now?', 'slicewp' ),
			add_query_arg( array( 'slicewp_apply_store_credit' => '1' ) ),
			__( 'Yes, apply discount', 'slicewp' )
		), 'notice' );

	}


	/**
	 * Applies the store credit as a discount code to the cart.
	 *
	 */
	public function apply_store_credit_as_discount() {

		if ( empty( $_GET['slicewp_apply_store_credit'] ) || $_GET['slicewp_apply_store_credit'] != '1' ) {
			return;
		}

		// Bail if coupon code is already applied.
		if ( WC()->cart->has_discount( $this->coupon_code ) ) {
			return;
		}

		$affiliate = $this->get_current_affiliate();

		// Bail quickly if we don't have a valid affiliate.
		if ( is_null( $affiliate ) ) {
			return;
		}

		$store_credit_balance = slicewp_get_affiliate_store_credit_balance( $affiliate->get( 'id' ) );

		// Bail if the affiliate doesn't have any credit
		if ( empty( $store_credit_balance ) ) {
			return;
		}

		// Set cart subtotal.
		$cart_subtotal = (float)( 'excl' == WC()->cart->get_tax_price_display_mode() ) ? WC()->cart->subtotal_ex_tax : WC()->cart->subtotal;

		// Bail if the cart's subtotal is zero
		if ( empty( $cart_subtotal ) ) {
			return;
		}

		// Apply the coupon code.
		WC()->cart->apply_coupon( $this->coupon_code );

		wp_redirect( remove_query_arg( 'slicewp_apply_store_credit' ) );
		exit;

	}


	/**
	 * Processes any store credit coupons and adjust the store credit if needed.
	 *
	 * @param int|WC_Order $order
	 *
	 */
	public function process_store_credit_coupon_code( $order ) {

		// Get order.
		$order = wc_get_order( $order );

		// Bail if the order doesn't have a user ID attached.
		if ( empty( $order->get_user_id() ) ) {
			return;
		}

		// Get coupon codes
		$coupon_codes = $order->get_coupon_codes();

		// Bail if no codes were used.
		if ( empty( $coupon_codes ) ) {
			return;
		}

		// Bail if no affiliate store credit coupon was used.
		if ( ! in_array( $this->coupon_code, $coupon_codes ) ) {
			return;
		}

		// Get affiliate by order user ID.
		$affiliate = slicewp_get_affiliate_by_user_id( $order->get_user_id() );

		if ( is_null( $affiliate ) ) {
			return;
		}

		$coupon = $this->get_order_coupon_item( $order );

		if ( empty( $coupon->get_discount( 'edit' ) ) ) {
			return;
		}

		slicewp_decrease_affiliate_store_credit_balance( $affiliate->get( 'id' ), $coupon->get_discount( 'edit' ) + $coupon->get_discount_tax( 'edit' ) );

	}

}

new SliceWP_Store_Credit_Integration_WooCommerce();