jQuery( function($) {

	/**
	 * The default configuration object for our charts.
	 * The configuration object will be modified as needed before instantiating a new chart.
	 *
	 */
	var default_config = {

		// Chart type.
		type: 'line',

		// The data.
		data: {
			datasets: []
		},

		// Options.
		options: {

			// Aspect ratio.
			maintainAspectRatio: false,

			// Elements.
			elements: {

				// Point element.
                point: {
                    radius: 4,
                    borderWidth: 2,
                    hoverRadius: 5,
                    hoverBorderWidth: 3,
                    hitRadius: 11,

                    backgroundColor: 'rgba( 0, 170, 220, 1 )',
                    borderColor: 'rgba( 255, 255, 255, 1 )',
                    hoverBorderColor: 'rgba( 255, 255, 255, 1 )',

                },

                // Line element.
                line: {
                    borderWidth: 2,
                    borderColor: 'rgba( 0, 170, 220, 1 )',
                    borderJoinStyle: 'round'
                },

                // Bar element.
                bar: {
                    borderRadius: { topLeft: 2, topRight: 2 },
                    backgroundColor: 'rgba( 0, 170, 220, 1 )',
                    hoverBackgroundColor: 'rgba( 0, 170, 220, 1 )'
                }

            },

            // Layout.
            layout: {
                padding: { top: 0, bottom: 10, left: 0, right: -60 }
            },

            // Animation.
            animation: {
            	duration: 0
            },

            // Scales.
            scales: {

            	// Vertical axis on the left.
                yAxis_left: {
                    beginAtZero: true,
                    grid: {
                        color: '#e9e9e9',
                        drawBorder: false,
                        tickLength: 150,
                    },
                    ticks: {
                        mirror: true,
                        align: 'end',
                        padding: -45
                    }
                },

                // Vertical axis on the right (it has only an aesthetic role).
                yAxis_right: {
                    position: 'right',
                    beginAtZero: true,
                    grid: {
                        color: '#e9e9e9',
                        drawBorder: false,
                        tickLength: 50
                    },
                    ticks: {
                        align: 'end',
                        padding: 0,
                        mirror: true,
                        padding: 30
                    }
                },

                // Horizontal axis.
                x: {
                    type: 'time',
                    time: {
                        unit: 'day',
                        isoWeekday: true,
                        tooltipFormat: 'MMM d, Y'
                    },
                    grid: {
                        display: false,
                        drawBorder: false,
                        tickLength: 10
                    },
                    ticks: {
                        maxTicksLimit: 12,
                        maxRotation: 0,
                        source: 'data'
                    }
                }

            },

            // Interaction.
            interaction: {
                intersect: false,
                mode: 'index',
            },

            // Plugins.
            plugins: {

            	// Crosshair.
            	crosshair: {
                    line: {
                      color: 'rgba( 200, 215, 225, 1 )',
                      width: 1
                    },
                    snap: {
                        enabled: true
                    },
                    sync: {
                    	enabled: false
                    },
                    zoom: {
                        enabled: false
                    }
                },

                // Legend.
                legend: {
                    display: false
                },

                // Tooltip.
                tooltip: {
                    enabled: false,
                    mode: 'index',
                    intersect: false
                }

            }

		}

	}


    /**
     * The HTML legend plugin that replaces the default chart legend.
     *
     */
    var html_legend_plugin = {

        id: 'html_legend',

        afterUpdate( chart, args, options ) {

            // var ul = getOrCreateLegendList(chart, options.containerID);
            var legendContainer = chart.canvas.parentNode.querySelector( 'div.slicewp-chart-legend' );
            var listContainer   = legendContainer.querySelector( 'ul' );

            if ( ! listContainer ) {

                listContainer = document.createElement('ul');
                listContainer.style.display = 'flex';
                listContainer.style.flexDirection = 'row';
                listContainer.style.margin = 0;
                listContainer.style.padding = 0;

                legendContainer.appendChild( listContainer );

            }

            // Remove old legend items.
            while ( listContainer.firstChild ) {
                listContainer.firstChild.remove();
            }

            // Reuse the built-in legendItems generator.
            var items = chart.options.plugins.legend.labels.generateLabels( chart );

            items.forEach( item => {

                var li = document.createElement('li');

                li.style.alignItems = 'center';
                li.style.cursor = 'pointer';
                li.style.display = 'flex';
                li.style.flexDirection = 'row';
                li.style.marginLeft = '20px';

                li.onclick = function() {
                    
                    var type = chart.config.type;

                    if ( type === 'pie' || type === 'doughnut' ) {

                        // Pie and doughnut charts only have a single dataset and visibility is per item
                        chart.toggleDataVisibility( item.index );

                    } else {

                        chart.setDatasetVisibility( item.datasetIndex, ! chart.isDatasetVisible( item.datasetIndex ) );

                    }

                    chart.update();

                };

                // Color box.
                var boxSpan = document.createElement('span');

                boxSpan.style.background = item.fillStyle;
                boxSpan.style.borderColor = item.strokeStyle;
                boxSpan.style.borderWidth = item.lineWidth + 'px';
                boxSpan.style.display = 'inline-block';
                boxSpan.style.height = '16px';
                boxSpan.style.marginRight = '5px';
                boxSpan.style.width = '16px';

                // Text.
                var textContainer = document.createElement('p');

                textContainer.style.color = item.fontColor;
                textContainer.style.margin = 0;
                textContainer.style.padding = 0;
                textContainer.style.textDecoration = item.hidden ? 'line-through' : '';

                var text = document.createTextNode( item.text );
                textContainer.appendChild( text );

                li.appendChild( boxSpan );
                li.appendChild( textContainer );

                listContainer.appendChild( li );

            });

        }

    };


    /**
     * Handles the creation of the chart's tooltip.
     *
     */
    function chart_external_tooltip_handler( context ) {

        var chart   = context.chart;
        var tooltip = context.tooltip;

        // Tooltip element.
        var tooltipEl = chart.canvas.parentNode.querySelector( 'div.slicewp-chart-tooltip' );

        if ( ! tooltipEl ) {

            tooltipEl = document.createElement( 'div' );
            tooltipEl.classList.add( 'slicewp-chart-tooltip' );
            
            tooltipEl.style.background    = 'rgba( 255, 255, 255, 1 )';
            tooltipEl.style.borderRadius  = '3px';
            tooltipEl.style.color         = 'rgba( 79, 116, 142, 1 )';
            tooltipEl.style.border        = '1px solid rgba( 200, 215, 225, 1 )';
            tooltipEl.style.boxShadow     = '0 2px 5px rgba( 0, 0, 0, 0.1 ), 0 0 56px rgba( 0, 0, 0, 0.08 )';
            tooltipEl.style.opacity       = 1;
            tooltipEl.style.pointerEvents = 'none';
            tooltipEl.style.position      = 'absolute';
            tooltipEl.style.transform     = 'translate(-50%, 0)';

            var table = document.createElement( 'table' );
            table.style.margin = '0px';
            tooltipEl.appendChild(table);
            
            chart.canvas.parentNode.appendChild( tooltipEl );

        }

        // Hide if no tooltip.
        if ( tooltip.opacity === 0 ) {

            tooltipEl.style.opacity = 0;
            return;

        }

        // Set text.
        if ( tooltip.body ) {

            var titleLines = tooltip.title || [];
            var bodyLines  = tooltip.body.map( b => b.lines );

            var tableHead = document.createElement( 'thead' );

            titleLines.forEach( title => {

                var tr = document.createElement( 'tr' );
                tr.style.borderWidth = 0;

                var th = document.createElement( 'th' );
                th.style.borderWidth = 0;
                var text = document.createTextNode( title );

                th.appendChild(text);
                tr.appendChild(th);
                tableHead.appendChild(tr);

            });

            var tableBody = document.createElement( 'tbody' );

            bodyLines.forEach( ( body, i ) => {

                var colors = tooltip.labelColors[i];

                var span = document.createElement('span');
                span.style.background = colors.backgroundColor;
                span.style.borderColor = colors.borderColor;
                span.style.borderWidth = '2px';
                span.style.marginRight = '10px';
                span.style.height = '10px';
                span.style.width = '10px';
                span.style.display = 'inline-block';

                var tr = document.createElement( 'tr' );
                tr.style.backgroundColor = 'inherit';
                tr.style.borderWidth = 0;

                var td = document.createElement( 'td' );
                td.style.borderWidth = 0;

                var text = document.createTextNode( body );

                td.appendChild( span );
                td.appendChild( text );
                tr.appendChild( td );
                tableBody.appendChild( tr );

            });

            var tableRoot = tooltipEl.querySelector( 'table' );

            // Remove old children.
            while ( tableRoot.firstChild ) {
                tableRoot.firstChild.remove();
            }

            // Add new children.
            tableRoot.appendChild( tableHead );
            tableRoot.appendChild( tableBody );

        }

        // Display, position, and set styles for font.
        var positionX = chart.canvas.offsetLeft;
        var positionY = chart.canvas.offsetTop;

        var tooltipXOffset = ( tooltip.caretX < chart.width / 2 ? ( tooltipEl.offsetWidth / 2 + 20 ) : -( tooltipEl.offsetWidth / 2 + 20 ) );

        tooltipEl.style.opacity = 1;
        tooltipEl.style.left    = positionX + tooltip.caretX + tooltipXOffset + 'px';
        tooltipEl.style.top     = positionY + 30 + 'px';
        tooltipEl.style.font    = tooltip.options.bodyFont.string;
        tooltipEl.style.padding = tooltip.options.padding + 'px ' + tooltip.options.padding + 'px';

    }


    /**
     * Handles the tooltip's label.
     *
     */
    function chart_external_tooltip_label_handler( context ) {

        var label = context.dataset.label || '';

        if ( context.dataset.external_data.is_amount ) {

            label += ': ' + new Intl.NumberFormat( 'en-US', { style: 'currency', currency: slicewp.settings.active_currency } ).format( context.parsed.y );

        } else {

            label += ': ' + context.parsed.y;

        }

        return label;

    }


    /**
     * Handles the values displayed for the Y axis scale ticks.
     *
     */
    function chart_tick_label_handler( value, index, values ) {

        var chart = this.chart;

        if ( chart.config.data.datasets[0].external_data.is_amount ) {

            value = new Intl.NumberFormat( 'en-US', { style: 'currency', currency: slicewp.settings.active_currency } ).format( value );

        }

        return value;

    }


    /**
     * Returns the chart's default config object.
     *
     */
    function get_chart_default_config() {

        var config = JSON.parse( JSON.stringify( default_config ) );

        config.options.plugins.tooltip.external         = chart_external_tooltip_handler;
        config.options.plugins.tooltip.callbacks        = { label: chart_external_tooltip_label_handler };
        config.options.scales.yAxis_left.ticks.callback = chart_tick_label_handler;

        config.plugins = [html_legend_plugin];

        return config;

    }


	/**
	 * Instantiate each chart.
	 *
	 */
    var charts = [];
    
    $('.slicewp-card-chart').each( function() {

        // Set the id.
        var id = $(this).attr( 'id' ).replace( 'slicewp-card-chart-', '' );

        // Set dataset.
        var dataset = maybe_parse_json( $(this).closest( '.slicewp-card-chart' ).find( '.slicewp-chart-filter-dataset' ).val() );

        // Set the time unit.
        var time_unit = $(this).closest( '.slicewp-card-chart' ).find( '.slicewp-chart-filter-time-unit' ).val();

        // Instantiate the chart.
        charts[id] = {
            type      : 'line',
            time_unit : time_unit,
            dataset   : ( dataset ? dataset : 'default' )
        };

        rebuild_chart( id );

    });


    /**
     * Change the chart type on chart type switcher button click.
     *
     */
    $(document).on( 'click', '.slicewp-field-wrapper-chart-type a', function(e) {

    	e.preventDefault();

    	// Get chart id and type.
    	var chart_id   = $(this).closest( '.slicewp-card-chart' ).attr( 'id' ).replace( 'slicewp-card-chart-', '' );
    	var chart_type = $(this).attr( 'data-chart-type' );

    	// Bail, if the chart type is the same.
    	if ( charts[chart_id].chart.config.type == chart_type ) {
            return false;
        }

        // Activate the clicked option.
        $(this).parent().find('a').removeClass( 'slicewp-active' );
        $(this).addClass( 'slicewp-active' );

        // Update the chart's type.
        charts[chart_id].type = $(this).attr( 'data-chart-type' );

        // Rebuild the chart
        rebuild_chart( chart_id );

    });


	/**
	 * Disables the time unit options based on the charts' data.
	 *
	 */
	$('.slicewp-card-chart .slicewp-chart-filter-time-unit').each( function() {

        // Get chart id.
        var chart_id = $(this).closest( '.slicewp-card-chart' ).attr( 'id' ).replace( 'slicewp-card-chart-', '' );

        // Get datasets.
        var datasets = get_canvas_datasets( chart_id, charts[chart_id].dataset );

        if ( datasets ) {

            var dataset = datasets[Object.keys( datasets )[0]];

            // Disable all options.
            $(this).find( 'option' ).attr( 'disabled', true );

            for ( var index in dataset.data ) {

                $(this).find( 'option[value="' + index + '"]' ).attr( 'disabled', false );

            }

        }

	});


	/**
	 * Update chart data based on selected time unit.
	 *
	 */
	$(document).on( 'change', '.slicewp-card-chart .slicewp-chart-filter-time-unit', function() {

		// Get chart id.
    	var chart_id   = $(this).closest( '.slicewp-card-chart' ).attr( 'id' ).replace( 'slicewp-card-chart-', '' );
        
        // Update the chart's time unit.
        charts[chart_id].time_unit = $(this).val();

        // Rebuild the chart
        rebuild_chart( chart_id );

	});


	/**
	 * Update the chart based on the selected dataset.
	 *
	 */
	$(document).on( 'change', '.slicewp-chart-filter-dataset', function() {

		// Get chart id.
        var chart_id = $(this).closest( '.slicewp-card-chart' ).attr( 'id' ).replace( 'slicewp-card-chart-', '' );

        // Update the chart's dataset.
        charts[chart_id].dataset = maybe_parse_json( $(this).val() != '' ? $(this).val() : 'default' );

        // Rebuild the chart
        rebuild_chart( chart_id );

	});


    /**
     * Returns the scale step size, based on the given maximum value of the dataset.
     *
     * To make sure the tooltip has room to display properly, we're adding another 25% to the data max
     * and calculate the step size based on the new higher value.
     *
     */
    function round_scale_step( data_max ) {

        // If there's no data in the graph, return a step size of 1.
        if ( data_max == 0 ) {
            return 1;
        }

    	var round_data_max = Math.ceil( data_max );
    	var round_unit 	   = ( data_max > 10 ? 5 : 1 );

    	return Math.ceil( ( round_data_max / 4 ) / round_unit ) * round_unit;

    }


    /**
     * Returns an array of dataset objects for the given chart.
     *
     * @param string       chart_id
     * @param string|array dataset_slug
     *
     * @return array|null
     *
     */
    function get_canvas_datasets( chart_id, dataset_slug ) {

        var $canvas = $( '#slicewp-card-chart-' + chart_id + ' canvas' ).first();

        if ( typeof $canvas == 'undefined' ) {
            return null;
        }

        if ( typeof $canvas.attr( 'data-datasets' ) == 'undefined' ) {
            return null;
        }

        // Turn the dataset slug into an array of slugs.
        var dataset_slugs = ( Array.isArray( dataset_slug ) ? dataset_slug : [dataset_slug] );

        // Get the datasets data attribute value of the canvas.
        var canvas_datasets = JSON.parse( $canvas.attr( 'data-datasets' ) );

        var datasets = [];

        for ( var index in dataset_slugs ) {

            if ( typeof canvas_datasets[dataset_slugs[index]] != 'undefined' ) {

                datasets[dataset_slugs[index]] = canvas_datasets[dataset_slugs[index]];

            }

        }

        return datasets;

    }


    /**
     * Returns the scale's max and step size based on the given config data.
     *
     */
    function get_chart_scale_data( config ) {

        var scale_data = {
            yAxis_left  : { max : 4, step_size : 1 },
            yAxis_right : { max : null, step_size : null }
        };

        // For bar charts that have a single Y axis we need the datasets' max values sum.
        if ( config.type == 'bar' && has_chart_single_y_axis( config ) ) {

            var arr = [];

            var scale_step = 0;
            var scale_max  = 0;

            for ( var index in config.data.datasets ) {

                for ( var date in config.data.datasets[index].data ) {

                    if ( typeof arr[date] == 'undefined' )
                        arr[date] = 0;

                    arr[date] += config.data.datasets[index].data[date];

                }

            }

            scale_step = round_scale_step( get_array_max( arr ) );
            scale_max  = scale_step * 4;

            scale_data[config.data.datasets[0].yAxisID].step_size = scale_step;
            scale_data[config.data.datasets[0].yAxisID].max       = scale_max;


        // For all other charts we need the max value of the datasets.
        } else {

            for ( var index in config.data.datasets ) {

                var scale_step = round_scale_step( get_array_max( config.data.datasets[index].data ) );
                var scale_max  = scale_step * 4;

                if ( scale_max > scale_data[config.data.datasets[index].yAxisID].max ) {

                    scale_data[config.data.datasets[index].yAxisID].step_size = scale_step;
                    scale_data[config.data.datasets[index].yAxisID].max       = scale_max;

                }

            }

        }

        // If the right axis doesn't have values, add the ones from the left axis.
        if ( scale_data.yAxis_right.max === null && scale_data.yAxis_right.step_size === null ) {

            scale_data.yAxis_right.max       = scale_data.yAxis_left.max;
            scale_data.yAxis_right.step_size = scale_data.yAxis_left.step_size;

        }

        return scale_data;

    }


    /**
     * Checks whether the chart has only one Y axis or not.
     *
     */
    function has_chart_single_y_axis( config ) {

        var y_axes = [];

        for ( var index in config.data.datasets ) {

            if ( ! y_axes.includes( config.data.datasets[index].yAxisID ) )
                y_axes.push( config.data.datasets[index].yAxisID );

        }

        return ( y_axes.length == 1 ? true : false );

    }


    /**
     * Rebuilds the given chart.
     *
     * @param string chart_id
     *
     */
    function rebuild_chart( chart_id ) {
            
        // Set canvas element.
        var ctx = $( '#slicewp-card-chart-' + chart_id + ' canvas' ).first();

        // Set the default config data.
        var config = get_chart_default_config();

        // Set the chart type for the config.
        config.type = charts[chart_id].type;

        // Set the offset for the config.
        if ( charts[chart_id].type == 'line' ) {
            config.options.scales.x.offset = false;
        } else {
            config.options.scales.x.offset = true;
        }

        // Set the time unit for the config.
        config.options.scales.x.time.unit = charts[chart_id].time_unit;

        // Set the data for the dataset.
        var datasets = get_canvas_datasets( chart_id, charts[chart_id].dataset );

        // If we have datasets, prepare the data
        if ( datasets ) {

            for ( var index in datasets ) {

                // Prepare the dataset
                var dataset_config = {

                    // Set the external dataset data.
                    external_data : datasets[index],
                    
                    // Set the label and data.
                    label : datasets[index].label,
                    data  : datasets[index].data[charts[chart_id].time_unit],

                    // Set the axis ID.
                    yAxisID : ( typeof datasets[index].axis_y == 'undefined' || datasets[index].axis_y != 'right' ? 'yAxis_left' : 'yAxis_right' ),

                    // Set the fill.
                    fill : false,

                    // Set the line chart styling.
                    pointBorderColor     : 'rgba( 255, 255, 255, 1 )',
                    pointBackgroundColor : hex_to_rgba( datasets[index].color ),
                    borderColor          : hex_to_rgba( datasets[index].color ),

                    // Set the bar chart styling.
                    backgroundColor      : hex_to_rgba( datasets[index].color ),
                    hoverBackgroundColor : hex_to_rgba( datasets[index].color )

                };

                // Set the fill attribute if there are less than 2 datasets.
                if ( Object.keys( datasets ).length <= 2 ) {

                    dataset_config.fill = {
                        target : 'origin',
                        above  : hex_to_rgba( datasets[index].color , 0.1 )
                    };

                }

                // Add the dataset.
                config.data.datasets.push( dataset_config );

            }

            // Set the point config options to zero, when datasets contain more than 53 items.
            if ( Object.keys( config.data.datasets[0].data ).length > 53 ) {

                config.options.elements.point.radius           = 0;
                config.options.elements.point.borderWidth      = 0;
                config.options.elements.point.hoverRadius      = 0;
                config.options.elements.point.hoverBorderWidth = 0;
                config.options.elements.point.hitRadius        = 1;

            }

            // Determine if the bar chart should be stacked.
            if ( charts[chart_id].type == 'bar' ) {

                if ( has_chart_single_y_axis( config ) ) {

                    config.options.scales.x.stacked           = true;
                    config.options.scales.yAxis_left.stacked  = true;
                    config.options.scales.yAxis_right.stacked = true;

                }

            }

            // Set the correct ticks values for max and step size.
            var scale_data = get_chart_scale_data( config );

            // Hide the right axis values if there are no custom values for this axis.
            if ( has_chart_single_y_axis( config ) ) {

                config.options.scales.yAxis_right.ticks.color = '#fff';

            }

            // Set the scale values.
            config.options.scales.yAxis_left.max  = scale_data.yAxis_left.max;
            config.options.scales.yAxis_right.max = scale_data.yAxis_right.max;

            config.options.scales.yAxis_left.ticks.stepSize  = scale_data.yAxis_left.step_size;
            config.options.scales.yAxis_right.ticks.stepSize = scale_data.yAxis_right.step_size;

            // Show/hide the legend based on the number of datasets.
            if ( Object.keys( datasets ).length == 1 ) {

                ctx.siblings('.slicewp-chart-legend').hide();
                config.options.layout.padding.top = 25;

            } else {

                ctx.siblings('.slicewp-chart-legend').show();
                config.options.layout.padding.top = 0;

            }

        }

        // Destroy and rebuild the chart.
        if ( typeof charts[chart_id].chart != 'undefined' ) {
            charts[chart_id].chart.destroy();
        }

        charts[chart_id].chart = new Chart( ctx, config );

    }


    /**
     * Chart Tab Navigation
     *
     */
    $('.slicewp-chart-nav-tab').on( 'click', function(e) {

        if ( typeof $(this).attr('data-tab')  != 'undefined' ) {

            e.preventDefault();

            // Nav Tab activation
            $('.slicewp-chart-nav-tab').removeClass('slicewp-active');
            $(this).addClass('slicewp-active');

            // Show tab
            $('.slicewp-chart-tab').removeClass('slicewp-active');

            var nav_tab = $(this).attr('data-tab');
            $('.slicewp-chart-tab[data-tab="' + nav_tab + '"]').addClass('slicewp-active');

        }

    });


    /**
     * Returns the rgba value of the given hex color
     *
     */
    function hex_to_rgba( hex, opacity = 1 ) {

        var r = parseInt( hex.slice(1, 3), 16 ),
            g = parseInt( hex.slice(3, 5), 16 ),
            b = parseInt( hex.slice(5, 7), 16 );

        return "rgba( " + r + "," + g + "," + b + ", " + opacity + " )";

    }


    /**
     * If the given value is JSON string, it attempts to parse it. If the value
     * is not a JSON string, it just returns it.
     *
     */
    function maybe_parse_json( value ) {

        if( ! ( value && typeof value === "string") ) {
            return value;
        }

        try {

            var json = JSON.parse( value );
            return json;

        } catch( error ) {

            return value;

        }

    }


    /**
     * Returns the highest value in the given array.
     *
     */
    function get_array_max( arr ) {

        var max = 0;

        for ( var index in arr ) {

            if ( arr[index] > max )
                max = arr[index];

        }

        return max;

    }

});