<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Registers the "group" collection type for the "affiliate" object context.
 * 
 * @param array $types
 * 
 * @return array
 * 
 */
function slicewp_register_collection_types_affiliate_group( $types ) {

	$types[] = array(
		'slug'                 => 'group',
		'object_context'       => 'affiliate',
		'object_relationships' => 'single'
	);

	return $types;

}
add_filter( 'slicewp_register_collection_types', 'slicewp_register_collection_types_affiliate_group' );


/**
 * Returns the affiliate group (collection) attached to the given affiliate.
 * 
 * @param int|SliceWP_Affiliate
 * 
 * @return null|SliceWP_Collection
 * 
 */
if ( ! function_exists( 'slicewp_get_affiliate_group' ) ) {

	function slicewp_get_affiliate_group( $affiliate ) {

		if ( empty( $affiliate ) ) {
			return null;
		}

		// Set affiliate ID.
		$affiliate_id = ( is_object( $affiliate ) ? $affiliate->get( 'id' ) : $affiliate );

		// Get all "group" collections attached to the affiliate.
		$collections = slicewp_get_object_collections( $affiliate_id, 'affiliate', 'group' );

		return ( ! empty( $collections ) ? $collections[0] : null );

	}

}


/**
 * Returns the affiliate group (collection) ID attached to the given affiliate.
 * 
 * @param int|SliceWP_Affiliate
 * 
 * @return int
 * 
 */
if ( ! function_exists( 'slicewp_get_affiliate_group_id' ) ) {

	function slicewp_get_affiliate_group_id( $affiliate ) {

		$collection = slicewp_get_affiliate_group( $affiliate );

		return ( ! is_null( $collection ) ? $collection->get( 'id' ) : 0 );

	}

}


/**
 * Adds the default affiliate group to the affiliate when inserting the affiliate into the DB.
 * 
 * @param int $affiliate_id
 * 
 */
function slicewp_insert_affiliate_set_default_affiliate_group( $affiliate_id ) {

	if ( slicewp_verify_request_action( 'add_affiliate' ) || slicewp_verify_request_action( 'update_affiliate' ) ) {
        return;
    }

	$default_group_id = absint( slicewp_get_setting( 'default_affiliate_group', 0 ) );

	if ( empty( $default_group_id ) ) {
		return;
	}

	// Make sure the collection exists and is valid.
	$collection = slicewp_get_collection( $default_group_id );

	if ( is_null( $collection ) || $collection->get( 'object_context' ) != 'affiliate' || $collection->get( 'type' ) != 'group' ) {
		return;
	}

	// Set affiliate group if all is good by now.
	slicewp_set_object_collections( $affiliate_id, 'affiliate', 'group', $default_group_id );

}
add_action( 'slicewp_insert_affiliate', 'slicewp_insert_affiliate_set_default_affiliate_group' );


/**
 * Modifies the commission amount based on the group commission rates the affiliate is attached to.
 *
 * @param float $commission_amount
 * @param float $amount
 * @param array $args
 *
 * @return float
 *
 */
function slicewp_calculate_commission_amount_affiliate_group_commission_rates( $commission_amount, $amount, $args ) {

	if ( empty( $args['type'] ) || $args['type'] == 'recurring' ) {
        return $commission_amount;
    }

	if ( empty( $args['affiliate_id'] ) ) {
        return $commission_amount;
    }

	$affiliate_id = absint( $args['affiliate_id'] );

	// Check if the affiliate is in a group.
	$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

	if ( empty( $affiliate_group_id ) ) {
		return $commission_amount;
	}

	$rate 	   = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_' . $args['type'], true );
	$rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_' . $args['type'], true );

	if ( ! is_numeric( $rate ) || empty( $rate_type ) ) {
        return $commission_amount;
    }

	$commission_amount = ( $rate_type == 'percentage' ? round( ( $amount * $rate / 100 ), 2 ) : $rate );

	return $commission_amount;

}
add_filter( 'slicewp_calculate_commission_amount', 'slicewp_calculate_commission_amount_affiliate_group_commission_rates', 15, 3 );


/**
 * Checks to see if the commissions are set to be calculated on a per order basis for the affiliate group.
 *
 * @param bool $return
 *
 * @return bool
 *
 */
function slicewp_is_commission_basis_per_order_from_per_affiliate_group( $return ) {

	// Get and check to see if referrer exists.
	$affiliate_id = slicewp_get_referrer_affiliate_id();

	if ( empty( $affiliate_id ) ) {
        return $return;
    }

	$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

	if ( empty( $affiliate_group_id ) ) {
		return $return;
	}

	// If the affiliate group doesn't have a commission rate set, return.
	$commission_rate_sale = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_sale', true );

	if ( empty( $commission_rate_sale ) ) {
        return $return;
    }

	if ( 'fixed_amount' == $commission_rate_sale ) {

		if ( 'order' == slicewp_get_collection_meta( $affiliate_group_id, '_commission_fixed_amount_rate_basis', true ) ) {
            $return = true;
        } else {
            $return = false;
        }

	}

	if ( 'percentage' == $commission_rate_sale ) {
        $return = false;
    }

	return $return;

}
add_filter( 'slicewp_is_commission_basis_per_order', 'slicewp_is_commission_basis_per_order_from_per_affiliate_group', 25 );


/**
 * Updates the commission rates displayed in the affiliate account from the values set for the affiliate group.
 *
 * @param array $affiliate_commission_rates
 * @param int	$affiliate_id
 * 
 * @return array $affiliate_commission_rates
 * 
 */
function slicewp_affiliate_commission_rates_from_per_affiliate_group( $affiliate_commission_rates, $affiliate_id ) {

	$affiliate_group_id = slicewp_get_affiliate_group_id( $affiliate_id );

	if ( empty( $affiliate_group_id ) ) {
		return $affiliate_commission_rates;
	}

	// If the affiliate doesn't have a commission rate set exit.
	foreach ( $affiliate_commission_rates as $type => $details ) {

		if ( $type == 'recurring' || $type == 'lifetime_sale' ) {
            continue;
        }

		$rate 	   = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_' . $type , true );
		$rate_type = slicewp_get_collection_meta( $affiliate_group_id, '_commission_rate_type_' . $type, true );

		if ( is_numeric( $rate ) && ! empty( $rate_type ) ){

			$affiliate_commission_rates[$type]['rate'] 		= $rate; 
			$affiliate_commission_rates[$type]['rate_type'] = $rate_type; 
		
		}

	}

	// Return the affiliate commission rates.
	return $affiliate_commission_rates;

}
add_filter( 'slicewp_affiliate_commission_rates', 'slicewp_affiliate_commission_rates_from_per_affiliate_group', 25, 2 );


/**
 * When deleting an affiliate group, if it's the default one, remove it from the settings as well so that
 * new affiliates aren't added to an inexistent group.
 * 
 * @param int $affiliate_group_id
 * 
 */
function slicewp_delete_affiliate_group_clear_default( $affiliate_group_id ) {

	$settings = slicewp_get_option( 'settings', array() );

    if ( empty( $settings['default_affiliate_group'] ) ) {
		return;
	}

	if ( absint( $affiliate_group_id ) != absint( $settings['default_affiliate_group'] ) ) {
		return;
	}

	$settings['default_affiliate_group'] = '';

    slicewp_update_option( 'settings', $settings );

}
add_action( 'slicewp_delete_collection', 'slicewp_delete_affiliate_group_clear_default' );


/**
 * Registers the "affiliate_group" column in the affiliates CSV export file.
 * 
 * @param array $cols
 * 
 * @return array
 * 
 */
function slicewp_export_csv_cols_affiliates_affiliate_group( $cols ) {

	$affiliate_groups_count = slicewp_get_collections( array( 'object_context' => 'affiliate', 'type' => 'group' ), true );

	if ( empty( $affiliate_groups_count ) ) {
		return $cols;
	}

	$cols['affiliate_group'] = __( 'Group', 'slicewp' );

	return $cols;

}
add_filter( 'slicewp_export_csv_cols_affiliates', 'slicewp_export_csv_cols_affiliates_affiliate_group', 15 );


/**
 * Adds the value for the "affiliate_group" column in the affiliates CSV export item data.
 *
 * @param array 			$item_data
 * @param SliceWP_Affiliate $affiliate
 *
 * @return array
 *
 */
function slicewp_export_get_data_item_affiliates_affiliate_group( $item_data, $affiliate ) {

	$affiliate_group = slicewp_get_affiliate_group( $affiliate );

	$item_data['affiliate_group'] = ( ! empty( $affiliate_group ) ? $affiliate_group->get( 'name' ) : '' );

	return $item_data;

}
add_filter( 'slicewp_export_get_data_item_affiliates', 'slicewp_export_get_data_item_affiliates_affiliate_group', 10, 2 );